@@ -70,6 +70,7 @@
 #define RIG_MODEL_FLRIG RIG_MAKE_MODEL(RIG_DUMMY, 4)
 #define RIG_MODEL_TRXMANAGER_RIG RIG_MAKE_MODEL(RIG_DUMMY, 5)
 #define RIG_MODEL_DUMMY_NOVFO RIG_MAKE_MODEL(RIG_DUMMY, 6)
+#define RIG_MODEL_TCI1X RIG_MAKE_MODEL(RIG_DUMMY, 7)
 
 
 /*
@@ -121,6 +122,11 @@
 #define RIG_MODEL_FTDX10 RIG_MAKE_MODEL(RIG_YAESU, 42)
 #define RIG_MODEL_FT897D RIG_MAKE_MODEL(RIG_YAESU, 43)
 #define RIG_MODEL_FTDX101MP RIG_MAKE_MODEL(RIG_YAESU, 44)
+#define RIG_MODEL_MCHFQRP RIG_MAKE_MODEL(RIG_YAESU, 45)
+#define RIG_MODEL_FT450D RIG_MAKE_MODEL(RIG_YAESU, 46)
+#define RIG_MODEL_FT650 RIG_MAKE_MODEL(RIG_YAESU, 47)
+#define RIG_MODEL_FT990UNI RIG_MAKE_MODEL(RIG_YAESU, 48)
+#define RIG_MODEL_FT710 RIG_MAKE_MODEL(RIG_YAESU, 49)
 
 
 /*
@@ -177,6 +183,8 @@
 #define RIG_MODEL_K4 RIG_MAKE_MODEL(RIG_KENWOOD, 47)
 #define RIG_MODEL_POWERSDR RIG_MAKE_MODEL(RIG_KENWOOD, 48)
 #define RIG_MODEL_MALACHITE RIG_MAKE_MODEL(RIG_KENWOOD, 49)
+#define RIG_MODEL_LAB599_TX500 RIG_MAKE_MODEL(RIG_KENWOOD,50)
+#define RIG_MODEL_SDRUNO RIG_MAKE_MODEL(RIG_KENWOOD,51)
 
 /*
  * Icom
@@ -260,7 +268,10 @@
 #define RIG_MODEL_ID31 RIG_MAKE_MODEL(RIG_ICOM, 83)
 #define RIG_MODEL_ID51 RIG_MAKE_MODEL(RIG_ICOM, 84)
 #define RIG_MODEL_IC705 RIG_MAKE_MODEL(RIG_ICOM, 85)
-/* next one is 86 */
+#define RIG_MODEL_ICF8101 RIG_MAKE_MODEL(RIG_ICOM, 86)
+#define RIG_MODEL_X6100 RIG_MAKE_MODEL(RIG_ICOM, 87)    /* Xiegu X6100 */
+#define RIG_MODEL_G90 RIG_MAKE_MODEL(RIG_ICOM, 88)    /* Xiegu G90 */
+#define RIG_MODEL_X5105 RIG_MAKE_MODEL(RIG_ICOM, 89)    /* Xiegu X5105 -- G90 compatible */
 
 
 /*
@@ -575,6 +586,7 @@
 #define RIG_MODEL_ESMC RIG_MAKE_MODEL(RIG_RS, 1)
 #define RIG_MODEL_EB200 RIG_MAKE_MODEL(RIG_RS, 2)
 #define RIG_MODEL_XK2100 RIG_MAKE_MODEL(RIG_RS, 3)
+#define RIG_MODEL_EK89X RIG_MAKE_MODEL(RIG_RS, 4)
 
 
 /*
@@ -622,6 +634,7 @@
 #define RIG_BACKEND_BARRETT "barrett"
 #define RIG_MODEL_BARRETT_2050 RIG_MAKE_MODEL(RIG_BARRETT, 1)
 #define RIG_MODEL_BARRETT_950 RIG_MAKE_MODEL(RIG_BARRETT, 2)
+#define RIG_MODEL_BARRETT_4050 RIG_MAKE_MODEL(RIG_BARRETT, 3)
 
 /*
  * Elad
@@ -629,8 +642,22 @@
 #define RIG_ELAD 33
 #define RIG_BACKEND_ELAD "elad"
 #define RIG_MODEL_ELAD_FDM_DUO RIG_MAKE_MODEL(RIG_ELAD, 1)
-//! @endcond
 
+/*
+ * CODAN
+ */
+#define RIG_CODAN 34
+#define RIG_BACKEND_CODAN "codan"
+#define RIG_MODEL_CODAN_ENVOY RIG_MAKE_MODEL(RIG_CODAN, 1)
+#define RIG_MODEL_CODAN_NGT RIG_MAKE_MODEL(RIG_CODAN, 2)
+
+/*
+ * Gomspace
+ */
+#define RIG_GOMSPACE 35
+#define RIG_BACKEND_GOMSPACE "gomspace"
+#define RIG_MODEL_GS100 RIG_MAKE_MODEL(RIG_GOMSPACE, 1)
+//! @endcond
 
 /*
  * TODO:
