@@ -19,13 +19,10 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdlib.h>
 #include <string.h>  /* String function definitions */
-#include <unistd.h>  /* UNIX standard function definitions */
 
 #include <hamlib/rig.h>
 #include <serial.h>
@@ -85,7 +82,7 @@ const struct rig_caps dxsr8_caps =
     .mfg_name =         "Alinco",
     .version =          BACKEND_VER ".0",
     .copyright =        "LGPL",
-    .status =           RIG_STATUS_BETA,
+    .status =           RIG_STATUS_STABLE,
     .rig_type =         RIG_TYPE_TRANSCEIVER,
     .ptt_type =         RIG_PTT_RIG,
     .dcd_type =         RIG_DCD_RIG,
@@ -216,7 +213,7 @@ const struct rig_caps dxsr8_caps =
     .get_func =         dxsr8_get_func, // AL~RR_AGC, AL~RR_NZB
     .set_level =        dxsr8_set_level, // AL~RW_RFG00, AL~RW_PWR00
     .get_level =        dxsr8_get_level, // AL~RR_RFG, AL~RR_PWR
-
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 /*
@@ -253,7 +250,7 @@ int dxsr8_transaction(RIG *rig,
 
     rig_flush(&rs->rigport);
 
-    retval = write_block(&rs->rigport, cmd, cmd_len);
+    retval = write_block(&rs->rigport, (unsigned char *) cmd, cmd_len);
 
     if (retval != RIG_OK)
     {
@@ -264,7 +261,8 @@ int dxsr8_transaction(RIG *rig,
      * Transceiver sends an echo of cmd followed by a CR/LF
      * TODO: check whether cmd and echobuf match (optional)
      */
-    retval = read_string(&rs->rigport, replybuf, BUFSZ, LF, strlen(LF));
+    retval = read_string(&rs->rigport, (unsigned char *) replybuf, BUFSZ,
+                         LF, strlen(LF), 0, 1);
 
     if (retval < 0)
     {
@@ -272,7 +270,8 @@ int dxsr8_transaction(RIG *rig,
     }
 
 
-    retval = read_string(&rs->rigport, replybuf, BUFSZ, LF, strlen(LF));
+    retval = read_string(&rs->rigport, (unsigned char *) replybuf, BUFSZ,
+                         LF, strlen(LF), 0, 1);
 
     if (retval < 0)
     {
@@ -338,7 +337,6 @@ int dxsr8_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 {
 
     char cmd[BUFSZ];
-    int cmd_len;
 
     /* max 10 digits */
     if (freq >= GHz(10))
@@ -347,8 +345,8 @@ int dxsr8_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
     }
 
     // cppcheck-suppress *
-    cmd_len = sprintf(cmd, AL "~RW_RXF%08"PRIll EOM, (int64_t)freq);
-    return dxsr8_transaction(rig, cmd, cmd_len, NULL, NULL);
+    SNPRINTF(cmd, sizeof(cmd), AL "~RW_RXF%08"PRIll EOM, (int64_t)freq);
+    return dxsr8_transaction(rig, cmd, strlen(cmd), NULL, NULL);
 }
 
 /*
@@ -382,7 +380,7 @@ int dxsr8_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 int dxsr8_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 {
     char mdbuf[BUFSZ];
-    int mdbuf_len, wide_filter, retval;
+    int wide_filter, retval;
     int amode;
 
     switch (mode)
@@ -406,8 +404,8 @@ int dxsr8_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
         return -RIG_EINVAL;
     }
 
-    mdbuf_len = sprintf(mdbuf, AL "~RW_RFM%02d" EOM, amode);
-    retval = dxsr8_transaction(rig, mdbuf, mdbuf_len, NULL, NULL);
+    SNPRINTF(mdbuf, sizeof(mdbuf), AL "~RW_RFM%02d" EOM, amode);
+    retval = dxsr8_transaction(rig, mdbuf, strlen(mdbuf), NULL, NULL);
 
     if (retval != RIG_OK)
     {
@@ -426,8 +424,8 @@ int dxsr8_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
         wide_filter = 0; // AL~RW_NAR00 Set wide bandwidth
     }
 
-    mdbuf_len = sprintf(mdbuf, AL "~RW_NAR%02d" EOM, wide_filter);
-    retval = dxsr8_transaction(rig, mdbuf, mdbuf_len, NULL, NULL);
+    SNPRINTF(mdbuf, sizeof(mdbuf), AL "~RW_NAR%02d" EOM, wide_filter);
+    retval = dxsr8_transaction(rig, mdbuf, strlen(mdbuf), NULL, NULL);
 
     return retval;
 }
@@ -493,20 +491,19 @@ int dxsr8_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
 int dxsr8_set_func(RIG *rig, vfo_t vfo, setting_t func, int status)
 {
 
-    int cmd_len;
     char cmd[BUFSZ];
 
     switch (func)
     {
     case RIG_FUNC_FAGC:
-        cmd_len = sprintf(cmd, AL "~RW_AGC%02d" EOM, status ? 0 : 1);
+        SNPRINTF(cmd, sizeof(cmd), AL "~RW_AGC%02d" EOM, status ? 0 : 1);
 
-        return dxsr8_transaction(rig, cmd, cmd_len, NULL, NULL);
+        return dxsr8_transaction(rig, cmd, strlen(cmd), NULL, NULL);
 
     case RIG_FUNC_NB:
-        cmd_len = sprintf(cmd, AL "~RW_NZB%d" EOM, status ? 1 : 0);
+        SNPRINTF(cmd, sizeof(cmd), AL "~RW_NZB%d" EOM, status ? 1 : 0);
 
-        return dxsr8_transaction(rig, cmd, cmd_len, NULL, NULL);
+        return dxsr8_transaction(rig, cmd, strlen(cmd), NULL, NULL);
 
 
     default:
@@ -571,7 +568,7 @@ int dxsr8_get_func(RIG *rig, vfo_t vfo, setting_t func, int *status)
  */
 int dxsr8_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 {
-    int cmd_len, lvl;
+    int lvl;
     char cmd[BUFSZ];
 
     switch (level)
@@ -590,8 +587,8 @@ int dxsr8_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
             return -RIG_EINVAL;
         }
 
-        cmd_len = sprintf(cmd, AL "~RW_RFG%02d" EOM, lvl);
-        return dxsr8_transaction(rig, cmd, cmd_len, NULL, NULL);
+        SNPRINTF(cmd, sizeof(cmd), AL "~RW_RFG%02d" EOM, lvl);
+        return dxsr8_transaction(rig, cmd, strlen(cmd), NULL, NULL);
 
     case RIG_LEVEL_ATT:
         switch (val.i)
@@ -609,8 +606,8 @@ int dxsr8_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
             return -RIG_EINVAL;
         }
 
-        cmd_len = sprintf(cmd, AL "~RW_RFG%02d" EOM, lvl);
-        return dxsr8_transaction(rig, cmd, cmd_len, NULL, NULL);
+        SNPRINTF(cmd, sizeof(cmd), AL "~RW_RFG%02d" EOM, lvl);
+        return dxsr8_transaction(rig, cmd, strlen(cmd), NULL, NULL);
 
     case RIG_LEVEL_RFPOWER:
 
@@ -627,8 +624,8 @@ int dxsr8_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
             lvl = 0; // AL~RW_PWR00 - High power
         }
 
-        cmd_len = sprintf(cmd, AL "~RW_PWR%02d" EOM, lvl);
-        return dxsr8_transaction(rig, cmd, cmd_len, NULL, NULL);
+        SNPRINTF(cmd, sizeof(cmd), AL "~RW_PWR%02d" EOM, lvl);
+        return dxsr8_transaction(rig, cmd, strlen(cmd), NULL, NULL);
 
     default:
         rig_debug(RIG_DEBUG_ERR, "Unsupported set_level %s\n", rig_strlevel(level));
@@ -767,8 +764,7 @@ int dxsr8_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
 {
 
     char cmd[BUFSZ];
-    int cmd_len;
 
-    cmd_len = sprintf(cmd, AL "~RW_PTT%02d" EOM, ptt);
-    return dxsr8_transaction(rig, cmd, cmd_len, NULL, NULL);
+    SNPRINTF(cmd, sizeof(cmd), AL "~RW_PTT%02d" EOM, ptt);
+    return dxsr8_transaction(rig, cmd, strlen(cmd), NULL, NULL);
 }
