@@ -19,15 +19,11 @@
  *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>             /* String function definitions */
-#include <unistd.h>             /* UNIX standard function definitions */
-#include <math.h>
 
 #include <hamlib/rig.h>
 #include <serial.h>
@@ -178,7 +174,8 @@ struct rig_caps trxmanager_caps =
     .set_split_vfo = trxmanager_set_split_vfo,
     .get_split_vfo = trxmanager_get_split_vfo,
     .set_split_freq_mode = trxmanager_set_split_freq_mode,
-    .get_split_freq_mode = trxmanager_get_split_freq_mode
+    .get_split_freq_mode = trxmanager_get_split_freq_mode,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 /*
@@ -242,7 +239,8 @@ static int read_transaction(RIG *rig, char *response, int response_len)
 
     rig_debug(RIG_DEBUG_TRACE, "%s\n", __func__);
 
-    len = read_string(&rs->rigport, response, response_len, delims,
+    len = read_string(&rs->rigport, (unsigned char *) response, response_len,
+                      delims,
                       strlen(delims), 0, 1);
 
     if (len <= 0)
@@ -264,8 +262,8 @@ static int trxmanager_init(RIG *rig)
 
     rig_debug(RIG_DEBUG_TRACE, "%s version %s\n", __func__, BACKEND_VER);
 
-    rig->state.priv = (struct trxmanager_priv_data *)malloc(
-                          sizeof(struct trxmanager_priv_data));
+    rig->state.priv = (struct trxmanager_priv_data *)calloc(1,
+                      sizeof(struct trxmanager_priv_data));
 
     if (!rig->state.priv)
     {
@@ -330,7 +328,7 @@ static int trxmanager_open(RIG *rig)
 
     // Turn off active messages
     cmd = "AI0;";
-    retval = write_block(&rs->rigport, cmd, strlen(cmd));
+    retval = write_block(&rs->rigport, (unsigned char *) cmd, strlen(cmd));
 
     if (retval < 0)
     {
@@ -353,7 +351,7 @@ static int trxmanager_open(RIG *rig)
     rig_debug(RIG_DEBUG_VERBOSE, "%s AI response=%s\n", __func__, response);
 
     cmd = "FN;";
-    retval = write_block(&rs->rigport, cmd, strlen(cmd));
+    retval = write_block(&rs->rigport, (unsigned char *) cmd, strlen(cmd));
 
     if (retval != RIG_OK)
     {
@@ -440,8 +438,8 @@ static int trxmanager_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
     }
 
     vfoab = vfo == RIG_VFO_A ? 'R' : 'T';
-    snprintf(cmd, sizeof(cmd), "X%c;", vfoab);
-    retval = write_block(&rs->rigport, cmd, strlen(cmd));
+    SNPRINTF(cmd, sizeof(cmd), "X%c;", vfoab);
+    retval = write_block(&rs->rigport, (unsigned char *) cmd, strlen(cmd));
 
     if (retval < 0)
     {
@@ -510,8 +508,8 @@ static int trxmanager_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
     }
 
     vfoab = vfo == RIG_VFO_A ? 'A' : 'B';
-    snprintf(cmd, sizeof(cmd), "F%c%011lu;", vfoab, (unsigned long)freq);
-    retval = write_block(&rs->rigport, cmd, strlen(cmd));
+    SNPRINTF(cmd, sizeof(cmd), "F%c%011lu;", vfoab, (unsigned long)freq);
+    retval = write_block(&rs->rigport, (unsigned char *) cmd, strlen(cmd));
 
     if (retval < 0)
     {
@@ -549,8 +547,8 @@ static int trxmanager_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
         return -RIG_EINVAL;
     }
 
-    snprintf(cmd, sizeof(cmd), "%s;", ptt == 1 ? "TX" : "RX");
-    retval = write_block(&rs->rigport, cmd, strlen(cmd));
+    SNPRINTF(cmd, sizeof(cmd), "%s;", ptt == 1 ? "TX" : "RX");
+    retval = write_block(&rs->rigport, (unsigned char *) cmd, strlen(cmd));
 
     if (retval < 0)
     {
@@ -588,8 +586,8 @@ static int trxmanager_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
     rig_debug(RIG_DEBUG_TRACE, "%s: vfo=%s\n", __func__,
               rig_strvfo(vfo));
 
-    snprintf(cmd, sizeof(cmd), "IF;");
-    retval = write_block(&rs->rigport, cmd, strlen(cmd));
+    SNPRINTF(cmd, sizeof(cmd), "IF;");
+    retval = write_block(&rs->rigport, (unsigned char *) cmd, strlen(cmd));
 
     if (retval < 0)
     {
@@ -713,8 +711,8 @@ static int trxmanager_set_mode(RIG *rig, vfo_t vfo, rmode_t mode,
 
     }
 
-    snprintf(cmd, sizeof(cmd), "MD%c;", ttmode);
-    retval = write_block(&rs->rigport, cmd, strlen(cmd));
+    SNPRINTF(cmd, sizeof(cmd), "MD%c;", ttmode);
+    retval = write_block(&rs->rigport, (unsigned char *) cmd, strlen(cmd));
 
     if (retval < 0)
     {
@@ -776,8 +774,8 @@ static int trxmanager_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode,
     rig_debug(RIG_DEBUG_TRACE, "%s: using vfo=%s\n", __func__,
               rig_strvfo(vfo));
 
-    snprintf(cmd, sizeof(cmd), "MD;");
-    retval = write_block(&rs->rigport, cmd, strlen(cmd));
+    SNPRINTF(cmd, sizeof(cmd), "MD;");
+    retval = write_block(&rs->rigport, (unsigned char *) cmd, strlen(cmd));
 
     if (retval < 0)
     {
@@ -849,8 +847,8 @@ static int trxmanager_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode,
     rig_debug(RIG_DEBUG_VERBOSE, "%s: mode='%s'\n", __func__, rig_strrmode(*mode));
 
     // now get the bandwidth
-    snprintf(cmd, sizeof(cmd), "BW;");
-    retval = write_block(&rs->rigport, cmd, strlen(cmd));
+    SNPRINTF(cmd, sizeof(cmd), "BW;");
+    retval = write_block(&rs->rigport, (unsigned char *) cmd, strlen(cmd));
 
     if (retval < 0)
     {
@@ -916,8 +914,8 @@ static int trxmanager_set_vfo(RIG *rig, vfo_t vfo)
         vfo = priv->vfo_curr;
     }
 
-    snprintf(cmd, sizeof(cmd), "FN%d;", vfo == RIG_VFO_A ? 0 : 1);
-    retval = write_block(&rs->rigport, cmd, strlen(cmd));
+    SNPRINTF(cmd, sizeof(cmd), "FN%d;", vfo == RIG_VFO_A ? 0 : 1);
+    retval = write_block(&rs->rigport, (unsigned char *) cmd, strlen(cmd));
 
     if (retval < 0)
     {
@@ -1001,8 +999,8 @@ static int trxmanager_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
         return -RIG_EINVAL;
     }
 
-    snprintf(cmd, sizeof(cmd), "XT%011lu;", (unsigned long) tx_freq);
-    retval = write_block(&rs->rigport, cmd, strlen(cmd));
+    SNPRINTF(cmd, sizeof(cmd), "XT%011lu;", (unsigned long) tx_freq);
+    retval = write_block(&rs->rigport, (unsigned char *) cmd, strlen(cmd));
 
     if (retval < 0)
     {
@@ -1069,8 +1067,8 @@ static int trxmanager_set_split_vfo(RIG *rig, vfo_t vfo, split_t split,
 
     if (tsplit == split) { return RIG_OK; } // don't need to change it
 
-    snprintf(cmd, sizeof(cmd), "SP%c;", split ? '1' : '0');
-    retval = write_block(&rs->rigport, cmd, strlen(cmd));
+    SNPRINTF(cmd, sizeof(cmd), "SP%c;", split ? '1' : '0');
+    retval = write_block(&rs->rigport, (unsigned char *) cmd, strlen(cmd));
 
     if (retval < 0)
     {
@@ -1111,8 +1109,8 @@ static int trxmanager_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split,
                                         rig->state.priv;
 
     rig_debug(RIG_DEBUG_TRACE, "%s\n", __func__);
-    snprintf(cmd, sizeof(cmd), "SP;");
-    retval = write_block(&rs->rigport, cmd, strlen(cmd));
+    SNPRINTF(cmd, sizeof(cmd), "SP;");
+    retval = write_block(&rs->rigport, (unsigned char *) cmd, strlen(cmd));
 
     if (retval < 0)
     {
@@ -1164,8 +1162,8 @@ static int trxmanager_set_split_freq_mode(RIG *rig, vfo_t vfo, freq_t freq,
 
     // assume split is on B
     //
-    snprintf(cmd, sizeof(cmd), "XT%011lu;", (unsigned long)freq);
-    retval = write_block(&rs->rigport, cmd, strlen(cmd));
+    SNPRINTF(cmd, sizeof(cmd), "XT%011lu;", (unsigned long)freq);
+    retval = write_block(&rs->rigport, (unsigned char *) cmd, strlen(cmd));
 
     if (retval < 0)
     {
