@@ -24,12 +24,9 @@
  * 25Mar09: Initial release
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdlib.h>
-#include <string.h>  /* String function definitions */
 
 #include <hamlib/rig.h>
 #include "token.h"
@@ -121,7 +118,7 @@ static const struct icom_priv_caps IC7200_priv_caps =
         { .level = RIG_AGC_OFF, .icom_level = 0 },
         { .level = RIG_AGC_FAST, .icom_level = 1 },
         { .level = RIG_AGC_SLOW, .icom_level = 2 },
-        { .level = -1, .icom_level = 0 },
+        { .level = RIG_AGC_LAST, .icom_level = -1 },
     },
 };
 
@@ -130,9 +127,9 @@ const struct rig_caps ic7200_caps =
     RIG_MODEL(RIG_MODEL_IC7200),
     .model_name = "IC-7200",
     .mfg_name =  "Icom",
-    .version =  BACKEND_VER ".0",
+    .version =  BACKEND_VER ".2",
     .copyright =  "LGPL",
-    .status =  RIG_STATUS_BETA,
+    .status =  RIG_STATUS_STABLE,
     .rig_type =  RIG_TYPE_TRANSCEIVER,
     .ptt_type =  RIG_PTT_RIG,
     .dcd_type =  RIG_DCD_RIG,
@@ -247,6 +244,7 @@ const struct rig_caps ic7200_caps =
     .set_mode =  icom_set_mode_with_data,
     .get_mode =  icom_get_mode_with_data,
     .set_vfo =  icom_set_vfo,
+    .get_vfo =  icom_get_vfo,
     .set_ant =  NULL,  /*automatically set by rig depending band */
     .get_ant =  NULL,
 
@@ -280,6 +278,7 @@ const struct rig_caps ic7200_caps =
     .set_split_vfo =  icom_set_split_vfo,
     .get_split_vfo =  NULL,
 
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 int ic7200_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
