@@ -19,9 +19,7 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <string.h>  /* String function definitions */
 
@@ -88,8 +86,6 @@ struct cmdparams ic756pro_cmdparms[] =
          { 247 ,60 } \
     } }
 
-int ic756_set_func(RIG *rig, vfo_t vfo, setting_t func, int status);
-
 /*
  *  This function deals with the older type radios with only 2 filter widths
  *  (0 - normal, 1 - narrow)
@@ -135,7 +131,7 @@ static const struct icom_priv_caps ic756_priv_caps =
         { .level = RIG_AGC_FAST, .icom_level = 1 },
         { .level = RIG_AGC_MEDIUM, .icom_level = 2 },
         { .level = RIG_AGC_SLOW, .icom_level = 3 },
-        { .level = -1, .icom_level = 0 },
+        { .level = RIG_AGC_LAST, .icom_level = -1 },
     },
 };
 
@@ -144,7 +140,7 @@ const struct rig_caps ic756_caps =
     RIG_MODEL(RIG_MODEL_IC756),
     .model_name = "IC-756",
     .mfg_name =  "Icom",
-    .version =  BACKEND_VER ".0",
+    .version =  BACKEND_VER ".3",
     .copyright =  "LGPL",
     .status =  RIG_STATUS_STABLE,
     .rig_type =  RIG_TYPE_TRANSCEIVER,
@@ -254,13 +250,14 @@ const struct rig_caps ic756_caps =
     .set_mode =  icom_set_mode,
     .get_mode =  icom_get_mode,
     .set_vfo =  icom_set_vfo,
+    .get_vfo =  icom_get_vfo,
     .set_ant =  icom_set_ant,
     .get_ant =  icom_get_ant,
 
     .decode_event =  icom_decode_event,
     .set_level =  icom_set_level,
     .get_level =  icom_get_level,
-    .set_func =  ic756_set_func,
+    .set_func =  icom_set_func,
     .get_func =  icom_get_func,
     .set_mem =  icom_set_mem,
     .vfo_op =  icom_vfo_op,
@@ -276,7 +273,7 @@ const struct rig_caps ic756_caps =
     .get_split_freq_mode =  icom_get_split_freq_mode,
     .set_split_vfo =  icom_set_split_vfo,
     .get_split_vfo =  NULL,
-
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 
@@ -298,7 +295,7 @@ static const struct icom_priv_caps ic756pro_priv_caps =
         { .level = RIG_AGC_FAST, .icom_level = 1 },
         { .level = RIG_AGC_MEDIUM, .icom_level = 2 },
         { .level = RIG_AGC_SLOW, .icom_level = 3 },
-        { .level = -1, .icom_level = 0 },
+        { .level = RIG_AGC_LAST, .icom_level = -2 },
     },
 };
 
@@ -307,7 +304,7 @@ const struct rig_caps ic756pro_caps =
     RIG_MODEL(RIG_MODEL_IC756PRO),
     .model_name = "IC-756PRO",
     .mfg_name =  "Icom",
-    .version =  BACKEND_VER ".0",
+    .version =  BACKEND_VER ".3",
     .copyright =  "LGPL",
     .status =  RIG_STATUS_STABLE,
     .rig_type =  RIG_TYPE_TRANSCEIVER,
@@ -416,13 +413,14 @@ const struct rig_caps ic756pro_caps =
     .set_mode =  icom_set_mode,
     .get_mode =  icom_get_mode,
     .set_vfo =  icom_set_vfo,
+    .get_vfo =  icom_get_vfo,
     .set_ant =  icom_set_ant,
     .get_ant =  icom_get_ant,
 
     .decode_event =  icom_decode_event,
     .set_level =  icom_set_level,
     .get_level =  icom_get_level,
-    .set_func =  ic756_set_func,
+    .set_func =  icom_set_func,
     .get_func =  icom_get_func,
     .set_mem =  icom_set_mem,
     .vfo_op =  icom_vfo_op,
@@ -448,6 +446,7 @@ const struct rig_caps ic756pro_caps =
     .set_split_vfo =  icom_set_split_vfo,
     .get_split_vfo =  NULL,
 
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 /*
@@ -468,7 +467,7 @@ static const struct icom_priv_caps ic756pro2_priv_caps =
         { .level = RIG_AGC_FAST, .icom_level = 1 },
         { .level = RIG_AGC_MEDIUM, .icom_level = 2 },
         { .level = RIG_AGC_SLOW, .icom_level = 3 },
-        { .level = -1, .icom_level = 0 },
+        { .level = RIG_AGC_LAST, .icom_level = -1 },
     },
     .extcmds = ic756pro_cmdparms,   /* Custom op parameters */
 };
@@ -544,7 +543,7 @@ const struct rig_caps ic756pro2_caps =
     RIG_MODEL(RIG_MODEL_IC756PROII),
     .model_name = "IC-756PROII",
     .mfg_name =  "Icom",
-    .version =  BACKEND_VER ".0",
+    .version =  BACKEND_VER ".3",
     .copyright =  "LGPL",
     .status =  RIG_STATUS_STABLE,
     .rig_type =  RIG_TYPE_TRANSCEIVER,
@@ -654,6 +653,7 @@ const struct rig_caps ic756pro2_caps =
     .set_mode =  icom_set_mode_with_data,
     .get_mode =  icom_get_mode_with_data,
     .set_vfo =  icom_set_vfo,
+    .get_vfo =  icom_get_vfo,
     .set_ant =  icom_set_ant,
     .get_ant =  icom_get_ant,
 
@@ -662,7 +662,7 @@ const struct rig_caps ic756pro2_caps =
     .get_parm =  icom_get_parm,
     .set_level =  icom_set_level,
     .get_level =  icom_get_level,
-    .set_func =  ic756_set_func,
+    .set_func =  icom_set_func,
     .get_func =  icom_get_func,
     .set_mem =  icom_set_mem,
     .vfo_op =  icom_vfo_op,
@@ -690,6 +690,7 @@ const struct rig_caps ic756pro2_caps =
 
     .set_ext_parm =  ic756pro2_set_ext_parm,
     .get_ext_parm =  ic756pro2_get_ext_parm,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 
@@ -897,7 +898,7 @@ static const struct icom_priv_caps ic756pro3_priv_caps =
         { .level = RIG_AGC_FAST, .icom_level = 1 },
         { .level = RIG_AGC_MEDIUM, .icom_level = 2 },
         { .level = RIG_AGC_SLOW, .icom_level = 3 },
-        { .level = -1, .icom_level = 0 },
+        { .level = RIG_AGC_LAST, .icom_level = -1 },
     },
     .extcmds = ic756pro_cmdparms,   /* Custom op parameters */
 };
@@ -976,7 +977,7 @@ const struct rig_caps ic756pro3_caps =
     RIG_MODEL(RIG_MODEL_IC756PROIII),
     .model_name = "IC-756PROIII",
     .mfg_name =  "Icom",
-    .version =  BACKEND_VER ".1",
+    .version =  BACKEND_VER ".3",
     .copyright =  "LGPL",
     .status =  RIG_STATUS_STABLE,
     .rig_type =  RIG_TYPE_TRANSCEIVER,
@@ -1097,6 +1098,7 @@ const struct rig_caps ic756pro3_caps =
     .set_mode =  icom_set_mode_with_data,
     .get_mode =  icom_get_mode_with_data,
     .set_vfo =  icom_set_vfo,
+    .get_vfo =  icom_get_vfo,
     .set_ant =  icom_set_ant,
     .get_ant =  icom_get_ant,
 
@@ -1105,7 +1107,7 @@ const struct rig_caps ic756pro3_caps =
     .get_parm =  icom_get_parm,
     .set_level =  icom_set_level,
     .get_level =  icom_get_level,
-    .set_func =  ic756_set_func,
+    .set_func =  icom_set_func,
     .get_func =  icom_get_func,
     .set_mem =  icom_set_mem,
     .vfo_op =  icom_vfo_op,
@@ -1129,38 +1131,5 @@ const struct rig_caps ic756pro3_caps =
 
     .set_ext_parm =  ic756pro2_set_ext_parm,
     .get_ext_parm =  ic756pro2_get_ext_parm,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
-
-int ic756_set_func(RIG *rig, vfo_t vfo, setting_t func, int status)
-{
-    unsigned char fctbuf[MAXFRAMELEN], ackbuf[MAXFRAMELEN];
-    int fct_len = 0, acklen, retval;
-    int fct_cn, fct_sc;        /* Command Number, Subcommand */
-
-    switch (func)
-    {
-    case RIG_FUNC_DUAL_WATCH:
-        fct_cn = C_SET_VFO;
-        fct_sc = status ? S_DUAL_ON : S_DUAL_OFF;
-        break;
-
-    default:
-        return icom_set_func(rig, vfo, func, status);
-    }
-
-    retval = icom_transaction(rig, fct_cn, fct_sc, fctbuf, fct_len, ackbuf,
-                              &acklen);
-
-    if (retval != RIG_OK)
-    {
-        return retval;
-    }
-
-    if (acklen != 1)
-    {
-        rig_debug(RIG_DEBUG_ERR, "%s: wrong frame len=%d\n", __func__, acklen);
-        return -RIG_EPROTO;
-    }
-
-    return RIG_OK;
-}
