@@ -22,9 +22,7 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdlib.h>
 
@@ -43,7 +41,9 @@
 
 #define K3_FUNC_ALL (RIG_FUNC_NB|RIG_FUNC_VOX|RIG_FUNC_APF|\
     RIG_FUNC_DUAL_WATCH|RIG_FUNC_DIVERSITY|\
-    RIG_FUNC_LOCK|RIG_FUNC_RIT|RIG_FUNC_XIT)
+    RIG_FUNC_LOCK|RIG_FUNC_RIT|RIG_FUNC_XIT|RIG_FUNC_SEND_MORSE)
+
+#define K4_FUNC_ALL (K3_FUNC_ALL|RIG_FUNC_MUTE)
 
 #define K3_LEVEL_ALL (RIG_LEVEL_ATT|RIG_LEVEL_PREAMP|RIG_LEVEL_AGC|RIG_LEVEL_SQL|\
     RIG_LEVEL_STRENGTH|RIG_LEVEL_ALC|RIG_LEVEL_RFPOWER|RIG_LEVEL_KEYSPD|\
@@ -141,6 +141,7 @@ static struct kenwood_priv_caps k3_priv_caps  =
 
 
 /* K3 specific function declarations */
+int k3_set_freq(RIG *rig, vfo_t vfo, freq_t freq);
 int k3_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width);
 int k3_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width);
 int k3_get_vfo(RIG *rig, vfo_t *vfo);
@@ -167,6 +168,9 @@ int k3_get_bar_graph_level(RIG *rig, float *smeter, float *pwr, float *alc,
                            int *mode_tx);
 int kx3_get_bar_graph_level(RIG *rig, float *level);
 
+/* K4 functions */
+int k4_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt);
+int k4_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt);
 
 /*
  * K3 rig capabilities.
@@ -183,7 +187,7 @@ const struct rig_caps k3_caps =
     RIG_MODEL(RIG_MODEL_K3),
     .model_name =       "K3",
     .mfg_name =     "Elecraft",
-    .version =      BACKEND_VER ".16",
+    .version =      BACKEND_VER ".23",
     .copyright =        "LGPL",
     .status =       RIG_STATUS_STABLE,
     .rig_type =     RIG_TYPE_TRANSCEIVER,
@@ -197,7 +201,7 @@ const struct rig_caps k3_caps =
     .serial_parity =    RIG_PARITY_NONE,
     .serial_handshake = RIG_HANDSHAKE_NONE,
     .write_delay =      0,  /* Timing between bytes */
-    .post_write_delay = 0,  /* Timing between command strings */
+    .post_write_delay = 10,  /* Timing between command strings */
     .timeout =      1000,   /* FA and FB make take up to 500 ms on band change */
     .retry =        5,
 
@@ -292,7 +296,7 @@ const struct rig_caps k3_caps =
     .rig_cleanup =      kenwood_cleanup,
     .rig_open =     elecraft_open,
     .rig_close =        kenwood_close,
-    .set_freq =     kenwood_set_freq,
+    .set_freq =     k3_set_freq,
     .get_freq =     kenwood_get_freq,
     .set_mode =     k3_set_mode,
     .get_mode =     k3_get_mode,
@@ -325,8 +329,9 @@ const struct rig_caps k3_caps =
     .set_ant =      kenwood_set_ant_no_ack,
     .get_ant =      kenwood_get_ant,
     .send_morse =       kenwood_send_morse,
-    .wait_morse =       rig_wait_morse
+    .wait_morse =       rig_wait_morse,
 
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 const struct rig_caps k3s_caps =
@@ -334,7 +339,7 @@ const struct rig_caps k3s_caps =
     RIG_MODEL(RIG_MODEL_K3S),
     .model_name =       "K3S",
     .mfg_name =     "Elecraft",
-    .version =      BACKEND_VER ".14",
+    .version =      BACKEND_VER ".18",
     .copyright =        "LGPL",
     .status =       RIG_STATUS_STABLE,
     .rig_type =     RIG_TYPE_TRANSCEIVER,
@@ -348,7 +353,7 @@ const struct rig_caps k3s_caps =
     .serial_parity =    RIG_PARITY_NONE,
     .serial_handshake = RIG_HANDSHAKE_NONE,
     .write_delay =      0,  /* Timing between bytes */
-    .post_write_delay = 0,  /* Timing between command strings */
+    .post_write_delay = 10,  /* Timing between command strings */
     .timeout =      1000,   /* FA and FB make take up to 500 ms on band change */
     .retry =        5,
 
@@ -442,7 +447,7 @@ const struct rig_caps k3s_caps =
     .rig_cleanup =      kenwood_cleanup,
     .rig_open =     elecraft_open,
     .rig_close =        kenwood_close,
-    .set_freq =     kenwood_set_freq,
+    .set_freq =     k3_set_freq,
     .get_freq =     kenwood_get_freq,
     .set_mode =     k3_set_mode,
     .get_mode =     k3_get_mode,
@@ -475,7 +480,8 @@ const struct rig_caps k3s_caps =
     .set_ant =      kenwood_set_ant_no_ack,
     .get_ant =      kenwood_get_ant,
     .send_morse =       kenwood_send_morse,
-    .wait_morse =       rig_wait_morse
+    .wait_morse =       rig_wait_morse,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 // How similar is this to the K3S?
@@ -484,7 +490,7 @@ const struct rig_caps k4_caps =
     RIG_MODEL(RIG_MODEL_K4),
     .model_name =       "K4",
     .mfg_name =     "Elecraft",
-    .version =      BACKEND_VER ".14",
+    .version =      BACKEND_VER ".24",
     .copyright =        "LGPL",
     .status =       RIG_STATUS_STABLE,
     .rig_type =     RIG_TYPE_TRANSCEIVER,
@@ -497,13 +503,13 @@ const struct rig_caps k4_caps =
     .serial_stop_bits = 1,
     .serial_parity =    RIG_PARITY_NONE,
     .serial_handshake = RIG_HANDSHAKE_NONE,
-    .write_delay =      0,  /* Timing between bytes */
+    .write_delay =      0, /* Timing between bytes */
     .post_write_delay = 0,  /* Timing between command strings */
     .timeout =      1000,   /* FA and FB make take up to 500 ms on band change */
     .retry =        5,
 
-    .has_get_func =     K3_FUNC_ALL,
-    .has_set_func =     K3_FUNC_ALL,
+    .has_get_func =     K4_FUNC_ALL,
+    .has_set_func =     K4_FUNC_ALL,
     .has_get_level =    K3_LEVEL_ALL,
     .has_set_level =    RIG_LEVEL_SET(K3_LEVEL_ALL),
     .has_get_parm =     RIG_PARM_NONE,
@@ -597,7 +603,7 @@ const struct rig_caps k4_caps =
     .set_mode =     k3_set_mode,
     .get_mode =     k3_get_mode,
     .set_vfo =      k3_set_vfo,
-    .get_vfo =      k3_get_vfo,
+    .get_vfo =      elecraft_get_vfo_tq,
     .set_split_mode =   k3_set_split_mode,
     .get_split_mode =   k3_get_split_mode,
     .set_split_vfo =    kenwood_set_split_vfo,
@@ -606,8 +612,8 @@ const struct rig_caps k4_caps =
     .get_rit =      kenwood_get_rit,
     .set_xit =      k3_set_xit,
     .get_xit =      kenwood_get_xit,
-    .get_ptt =      kenwood_get_ptt,
-    .set_ptt =      kenwood_set_ptt,
+    .get_ptt =      k4_get_ptt,
+    .set_ptt =      k4_set_ptt,
     .get_dcd =      kenwood_get_dcd,
     .set_func =     k3_set_func,
     .get_func =     k3_get_func,
@@ -625,7 +631,8 @@ const struct rig_caps k4_caps =
     .set_ant =      kenwood_set_ant_no_ack,
     .get_ant =      kenwood_get_ant,
     .send_morse =       kenwood_send_morse,
-    .wait_morse =       rig_wait_morse
+    .wait_morse =       rig_wait_morse,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 const struct rig_caps kx3_caps =
@@ -633,7 +640,7 @@ const struct rig_caps kx3_caps =
     RIG_MODEL(RIG_MODEL_KX3),
     .model_name =       "KX3",
     .mfg_name =     "Elecraft",
-    .version =      BACKEND_VER ".14",
+    .version =      BACKEND_VER ".17",
     .copyright =        "LGPL",
     .status =       RIG_STATUS_STABLE,
     .rig_type =     RIG_TYPE_TRANSCEIVER,
@@ -774,7 +781,8 @@ const struct rig_caps kx3_caps =
     .set_ant =      kenwood_set_ant_no_ack,
     .get_ant =      kenwood_get_ant,
     .send_morse =       kenwood_send_morse,
-    .wait_morse =       rig_wait_morse
+    .wait_morse =       rig_wait_morse,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 const struct rig_caps kx2_caps =
@@ -782,7 +790,7 @@ const struct rig_caps kx2_caps =
     RIG_MODEL(RIG_MODEL_KX2),
     .model_name =       "KX2",
     .mfg_name =     "Elecraft",
-    .version =      BACKEND_VER ".14",
+    .version =      BACKEND_VER ".16",
     .copyright =        "LGPL",
     .status =       RIG_STATUS_STABLE,
     .rig_type =     RIG_TYPE_TRANSCEIVER,
@@ -923,7 +931,8 @@ const struct rig_caps kx2_caps =
     .set_ant =      kenwood_set_ant_no_ack,
     .get_ant =      kenwood_get_ant,
     .send_morse =       kenwood_send_morse,
-    .wait_morse =       rig_wait_morse
+    .wait_morse =       rig_wait_morse,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 /*
@@ -943,8 +952,19 @@ int k3_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
     int err;
     rmode_t temp_m;
     pbwidth_t temp_w;
+    char *cmd_mode = "DT";
+    char *cmd_bw = "BW";
+    int cmd_bw_len = 6;
 
-    rig_debug(RIG_DEBUG_VERBOSE, "%s called\n", __func__);
+    rig_debug(RIG_DEBUG_VERBOSE, "%s called vfo=%s\n", __func__, rig_strvfo(vfo));
+
+    if (vfo == RIG_VFO_B && rig->caps->rig_model == RIG_MODEL_K4)
+    {
+        cmd_mode = "DT$";
+        cmd_bw = "BW$";
+        cmd_bw_len = 7;
+
+    }
 
     if (!mode || !width)
     {
@@ -965,7 +985,8 @@ int k3_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
 
     if (temp_m == RIG_MODE_RTTY)
     {
-        err = kenwood_safe_transaction(rig, "DT", buf, KENWOOD_MAX_BUF_LEN, 3);
+        err = kenwood_safe_transaction(rig, cmd_mode, buf, KENWOOD_MAX_BUF_LEN,
+                                       strlen(cmd_mode) + 1);
 
         if (err != RIG_OK)
         {
@@ -992,7 +1013,8 @@ int k3_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
     }
     else if (temp_m == RIG_MODE_RTTYR)
     {
-        err = kenwood_safe_transaction(rig, "DT", buf, KENWOOD_MAX_BUF_LEN, 3);
+        err = kenwood_safe_transaction(rig, cmd_mode, buf, KENWOOD_MAX_BUF_LEN,
+                                       strlen(cmd_mode) + 1);
 
         if (err != RIG_OK)
         {
@@ -1026,14 +1048,8 @@ int k3_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
     /* The K3 is not limited to specific filter widths so we can query
      * the actual bandwidth using the BW command
      */
-    if (vfo == RIG_VFO_B)
-    {
-        err = kenwood_safe_transaction(rig, "BW$", buf, KENWOOD_MAX_BUF_LEN, 7);
-    }
-    else
-    {
-        err = kenwood_safe_transaction(rig, "BW", buf, KENWOOD_MAX_BUF_LEN, 6);
-    }
+    err = kenwood_safe_transaction(rig, cmd_bw, buf, KENWOOD_MAX_BUF_LEN,
+                                   cmd_bw_len);
 
     if (err != RIG_OK)
     {
@@ -1041,14 +1057,7 @@ int k3_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
         return err;
     }
 
-    if (vfo == RIG_VFO_B)
-    {
-        *width = atoi(&buf[3]) * 10;
-    }
-    else
-    {
-        *width = atoi(&buf[2]) * 10;
-    }
+    *width = atoi(&buf[cmd_bw_len - 4]) * 10;
 
     return RIG_OK;
 }
@@ -1087,23 +1096,30 @@ int k3_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
 
 int k3_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 {
-    int err;
-    char cmd_m[4];
+    int err, err2;
+    char cmd_m[5];
     char buf[KENWOOD_MAX_BUF_LEN];
+    char *dtcmd;
     struct kenwood_priv_caps *caps = kenwood_caps(rig);
+    struct kenwood_priv_data *priv = rig->state.priv;
 
-    rig_debug(RIG_DEBUG_VERBOSE, "%s called\n", __func__);
+    rig_debug(RIG_DEBUG_VERBOSE, "%s called vfo=%s mode=%s width=%d\n", __func__,
+              rig_strvfo(vfo), rig_strrmode(mode), (int)width);
 
     if (vfo == RIG_VFO_CURR)
     {
         vfo = rig->state.current_vfo;
     }
 
-    rmode_t tmode;
+    rmode_t tmodeA, tmodeB;
     pbwidth_t twidth;
-    err = k3_get_mode(rig, vfo, &tmode, &twidth);
+    err = k3_get_mode(rig, RIG_VFO_A, &tmodeA, &twidth);
+    err2 = k3_get_mode(rig, RIG_VFO_B, &tmodeB, &twidth);
 
-    if (err == RIG_OK && tmode == mode && width == RIG_PASSBAND_NOCHANGE)
+    // we keep both vfos in the same mode -- any reason they should ever be different?  If so, fix this
+    // if we change mode on one VFO we'll also change the other
+    if (err == RIG_OK && err2 == RIG_OK && tmodeA == mode && tmodeB == mode
+            && width == RIG_PASSBAND_NOCHANGE)
     {
         rig_debug(RIG_DEBUG_TRACE, "%s(%d): mode/width no change, skipping\n", __FILE__,
                   __LINE__);
@@ -1112,40 +1128,49 @@ int k3_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
     else
     {
         rig_debug(RIG_DEBUG_TRACE,
-                  "%s(%d): changing mode=%s, oldmode=%s, width=%ld, oldwidth=%ld\n", __FILE__,
-                  __LINE__, rig_strrmode(tmode), rig_strrmode(mode), twidth, width);
+                  "%s(%d): changing oldmode=A=%s B=%s, to mode=%s, oldwidth=%ld, to width=%ld\n",
+                  __FILE__,
+                  __LINE__, rig_strrmode(tmodeA), rig_strrmode(tmodeB), rig_strrmode(mode),
+                  twidth, width);
+    }
+
+    dtcmd = "DT";
+
+    if ((priv->is_k4 || priv->is_k4d || priv->is_k4hd)  && vfo == RIG_VFO_B)
+    {
+        dtcmd = "DT$";
     }
 
     switch (mode)
     {
     case RIG_MODE_PKTLSB:
         mode = RIG_MODE_RTTYR; // in "DT0" Subband RIG_MODE_RTTYR = USB and RIG_MODE_RTTY = LSB
-        snprintf(cmd_m, sizeof(cmd_m),
-                 "DT0"); /* DATA A mode - DATA (REV) on LSB optimized for HF Packet, VFO dial is suppressed carrier QRG */
+        SNPRINTF(cmd_m, sizeof(cmd_m),
+                 "%s0", dtcmd); /* DATA A mode - DATA (REV) on LSB optimized for HF Packet, VFO dial is suppressed carrier QRG */
         break;
 
     case RIG_MODE_PKTUSB:
         mode = RIG_MODE_RTTY; // in "DT0" Subband RIG_MODE_RTTYR = USB and RIG_MODE_RTTY = LSB
-        snprintf(cmd_m, sizeof(cmd_m),
-                 "DT0"); /* DATA A mode - DATA on USB general, VFO dial is suppressed carrier QRG */
+        SNPRINTF(cmd_m, sizeof(cmd_m),
+                 "%s0", dtcmd); /* DATA A mode - DATA on USB general, VFO dial is suppressed carrier QRG */
         break;
 
     case RIG_MODE_RTTY:
         mode = RIG_MODE_RTTY; // in "DT1" Subband RIG_MODE_RTTY = LSB and RIG_MODE_RTTYR = USB
-        snprintf(cmd_m, sizeof(cmd_m),
-                 "DT2"); /* FSK D mode - direct FSK on LSB optimized for RTTY, VFO dial is MARK */
+        SNPRINTF(cmd_m, sizeof(cmd_m),
+                 "%s2", dtcmd); /* FSK D mode - direct FSK on LSB optimized for RTTY, VFO dial is MARK */
         break;
 
     case RIG_MODE_RTTYR:
         mode = RIG_MODE_RTTYR; // in "DT2" Subband RIG_MODE_RTTY = LSB and RIG_MODE_RTTYR = USB
-        snprintf(cmd_m, sizeof(cmd_m),
-                 "DT1"); /* FSK D mode - direct FSK keying, LSB is "normal", VFO dial is MARK */
+        SNPRINTF(cmd_m, sizeof(cmd_m),
+                 "%s1", dtcmd); /* FSK D mode - direct FSK keying, LSB is "normal", VFO dial is MARK */
         break;
 
     case RIG_MODE_PSK:
         mode = RIG_MODE_PSK; // in "DT3" subband RIG_MODE_PSK = USB # kenwood.c mode but may need kenwwod.c mode table review.
-        snprintf(cmd_m, sizeof(cmd_m),
-                 "DT3"); /* PSK D Mode - direct PSK keying, USB is "normal", VFO dial is MARK */
+        SNPRINTF(cmd_m, sizeof(cmd_m),
+                 "%s3", dtcmd); /* PSK D Mode - direct PSK keying, USB is "normal", VFO dial is MARK */
         break;
 
     default:
@@ -1172,15 +1197,21 @@ int k3_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
         c = 'A' + kmode - 10;
     }
 
-    rig_debug(RIG_DEBUG_VERBOSE, "%s: kmode=%d, cmode=%c\n", __func__, kmode, c)
+    rig_debug(RIG_DEBUG_VERBOSE, "%s: kmode=%d, cmode=%c\n", __func__, kmode, c);
 
     if (vfo == RIG_VFO_B)
     {
-        snprintf(buf, sizeof(buf), "MD$%c", c);
+        SNPRINTF(buf, sizeof(buf), "MD$%c", c);
     }
     else
     {
-        snprintf(buf, sizeof(buf), "MD%c", c);
+        SNPRINTF(buf, sizeof(buf), "MD%c", c);
+    }
+
+    if (priv->split)
+    {
+        // then we keep both VFOS in the same mode
+        SNPRINTF(buf, sizeof(buf), "MD%c;MD$%c", c, c);
     }
 
     err = kenwood_transaction(rig, buf, NULL, 0);
@@ -1225,27 +1256,17 @@ int k3_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
         }
 
 #endif
-        // width is 50Hz rounded down
-        struct kenwood_priv_data *priv = rig->state.priv;
-
-        if (priv->is_k4d || priv->is_k4hd)
-        {
-            width += 9;  // rounds to 10Hz
-        }
-        else
-        {
-            width += 49;
-        }
+        width += 9;  // rounds to 10Hz
 
         if (width > 99999) { width = 99999; }
 
         if (vfo == RIG_VFO_B)
         {
-            snprintf(cmd_s, sizeof(cmd_s), "BW$%04ld", width / 10);
+            SNPRINTF(cmd_s, sizeof(cmd_s), "BW$%04ld", width / 10);
         }
         else
         {
-            snprintf(cmd_s, sizeof(cmd_s), "BW%04ld", width / 10);
+            SNPRINTF(cmd_s, sizeof(cmd_s), "BW%04ld", width / 10);
         }
 
         err = kenwood_transaction(rig, cmd_s, NULL, 0);
@@ -1320,19 +1341,19 @@ int k3_set_ext_level(RIG *rig, vfo_t vfo, token_t token, value_t val)
         return kenwood_transaction(rig, "RC", NULL, 0);
 
     case TOK_ESSB:
-        snprintf(buf, sizeof(buf), "ES%c", (val.i == 0) ? '0' : '1');
+        SNPRINTF(buf, sizeof(buf), "ES%c", (val.i == 0) ? '0' : '1');
         return kenwood_transaction(rig, buf, NULL, 0);
 
     case TOK_RX_ANT:
-        snprintf(buf, sizeof(buf), "AR%c", (val.i == 0) ? '0' : '1');
+        SNPRINTF(buf, sizeof(buf), "AR%c", (val.i == 0) ? '0' : '1');
         return kenwood_transaction(rig, buf, NULL, 0);
 
     case TOK_LINK_VFOS:
-        snprintf(buf, sizeof(buf), "LN%c", (val.i == 0) ? '0' : '1');
+        SNPRINTF(buf, sizeof(buf), "LN%c", (val.i == 0) ? '0' : '1');
         return kenwood_transaction(rig, buf, NULL, 0);
 
     case TOK_TX_METER:
-        snprintf(buf, sizeof(buf), "TM%c", val.i + '0');
+        SNPRINTF(buf, sizeof(buf), "TM%c", val.i + '0');
         return kenwood_transaction(rig, buf, NULL, 0);
 
     case TOK_IF_NB:
@@ -1481,7 +1502,7 @@ int k3_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width)
     {
     case RIG_MODE_PKTLSB:
         tx_mode = RIG_MODE_RTTYR; // "DT0" RIG_MODE_RTTY = LSB
-        snprintf(cmd_m, sizeof(cmd_m),
+        SNPRINTF(cmd_m, sizeof(cmd_m),
                  "DT0;"); /* DATA A mode - DATA-R LSB, suppressed carrier */
 
         if (priv->is_k4d || priv->is_k4hd)
@@ -1493,7 +1514,7 @@ int k3_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width)
 
     case RIG_MODE_PKTUSB:
         tx_mode = RIG_MODE_RTTY; // "DT0" RIG_MODE_RTTYR = USB
-        snprintf(cmd_m, sizeof(cmd_m),
+        SNPRINTF(cmd_m, sizeof(cmd_m),
                  "DT0;"); /* DATA A mode - DATA on USB, suppressed carrier */
 
         if (priv->is_k4d || priv->is_k4hd)
@@ -1505,7 +1526,7 @@ int k3_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width)
 
     case RIG_MODE_RTTY:
         tx_mode = RIG_MODE_RTTY; // DT1" RIG_MODE_RTTY = LSB and RIG_MODE_RTTYR = USB
-        snprintf(cmd_m, sizeof(cmd_m),
+        SNPRINTF(cmd_m, sizeof(cmd_m),
                  "DT2;"); /* FSK D mode - direct FSK on LSB optimized for RTTY, VFO dial is MARK */
 
         if (priv->is_k4d || priv->is_k4hd)
@@ -1517,7 +1538,7 @@ int k3_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width)
 
     case RIG_MODE_RTTYR:
         tx_mode = RIG_MODE_RTTYR; // "DT2" RIG_MODE_RTTY = USB and RIG_MODE_RTTYR = USB
-        snprintf(cmd_m, sizeof(cmd_m),
+        SNPRINTF(cmd_m, sizeof(cmd_m),
                  "DT1;"); /* FSK D mode - direct FSK on USB optimized for RTTY, VFO dial is MARK */
 
         if (priv->is_k4d || priv->is_k4hd)
@@ -1529,7 +1550,7 @@ int k3_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width)
 
     case RIG_MODE_PSK:
         tx_mode = RIG_MODE_PSK;
-        snprintf(cmd_m, sizeof(cmd_m),
+        SNPRINTF(cmd_m, sizeof(cmd_m),
                  "DT3;FT1;"); /* PSK D Mode - direct PSK keying, USB is "normal", VFO dial is MARK */
 
         if (priv->is_k4d || priv->is_k4hd)
@@ -1549,7 +1570,7 @@ int k3_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width)
     if (priv->is_k4d || priv->is_k4hd)
     {
         // split can get turned off when modes are changing
-        // so if the rig did this independtly of us we turn it back on
+        // so if the rig did this independently of us we turn it back on
         // even if the rig changes the split status should be the last thing we did
         if (priv->split) { strcat(cmd_m, "FT1;"); }
 
@@ -1579,7 +1600,7 @@ int k3_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width)
         return -RIG_EINVAL;
     }
 
-    sprintf(buf, "MD$%c", '0' + kmode);
+    SNPRINTF(buf, sizeof(buf), "MD$%c", '0' + kmode);
     err = kenwood_transaction(rig, buf, NULL, 0);
 
     if (err != RIG_OK)
@@ -1626,7 +1647,7 @@ int k3_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width)
 
 #endif
 
-        snprintf(cmd_s, sizeof(cmd_s), "BW$%04ld", tx_width / 10);
+        SNPRINTF(cmd_s, sizeof(cmd_s), "BW$%04ld", tx_width / 10);
         err = kenwood_transaction(rig, cmd_s, NULL, 0);
 
         if (err != RIG_OK)
@@ -1752,9 +1773,12 @@ static int k3_get_maxpower(RIG *rig)
     struct kenwood_priv_data *priv = rig->state.priv;
 
     // default range is 0-12 if there is no KPA3 installed
-    if (priv->has_kpa3 || priv->has_kpa100) { maxpower = 110; }
+    if (priv->has_kpa3 || priv->has_kpa100)
+    {
+        maxpower = 110;
+    }
 
-    if (RIG_IS_KX2 || RIG_IS_KX3)
+    else if (RIG_IS_KX2 || RIG_IS_KX3)
     {
 
         int bandnum = -1;
@@ -1836,17 +1860,17 @@ int k3_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
             return -RIG_EINVAL;
         }
 
-        snprintf(levelbuf, sizeof(levelbuf), "GT%03d", kenwood_val);
+        SNPRINTF(levelbuf, sizeof(levelbuf), "GT%03d", kenwood_val);
         break;
 
     case RIG_LEVEL_ATT:
         if (val.i == 0)
         {
-            snprintf(levelbuf, sizeof(levelbuf), "RA00");
+            SNPRINTF(levelbuf, sizeof(levelbuf), "RA00");
         }
         else if (val.i == 10)
         {
-            snprintf(levelbuf, sizeof(levelbuf), "RA01");
+            SNPRINTF(levelbuf, sizeof(levelbuf), "RA01");
         }
         else
         {
@@ -1857,7 +1881,7 @@ int k3_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
             {
                 if (val.i == rig->state.attenuator[i])
                 {
-                    snprintf(levelbuf, sizeof(levelbuf), "RA%02d", i + 1);
+                    SNPRINTF(levelbuf, sizeof(levelbuf), "RA%02d", i + 1);
                     foundit = 1;
                     break;
                 }
@@ -1872,34 +1896,34 @@ int k3_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
         break;
 
     case RIG_LEVEL_MICGAIN:
-        snprintf(levelbuf, sizeof(levelbuf), "MG%03d", (int)(val.f * 60.0f));
+        SNPRINTF(levelbuf, sizeof(levelbuf), "MG%03d", (int)(val.f * 60.0f));
         break;
 
     case RIG_LEVEL_COMP:
-        snprintf(levelbuf, sizeof(levelbuf), "CP%03d", (int)(val.f * 40.0f));
+        SNPRINTF(levelbuf, sizeof(levelbuf), "CP%03d", (int)(val.f * 40.0f));
         break;
 
     case RIG_LEVEL_SQL:
-        snprintf(levelbuf, sizeof(levelbuf), "SQ%03d", (int)(val.f * 29.0f));
+        SNPRINTF(levelbuf, sizeof(levelbuf), "SQ%03d", (int)(val.f * 29.0f));
         break;
 
     case RIG_LEVEL_AF:
-        snprintf(levelbuf, sizeof(levelbuf), "AG%03d", (int)(val.f * 250.0f));
+        SNPRINTF(levelbuf, sizeof(levelbuf), "AG%03d", (int)(val.f * 250.0f));
         break;
 
     case RIG_LEVEL_RF:
-        snprintf(levelbuf, sizeof(levelbuf), "RG%03d", (int)(val.f * 250.0f));
+        SNPRINTF(levelbuf, sizeof(levelbuf), "RG%03d", (int)(val.f * 250.0f));
         break;
 
     case RIG_LEVEL_NR:
         return k3_set_nb_level(rig, val.f, -1);
 
     case RIG_LEVEL_MONITOR_GAIN:
-        snprintf(levelbuf, sizeof(levelbuf), "ML%03d", (int)(val.f * 60.0f));
+        SNPRINTF(levelbuf, sizeof(levelbuf), "ML%03d", (int)(val.f * 60.0f));
         break;
 
     case RIG_LEVEL_RFPOWER:
-        snprintf(levelbuf, sizeof(levelbuf), "PC%03d",
+        SNPRINTF(levelbuf, sizeof(levelbuf), "PC%03d",
                  (int)(val.f * k3_get_maxpower(rig)));
         break;
 
@@ -1918,6 +1942,7 @@ int k3_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
     char levelbuf[16];
     int retval;
     int lvl;
+    size_t len;
     struct kenwood_priv_data *priv = rig->state.priv;
 
     rig_debug(RIG_DEBUG_VERBOSE, "%s called\n", __func__);
@@ -2183,15 +2208,32 @@ int k3_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
         break;
 
     case RIG_LEVEL_RFPOWER:
-        retval = kenwood_safe_transaction(rig, "PC", levelbuf, sizeof(levelbuf), 5);
+        retval = kenwood_transaction(rig, "PC", levelbuf, sizeof(levelbuf));
 
         if (retval != RIG_OK)
         {
             return retval;
         }
 
-        sscanf(levelbuf + 2, "%d", &lvl);
+        len = strlen(levelbuf);
+
+        if (len == 5 || len == 6)
+        {
+            sscanf(levelbuf + 2, "%d", &lvl);
+
+            if (len == 6)
+            {
+                // K2 extended reply
+                lvl /= 10;
+            }
+        }
+        else
+        {
+            return RIG_EPROTO;
+        }
+
         val->f = (float) lvl / k3_get_maxpower(rig);
+
         break;
 
     default:
@@ -2214,17 +2256,17 @@ int kx3_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
     {
     case RIG_LEVEL_RF:
         ival = val.f * (250.0 - 190.0) + 190.0;
-        snprintf(cmdbuf, sizeof(cmdbuf) - 1, "RG%03d", ival);
+        SNPRINTF(cmdbuf, sizeof(cmdbuf) - 1, "RG%03d", ival);
         return kenwood_transaction(rig, cmdbuf, NULL, 0);
 
     case RIG_LEVEL_AF:
         // manual says 0-255 as of Rev G5 but experiment says 0-60
-        snprintf(cmdbuf, sizeof(cmdbuf), "AG%03d", (int)(val.f * 60.0f));
+        SNPRINTF(cmdbuf, sizeof(cmdbuf), "AG%03d", (int)(val.f * 60.0f));
         return kenwood_transaction(rig, cmdbuf, NULL, 0);
 
     case RIG_LEVEL_MICGAIN:
         // manual says 0-255 as of Rev G5 but experiment says 0-80
-        snprintf(cmdbuf, sizeof(cmdbuf), "MG%03d", (int)(val.f * 80.0f));
+        SNPRINTF(cmdbuf, sizeof(cmdbuf), "MG%03d", (int)(val.f * 80.0f));
         return kenwood_transaction(rig, cmdbuf, NULL, 0);
     }
 
@@ -2326,15 +2368,24 @@ int k3_set_func(RIG *rig, vfo_t vfo, setting_t func, int status)
     switch (func)
     {
     case RIG_FUNC_APF:
-        snprintf(buf, sizeof(buf), "AP%c", (status == 0) ? '0' : '1');
+        SNPRINTF(buf, sizeof(buf), "AP%c", (status == 0) ? '0' : '1');
         break;
 
     case RIG_FUNC_DUAL_WATCH:
-        snprintf(buf, sizeof(buf), "SB%c", (status == 0) ? '0' : '1');
+        SNPRINTF(buf, sizeof(buf), "SB%c", (status == 0) ? '0' : '1');
         break;
 
     case RIG_FUNC_DIVERSITY:
-        snprintf(buf, sizeof(buf), "DV%c", (status == 0) ? '0' : '1');
+        SNPRINTF(buf, sizeof(buf), "DV%c", (status == 0) ? '0' : '1');
+        break;
+
+    case RIG_FUNC_SEND_MORSE:
+        // Transmit a CW character - K3 does not return any response
+        snprintf(buf, sizeof(buf), "KYW%c", status);
+        break;
+
+    case RIG_FUNC_MUTE: // K4 Only that we know of
+        SNPRINTF(buf, sizeof(buf), "AG%c", (status == 0) ? '/' : '0');
         break;
 
     default:
@@ -2412,7 +2463,7 @@ int set_rit_xit(RIG *rig, shortfreq_t rit)
         char cmd[16];
         char offs;
         offs = (rit < 0) ? '-' : '+';
-        snprintf(cmd, 8, "RO%c%04d", offs, abs((int)rit));
+        SNPRINTF(cmd, 8, "RO%c%04d", offs, abs((int)rit));
 
         err = kenwood_transaction(rig, cmd, NULL, 0);
 
@@ -2473,7 +2524,7 @@ int k3_set_nb_level(RIG *rig, float dsp_nb, float if_nb)
         }
     }
 
-    snprintf(levelbuf, sizeof(levelbuf), "NL%02d%02d", dsp_nb_raw, if_nb_raw);
+    SNPRINTF(levelbuf, sizeof(levelbuf), "NL%02d%02d", dsp_nb_raw, if_nb_raw);
 
     return kenwood_transaction(rig, levelbuf, NULL, 0);
 }
@@ -2645,3 +2696,98 @@ int kx3_get_bar_graph_level(RIG *rig, float *level)
 
     return RIG_OK;
 }
+
+int k4_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
+{
+    char pttbuf[6];
+    int retval;
+
+    rig_debug(RIG_DEBUG_VERBOSE, "%s called\n", __func__);
+
+    if (!ptt)
+    {
+        return -RIG_EINVAL;
+    }
+
+    retval = kenwood_safe_transaction(rig, "TQ", pttbuf, 6, 3);
+
+    if (retval != RIG_OK)
+    {
+        return retval;
+    }
+
+    *ptt = pttbuf[2] == '1' ? RIG_PTT_ON : RIG_PTT_OFF;
+    // we're not caching this for now
+    return RIG_OK;
+}
+
+
+// The K4 has a problem in Fake It mode where the FA command is ignored
+// We will use it's special TQ command to try and ensure PTT is really off
+int k4_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
+{
+    char pttbuf[6];
+    int i;
+    int retval;
+    ptt_t ptt2 = -1;
+    char cmd[4];
+
+    rig_debug(RIG_DEBUG_VERBOSE, "%s called\n", __func__);
+
+    SNPRINTF(cmd, sizeof(cmd), "RX");
+
+    if (ptt) { cmd[0] = 'T'; }
+
+    retval = kenwood_transaction(rig, cmd, NULL, 0);
+
+    if (retval != RIG_OK)
+    {
+        return retval;
+    }
+
+    for (i = 0; i < 5 && ptt2 != ptt; ++i)
+    {
+        retval = kenwood_safe_transaction(rig, "TQ", pttbuf, 6, 3);
+
+        if (retval != RIG_OK)
+        {
+            return retval;
+        }
+
+        ptt2 = pttbuf[2] == '1' ? RIG_PTT_ON : RIG_PTT_OFF;
+
+        if (ptt2 != ptt)
+        {
+            hl_usleep(100 * 1000);
+            rig_debug(RIG_DEBUG_TRACE, "%s: ptt=%d, expected=%d\n", __func__, ptt2, ptt);
+        }
+    }
+
+    // had one report of Fake It not returning to RX freq after TX -- so a little more time for the K4
+    if (ptt == RIG_PTT_OFF) { hl_usleep(100 * 1000); }
+
+    return RIG_OK;
+}
+
+// K3S band memory needs some time to do it's thing after freq change
+// K3 probably does too
+// But what about the K4?
+int k3_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
+{
+    int retval;
+    freq_t tfreq;
+    retval = kenwood_get_freq(rig, vfo, &tfreq);
+
+    if (retval != RIG_OK) { return retval; }
+
+    retval = kenwood_set_freq(rig, vfo, freq);
+
+    // if more than 1MHz probably a band change so give it some time
+    // before continuing
+    if (abs((long)(tfreq - freq)) > 1e6)
+    {
+        hl_usleep(200 * 1000);    // give 200ms for rig to do band switch if needed
+    }
+
+    return retval;
+}
