@@ -19,12 +19,9 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdlib.h>
-#include <unistd.h>
 #include <math.h>
 
 #include "hamlib/rig.h"
@@ -275,7 +272,7 @@ static int thd74_get_freq_info(RIG *rig, vfo_t vfo, char *buf)
         return retval;
     }
 
-    sprintf(cmd, "FO %c", c);
+    SNPRINTF(cmd, sizeof(cmd), "FO %c", c);
     retval = kenwood_transaction(rig, cmd, buf, 73);
     return retval;
 }
@@ -420,7 +417,7 @@ static int thd74_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
     }
 
     freq = thd74_round_freq(rig, vfo, freq);
-    sprintf(fbuf, "%010"PRIll, (int64_t)freq);
+    SNPRINTF(fbuf, sizeof(fbuf), "%010"PRIll, (int64_t)freq);
     memcpy(buf + 5, fbuf, 10);
     retval = kenwood_simple_transaction(rig, buf, 72);
     return retval;
@@ -511,7 +508,7 @@ int thd74_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
         }
     }
 
-    sprintf(mdbuf, "MD %c,%c", v, kmode);
+    SNPRINTF(mdbuf, sizeof(mdbuf), "MD %c,%c", v, kmode);
     rig_debug(RIG_DEBUG_ERR, "%s: mdbuf: %s\n", __func__, mdbuf);
 
     retval = kenwood_transaction(rig, mdbuf, replybuf, 7);
@@ -608,7 +605,7 @@ static int thd74_set_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t offs)
         return retval;
     }
 
-    sprintf(boff, "%010ld", offs);
+    SNPRINTF(boff, sizeof(boff), "%010ld", offs);
     memcpy(buf + 16, boff, 10);
     retval = kenwood_simple_transaction(rig, buf, 72);
     return retval;
@@ -638,11 +635,23 @@ static int thd74_set_ts(RIG *rig, vfo_t vfo, shortfreq_t ts)
 
     rig_debug(RIG_DEBUG_TRACE, "%s: called\n", __func__);
 
+    for (tsinx = 0; tsinx < 4; tsinx++)
+    {
+        if (thd74tuningstep_fine[tsinx] >= ts)
+        {
+            thd74_set_freq_item(rig, vfo, 33, 1); // Turn fine mode on
+            thd74_set_freq_item(rig, vfo, 35, tsinx);
+            return RIG_OK;
+        }
+    }
+
     for (tsinx = 0; tsinx < 10; tsinx++)
     {
+
         if (thd74tuningstep[tsinx] >= ts)
         {
-            thd74_set_freq_item(rig, vfo, 16, tsinx);
+            thd74_set_freq_item(rig, vfo, 33, 0); //Turn fine mode off
+            thd74_set_freq_item(rig, vfo, 27, tsinx);
             return RIG_OK;
         }
     }
@@ -683,7 +692,7 @@ static int thd74_set_ctcss_tone(RIG *rig, vfo_t vfo, tone_t tone)
     }
 
     buf[22] = (tone == 0) ? '0' : '1';
-    sprintf(tmp, "%02d", tinx);
+    SNPRINTF(tmp, sizeof(tmp), "%02d", tinx);
     memcpy(buf + 30, tmp, 2);
     return kenwood_simple_transaction(rig, buf, 52);
 }
@@ -756,7 +765,7 @@ static int thd74_set_dcs_code(RIG *rig, vfo_t vfo, tone_t code)
     }
 
     buf[26] = (code == 0) ? '0' : '1';
-    sprintf(tmp, "%03d", cinx);
+    SNPRINTF(tmp, sizeof(tmp), "%03d", cinx);
     memcpy(buf + 36, tmp, 3);
     return kenwood_simple_transaction(rig, buf, 52);
 }
@@ -821,7 +830,7 @@ static int thd74_set_ctcss_sql(RIG *rig, vfo_t vfo, tone_t tone)
     }
 
     buf[24] = (tone == 0) ? '0' : '1';
-    sprintf(tmp, "%02d", tinx);
+    SNPRINTF(tmp, sizeof(tmp), "%02d", tinx);
     memcpy(buf + 33, tmp, 2);
     return kenwood_simple_transaction(rig, buf, 52);
 }
@@ -908,11 +917,11 @@ static int thd74_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
         else if (val.f <= 0.4) { lvlc = '1'; }
         else { lvlc = '0'; }
 
-        sprintf(cmd, "PC %c,%c", c, lvlc);
+        SNPRINTF(cmd, sizeof(cmd), "PC %c,%c", c, lvlc);
         return kenwood_simple_transaction(rig, cmd, 6);
 
     case RIG_LEVEL_VOXGAIN:
-        sprintf(cmd, "VG %d", (int)(val.f * 10.0 - 0.5));
+        SNPRINTF(cmd, sizeof(cmd), "VG %d", (int)(val.f * 10.0 - 0.5));
         return kenwood_simple_transaction(rig, cmd, 4);
 
     case RIG_LEVEL_VOXDELAY:
@@ -920,15 +929,15 @@ static int thd74_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
         else if (val.i > 10000) { lvl = val.i / 10000 + 3; }
         else { lvl = val.i / 2500; }
 
-        sprintf(cmd, "VD %d", lvl);
+        SNPRINTF(cmd, sizeof(cmd), "VD %d", lvl);
         return kenwood_simple_transaction(rig, cmd, 4);
 
     case RIG_LEVEL_SQL:
-        sprintf(cmd, "SQ %c,%d", c, (int)val.f);
+        SNPRINTF(cmd, sizeof(cmd), "SQ %c,%d", c, (int)val.f);
         return kenwood_simple_transaction(rig, cmd, 6);
 
     case RIG_LEVEL_ATT: // no value provided to distinguish between on/off??
-        sprintf(cmd, "RA %c,%d", c, (int)val.f);
+        SNPRINTF(cmd, sizeof(cmd), "RA %c,%d", c, (int)val.f);
         return kenwood_simple_transaction(rig, cmd, 6);
 
     default:
@@ -957,7 +966,7 @@ static int thd74_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
     switch (level)
     {
     case RIG_LEVEL_RFPOWER:
-        sprintf(cmd, "PC %c", c);
+        SNPRINTF(cmd, sizeof(cmd), "PC %c", c);
         retval = kenwood_transaction(rig, cmd, buf, sizeof(buf));
 
         if (retval != RIG_OK)
@@ -987,7 +996,7 @@ static int thd74_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
         break;
 
     case RIG_LEVEL_VOXGAIN:
-        sprintf(cmd, "VG");
+        SNPRINTF(cmd, sizeof(cmd), "VG");
         retval = kenwood_transaction(rig, cmd, buf, sizeof(buf));
 
         if (retval != RIG_OK)
@@ -1001,7 +1010,7 @@ static int thd74_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
         break;
 
     case RIG_LEVEL_VOXDELAY:
-        sprintf(cmd, "VD");
+        SNPRINTF(cmd, sizeof(cmd), "VD");
         retval = kenwood_transaction(rig, cmd, buf, sizeof(buf));
 
         if (retval != RIG_OK)
@@ -1015,7 +1024,7 @@ static int thd74_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
         break;
 
     case RIG_LEVEL_SQL:
-        sprintf(cmd, "SQ %c", c);
+        SNPRINTF(cmd, sizeof(cmd), "SQ %c", c);
         retval = kenwood_transaction(rig, cmd, buf, sizeof(buf));
 
         if (retval != RIG_OK)
@@ -1035,7 +1044,7 @@ static int thd74_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
         break;
 
     case RIG_LEVEL_ATT:
-        sprintf(cmd, "RA %c", c);
+        SNPRINTF(cmd, sizeof(cmd), "RA %c", c);
         retval = kenwood_transaction(rig, cmd, buf, 7);
 
         if (retval != RIG_OK)
@@ -1159,7 +1168,7 @@ static int thd74_set_mem(RIG *rig, vfo_t vfo, int ch)
         return retval;
     }
 
-    sprintf(cmd, "MR %c,%03d", c, ch);
+    SNPRINTF(cmd, sizeof(cmd), "MR %c,%03d", c, ch);
     return kenwood_simple_transaction(rig, cmd, 8);
 }
 
@@ -1177,7 +1186,7 @@ static int thd74_get_mem(RIG *rig, vfo_t vfo, int *ch)
         return retval;
     }
 
-    sprintf(cmd, "MR %c", c);
+    SNPRINTF(cmd, sizeof(cmd), "MR %c", c);
     retval = kenwood_transaction(rig, cmd, buf, sizeof(buf));
 
     if (retval != RIG_OK)
@@ -1288,7 +1297,7 @@ static int thd74_get_channel(RIG *rig, vfo_t vfo, channel_t *chan,
     {
         int len;
         char cmd[16];
-        sprintf(cmd, "ME %03d", chan->channel_num);
+        SNPRINTF(cmd, sizeof(cmd), "ME %03d", chan->channel_num);
         retval = kenwood_transaction(rig, cmd, buf, sizeof(buf));
 
         if (retval != RIG_OK)
@@ -1426,7 +1435,7 @@ int thd74_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
         }
 
         tx_freq = thd74_round_freq(rig, RIG_VFO_A, tx_freq);
-        sprintf(fbuf, "%010"PRIll, (int64_t)tx_freq);
+        SNPRINTF(fbuf, sizeof(fbuf), "%010"PRIll, (int64_t)tx_freq);
         memcpy(buf + 5, fbuf, 10);
         return  kenwood_simple_transaction(rig, buf, 72);
     }
@@ -1643,7 +1652,7 @@ const struct rig_caps thd74_caps =
     RIG_MODEL(RIG_MODEL_THD74),
     .model_name = "TH-D74",
     .mfg_name =  "Kenwood",
-    .version =  BACKEND_VER ".2",
+    .version =  BACKEND_VER ".3",
     .copyright =  "LGPL",
     .status =  RIG_STATUS_STABLE,
     .rig_type =  RIG_TYPE_HANDHELD | RIG_FLAG_APRS | RIG_FLAG_TNC | RIG_FLAG_DXCLUSTER,
@@ -1763,4 +1772,5 @@ const struct rig_caps thd74_caps =
 //.get_chan_all_cb = thd74_get_chan_all_cb, this doesn't work yet
 
     .get_info =  th_get_info,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
