@@ -19,14 +19,11 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>  /* String function definitions */
-#include <unistd.h>  /* UNIX standard function definitions */
 
 #include <hamlib/rig.h>
 #include "kenwood.h"
@@ -92,7 +89,7 @@ const struct rig_caps thg71_caps =
     .mfg_name =  "Kenwood",
     .version =  TH_VER ".0",
     .copyright =  "LGPL",
-    .status =  RIG_STATUS_BETA,
+    .status =  RIG_STATUS_STABLE,
     .rig_type =  RIG_TYPE_HANDHELD,
     .ptt_type =  RIG_PTT_RIG,
     .dcd_type =  RIG_DCD_RIG,
@@ -206,6 +203,7 @@ const struct rig_caps thg71_caps =
     .set_ptt = th_set_ptt,
     .get_dcd = th_get_dcd,
     .decode_event =  thg71_decode_event,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 
@@ -411,11 +409,11 @@ int thg71_set_vfo(RIG *rig, vfo_t vfo)
     {
     case RIG_VFO_A:
     case RIG_VFO_VFO:
-        sprintf(vfobuf, "VMC 0,0");
+        SNPRINTF(vfobuf, sizeof(vfobuf), "VMC 0,0");
         break;
 
     case RIG_VFO_MEM:
-        sprintf(vfobuf, "VMC 0,2");
+        SNPRINTF(vfobuf, sizeof(vfobuf), "VMC 0,2");
         break;
 
     default:
