@@ -19,11 +19,8 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <hamlib/config.h>
 
-#include <stdlib.h>
 #include <stdio.h>
 
 #include "hamlib/rig.h"
@@ -44,7 +41,7 @@ int ts590_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val);
 #define TS590_LEVEL_ALL (RIG_LEVEL_RFPOWER|RIG_LEVEL_AF|RIG_LEVEL_RF|\
         RIG_LEVEL_CWPITCH|RIG_LEVEL_METER|RIG_LEVEL_SWR|RIG_LEVEL_ALC|\
         RIG_LEVEL_SQL|RIG_LEVEL_AGC|RIG_LEVEL_RAWSTR|RIG_LEVEL_STRENGTH|\
-        RIG_LEVEL_MICGAIN|RIG_LEVEL_KEYSPD)
+        RIG_LEVEL_MICGAIN|RIG_LEVEL_KEYSPD|RIG_LEVEL_PREAMP|RIG_LEVEL_ATT)
 #define TS590_FUNC_ALL (RIG_FUNC_LOCK|RIG_FUNC_AIP|RIG_FUNC_TONE|\
         RIG_FUNC_NB|RIG_FUNC_COMP|RIG_FUNC_VOX|RIG_FUNC_NR|RIG_FUNC_NR|RIG_FUNC_BC)
 
@@ -92,7 +89,7 @@ const struct rig_caps ts590_caps =
     RIG_MODEL(RIG_MODEL_TS590S),
     .model_name = "TS-590S",
     .mfg_name = "Kenwood",
-    .version = BACKEND_VER ".0",
+    .version = BACKEND_VER ".2",
     .copyright = "LGPL",
     .status = RIG_STATUS_STABLE,
     .rig_type = RIG_TYPE_TRANSCEIVER,
@@ -255,6 +252,7 @@ const struct rig_caps ts590_caps =
     .get_channel =  kenwood_get_channel,
     .vfo_ops = TS590_VFO_OPS,
     .vfo_op =  kenwood_vfo_op,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 const struct rig_caps ts590sg_caps =
@@ -262,7 +260,7 @@ const struct rig_caps ts590sg_caps =
     RIG_MODEL(RIG_MODEL_TS590SG),
     .model_name = "TS-590SG",
     .mfg_name = "Kenwood",
-    .version = BACKEND_VER ".0",
+    .version = BACKEND_VER ".1",
     .copyright = "LGPL",
     .status = RIG_STATUS_STABLE,
     .rig_type = RIG_TYPE_TRANSCEIVER,
@@ -425,6 +423,7 @@ const struct rig_caps ts590sg_caps =
     .get_channel =  kenwood_get_channel,
     .vfo_ops = TS590_VFO_OPS,
     .vfo_op =  kenwood_vfo_op,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 
@@ -471,6 +470,7 @@ int ts590_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
     int retval;
     char lvlbuf[50];
 
+
     rig_debug(RIG_DEBUG_VERBOSE, "%s called\n", __func__);
 
     switch (level)
@@ -570,6 +570,58 @@ int ts590_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
         sscanf(lvlbuf + 3, "%d", &val->i);
         return retval;
 
+    case RIG_LEVEL_PREAMP:
+        retval = kenwood_transaction(rig, "PA", lvlbuf, sizeof(lvlbuf));
+
+        if (retval != RIG_OK)
+        {
+            RETURNFUNC(retval);
+        }
+
+        if (lvlbuf[2] == '0')
+        {
+            val->i = 0;
+        }
+        else if (lvlbuf[2] == '1')
+        {
+            val->i = rig->state.preamp[0];
+        }
+        else
+        {
+            rig_debug(RIG_DEBUG_ERR, "%s: "
+                      "unexpected preamp char '%c'\n",
+                      __func__, lvlbuf[2]);
+            RETURNFUNC(-RIG_EPROTO);
+        }
+
+        return retval;
+
+    case RIG_LEVEL_ATT:
+        retval = kenwood_transaction(rig, "RA", lvlbuf, sizeof(lvlbuf));
+
+        if (retval != RIG_OK)
+        {
+            RETURNFUNC(retval);
+        }
+
+        if (lvlbuf[3] == '0')
+        {
+            val->i = 0;
+        }
+        else if (lvlbuf[3] == '1')
+        {
+            val->i = rig->state.attenuator[0];
+        }
+        else
+        {
+            rig_debug(RIG_DEBUG_ERR, "%s: "
+                      "unexpected att char '%c'\n",
+                      __func__, lvlbuf[2]);
+            RETURNFUNC(-RIG_EPROTO);
+        }
+
+        return retval;
+
     case RIG_LEVEL_RAWSTR:
     case RIG_LEVEL_STRENGTH:
         retval = kenwood_transaction(rig, "SM0", lvlbuf, sizeof(lvlbuf));
