@@ -19,11 +19,8 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <hamlib/config.h>
 
-#include <stdlib.h>
 #include <stdio.h>
 
 #include "hamlib/rig.h"
@@ -71,8 +68,8 @@ static int ts680_set_vfo(RIG *rig, vfo_t vfo)
         return -RIG_EINVAL;
     }
 
-    sprintf(cmdbuf, "FN%c",
-            vfo_function); /* The 680 and 140 need this to set the VFO on the radio */
+    SNPRINTF(cmdbuf, sizeof(cmdbuf), "FN%c",
+             vfo_function); /* The 680 and 140 need this to set the VFO on the radio */
     return kenwood_transaction(rig, cmdbuf, NULL, 0);
 }
 
@@ -88,7 +85,7 @@ const struct rig_caps ts680s_caps =
     .mfg_name =  "Kenwood",
     .version =  BACKEND_VER ".0",
     .copyright =  "LGPL",
-    .status =  RIG_STATUS_BETA,
+    .status =  RIG_STATUS_STABLE,
     .rig_type =  RIG_TYPE_TRANSCEIVER,
     .ptt_type =  RIG_PTT_RIG,
     .port_type =  RIG_PORT_SERIAL,
@@ -196,7 +193,7 @@ const struct rig_caps ts680s_caps =
     .set_mem =  kenwood_set_mem,
     .get_mem = kenwood_get_mem_if,
     .reset =  kenwood_reset,
-
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 /*
