@@ -24,14 +24,11 @@
  * https://sites.google.com/site/rshfiqtransceiver/home/technical-data/interface-commands
  *
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>  /* String function definitions */
-#include <unistd.h>  /* UNIX standard function definitions */
 
 #include "hamlib/rig.h"
 #include "serial.h"
@@ -93,16 +90,18 @@ static int rshfiq_open(RIG *rig)
             && (retval == -RIG_ETIMEOUT); init_retry_count++)
     {
         rig_flush(&rig->state.rigport);
-        snprintf(versionstr, sizeof(versionstr), "*w\r");
+        SNPRINTF(versionstr, sizeof(versionstr), "*w\r");
         rig_debug(RIG_DEBUG_TRACE, "%s: cmdstr = %s\n", __func__, versionstr);
-        retval = write_block(&rig->state.rigport, versionstr, strlen(versionstr));
+        retval = write_block(&rig->state.rigport, (unsigned char *) versionstr,
+                             strlen(versionstr));
 
         if (retval != RIG_OK)
         {
             return retval;
         }
 
-        retval = read_string(&rig->state.rigport, versionstr, 20, stopset, 2);
+        retval = read_string(&rig->state.rigport, (unsigned char *) versionstr, 20,
+                             stopset, 2, 0, 1);
     }
 
     if (retval <= 0)
@@ -153,15 +152,16 @@ static int rshfiq_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
     char cmdstr[15];
     int retval;
 
-    snprintf(fstr, sizeof(fstr), "%lu", (unsigned long int)(freq));
+    SNPRINTF(fstr, sizeof(fstr), "%lu", (unsigned long int)(freq));
     rig_debug(RIG_DEBUG_TRACE, "%s called: %s %s\n", __func__,
               rig_strvfo(vfo), fstr);
 
     rig_flush(&rig->state.rigport);
 
-    snprintf(cmdstr, sizeof(cmdstr), "*f%lu\r", (unsigned long int)(freq));
+    SNPRINTF(cmdstr, sizeof(cmdstr), "*f%lu\r", (unsigned long int)(freq));
 
-    retval = write_block(&rig->state.rigport, cmdstr, strlen(cmdstr));
+    retval = write_block(&rig->state.rigport, (unsigned char *) cmdstr,
+                         strlen(cmdstr));
 
     if (retval != RIG_OK)
     {
@@ -180,18 +180,20 @@ static int rshfiq_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
     stopset[0] = '\r';
     stopset[1] = '\n';
 
-    snprintf(cmdstr, sizeof(cmdstr), "*f?\r");
+    SNPRINTF(cmdstr, sizeof(cmdstr), "*f?\r");
 
     rig_debug(RIG_DEBUG_TRACE, "%s: cmdstr = %s\n", __func__, cmdstr);
 
-    retval = write_block(&rig->state.rigport, cmdstr, strlen(cmdstr));
+    retval = write_block(&rig->state.rigport, (unsigned char *) cmdstr,
+                         strlen(cmdstr));
 
     if (retval != RIG_OK)
     {
         return retval;
     }
 
-    retval = read_string(&rig->state.rigport, cmdstr, 9, stopset, 2);
+    retval = read_string(&rig->state.rigport, (unsigned char *) cmdstr, 9,
+                         stopset, 2, 0, 1);
 
     if (retval <= 0)
     {
@@ -232,7 +234,8 @@ static int rshfiq_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
 
     rig_debug(RIG_DEBUG_TRACE, "%s: cmdstr = %s\n", __func__, cmdstr);
 
-    retval = write_block(&rig->state.rigport, cmdstr, strlen(cmdstr));
+    retval = write_block(&rig->state.rigport, (unsigned char *) cmdstr,
+                         strlen(cmdstr));
     return retval;
 }
 
@@ -262,11 +265,12 @@ static int rshfiq_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 
         rig_flush(&rig->state.rigport);
 
-        snprintf(cmdstr, sizeof(cmdstr), "*L\r");
+        SNPRINTF(cmdstr, sizeof(cmdstr), "*L\r");
 
         rig_debug(RIG_DEBUG_TRACE, "RIG_LEVEL_RFPOWER_METER command=%s\n", cmdstr);
 
-        retval = write_block(&rig->state.rigport, cmdstr, strlen(cmdstr));
+        retval = write_block(&rig->state.rigport, (unsigned char *) cmdstr,
+                             strlen(cmdstr));
 
         if (retval != RIG_OK)
         {
@@ -276,7 +280,8 @@ static int rshfiq_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
         stopset[0] = '\r';
         stopset[1] = '\n';
 
-        retval = read_string(&rig->state.rigport, cmdstr, 9, stopset, 2);
+        retval = read_string(&rig->state.rigport, (unsigned char *) cmdstr, 9,
+                             stopset, 2, 0, 1);
 
         rig_debug(RIG_DEBUG_TRACE, "RIG_LEVEL_RFPOWER_METER reply=%s\n", cmdstr);
 
@@ -300,11 +305,12 @@ static int rshfiq_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 
         rig_flush(&rig->state.rigport);
 
-        snprintf(cmdstr, sizeof(cmdstr), "*T\r");
+        SNPRINTF(cmdstr, sizeof(cmdstr), "*T\r");
 
         rig_debug(RIG_DEBUG_TRACE, "RIG_LEVEL_TEMP_METER command=%s\n", cmdstr);
 
-        retval = write_block(&rig->state.rigport, cmdstr, strlen(cmdstr));
+        retval = write_block(&rig->state.rigport, (unsigned char *) cmdstr,
+                             strlen(cmdstr));
 
         if (retval != RIG_OK)
         {
@@ -314,7 +320,8 @@ static int rshfiq_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
         stopset[0] = '\r';
         stopset[1] = '\n';
 
-        retval = read_string(&rig->state.rigport, cmdstr, 9, stopset, 2);
+        retval = read_string(&rig->state.rigport, (unsigned char *) cmdstr, 9,
+                             stopset, 2, 0, 1);
 
         rig_debug(RIG_DEBUG_TRACE, "RIG_LEVEL_TEMP_METER reply=%s\n", cmdstr);
 
@@ -326,8 +333,9 @@ static int rshfiq_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
         cmdstr[retval] = 0;
 
         sscanf(cmdstr, "%d.", &val->i);
+        val->f = val->i;
 
-        rig_debug(RIG_DEBUG_TRACE, "RIG_LEVEL_TEMP_METER val=%d\n", val->i);
+        rig_debug(RIG_DEBUG_TRACE, "RIG_LEVEL_TEMP_METER val=%g\n", val->f);
 
         return RIG_OK;
         break;
@@ -341,12 +349,19 @@ static int rshfiq_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
     }
 }
 
+static int rshfiq_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
+{
+    *mode = RIG_MODE_IQ;
+    return RIG_OK;
+}
+
+
 const struct rig_caps rshfiq_caps =
 {
     RIG_MODEL(RIG_MODEL_RSHFIQ),
     .model_name =     "RS-HFIQ",
     .mfg_name =       "HobbyPCB",
-    .version =        "20210805.0",
+    .version =        "20220430.0",
     .copyright =      "LGPL",
     .status =         RIG_STATUS_STABLE,
     .rig_type =       RIG_TYPE_TRANSCEIVER,
@@ -366,28 +381,29 @@ const struct rig_caps rshfiq_caps =
 
     .has_get_level = RSHFIQ_LEVEL_ALL,
 
-    .rx_range_list1 =  { {.startf = kHz(3500), .endf = MHz(30), .modes = RIG_MODE_NONE, .low_power = -1, .high_power = -1, RIG_VFO_A},
+    .rx_range_list1 =  { {.startf = kHz(3500), .endf = MHz(30), .modes = RIG_MODE_IQ, .low_power = -1, .high_power = -1, RIG_VFO_A},
         RIG_FRNG_END,
     },
-    .rx_range_list2 =  { {.startf = kHz(3500), .endf = MHz(30), .modes = RIG_MODE_NONE, .low_power = -1, .high_power = -1, RIG_VFO_A},
+    .rx_range_list2 =  { {.startf = kHz(3500), .endf = MHz(30), .modes = RIG_MODE_IQ, .low_power = -1, .high_power = -1, RIG_VFO_A},
         RIG_FRNG_END,
     },
-    .tx_range_list1 =   { {.startf = kHz(3500), .endf = kHz(3800), .modes = RIG_MODE_NONE, .low_power = -1, .high_power = -1, RIG_VFO_A},
-        {.startf = kHz(7000), .endf = kHz(7200), .modes = RIG_MODE_NONE, .low_power = -1, .high_power = -1, RIG_VFO_A},
-        {.startf = kHz(10100), .endf = kHz(10150), .modes = RIG_MODE_NONE, .low_power = -1, .high_power = -1, RIG_VFO_A},
-        {.startf = MHz(14), .endf = kHz(14350), .modes = RIG_MODE_NONE, .low_power = -1, .high_power = -1, RIG_VFO_A},
-        {.startf = MHz(21), .endf = kHz(21450), .modes = RIG_MODE_NONE, .low_power = -1, .high_power = -1, RIG_VFO_A},
-        {.startf = kHz(24890), .endf = kHz(24990), .modes = RIG_MODE_NONE, .low_power = -1, .high_power = -1, RIG_VFO_A},
-        {.startf = MHz(28), .endf = kHz(29700), .modes = RIG_MODE_NONE, .low_power = -1, .high_power = -1, RIG_VFO_A},
+    .tx_range_list1 =   { {.startf = kHz(3500), .endf = kHz(3800), .modes = RIG_MODE_IQ, .low_power = -1, .high_power = -1, RIG_VFO_A},
+        {.startf = kHz(7000), .endf = kHz(7200), .modes = RIG_MODE_IQ, .low_power = -1, .high_power = -1, RIG_VFO_A},
+        {.startf = kHz(10100), .endf = kHz(10150), .modes = RIG_MODE_IQ, .low_power = -1, .high_power = -1, RIG_VFO_A},
+        {.startf = MHz(14), .endf = kHz(14350), .modes = RIG_MODE_IQ, .low_power = -1, .high_power = -1, RIG_VFO_A},
+        {.startf = MHz(21), .endf = kHz(21450), .modes = RIG_MODE_IQ, .low_power = -1, .high_power = -1, RIG_VFO_A},
+        {.startf = kHz(24890), .endf = kHz(24990), .modes = RIG_MODE_IQ, .low_power = -1, .high_power = -1, RIG_VFO_A},
+        {.startf = MHz(28), .endf = kHz(29700), .modes = RIG_MODE_IQ, .low_power = -1, .high_power = -1, RIG_VFO_A},
         RIG_FRNG_END,
     },
-    .tuning_steps =  { {RIG_MODE_NONE, Hz(1)}, RIG_TS_END, },
+    .tuning_steps =  { {RIG_MODE_IQ, Hz(1)}, RIG_TS_END, },
 
     .rig_open =     rshfiq_open,
     .get_freq =     rshfiq_get_freq,
     .set_freq =     rshfiq_set_freq,
     .set_ptt  =     rshfiq_set_ptt,
     .get_level =     rshfiq_get_level,
-
+    .get_mode =     rshfiq_get_mode,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
