@@ -166,7 +166,7 @@ const struct rig_caps g313_caps =
     .mfg_name =       "Winradio",
     .version =        "20191204.0",
     .copyright =        "LGPL", /* This wrapper, not the G313 DLL */
-    .status =         RIG_STATUS_BETA,
+    .status =         RIG_STATUS_STABLE,
     .rig_type =       RIG_TYPE_PCRECEIVER,
     .port_type =      RIG_PORT_NONE,
     .targetable_vfo =    0,
@@ -225,6 +225,7 @@ const struct rig_caps g313_caps =
     .get_level =     g313_get_level,
 
     .get_info =      g313_get_info,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 
@@ -233,7 +234,7 @@ int g313_init(RIG *rig)
 {
     struct g313_priv_data *priv;
 
-    rig->state.priv = (struct g313_priv_data *)malloc(sizeof(
+    rig->state.priv = (struct g313_priv_data *)calloc(1, sizeof(
                           struct g313_priv_data));
 
     if (!rig->state.priv)
@@ -696,14 +697,14 @@ int g313_set_conf(RIG *rig, token_t token, const char *val)
     return RIG_OK;
 }
 
-int g313_get_conf(RIG *rig, token_t token, char *val)
+int g313_get_conf2(RIG *rig, token_t token, char *val, int val_len)
 {
     struct g313_priv_data *priv = (struct g313_priv_data *)rig->state.priv;
 
     switch (token)
     {
     case WAVEOUT_SOUNDCARDID:
-        sprintf(val, "%d", priv->WaveOutDeviceID);
+        SNPRINTF(val, val_len, "%d", priv->WaveOutDeviceID);
         break;
 
     default:
@@ -713,4 +714,9 @@ int g313_get_conf(RIG *rig, token_t token, char *val)
     return RIG_OK;
 }
 
+int g313_get_conf(RIG *rig, token_t token, char *val)
+{
+    return g313_get_conf2(rig, token, val, 128);
+}
+
 #endif /* _WIN32 */
