@@ -22,12 +22,10 @@
 
 #include "winradio.h"   /* config.h */
 
-#include <stdlib.h>
 #include <string.h>  /* String function definitions */
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
-#include <math.h>
 
 #include "hamlib/rig.h"
 #include "serial.h"
