@@ -22,13 +22,10 @@
  */
 
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdlib.h>
 #include <string.h>  /* String function definitions */
-#include <unistd.h>  /* UNIX standard function definitions */
 
 #include "hamlib/rig.h"
 #include "serial.h"
@@ -178,7 +175,7 @@ const struct rig_caps frg100_caps =
     .has_get_func =       RIG_FUNC_LOCK,
     .has_set_func =       RIG_FUNC_LOCK,
     .has_get_level =      RIG_LEVEL_RAWSTR,
-    .has_set_level =      RIG_LEVEL_NONE,
+    .has_set_level =      RIG_LEVEL_BAND_SELECT,
     .has_get_parm =       RIG_PARM_NONE,
     .has_set_parm =       RIG_PARM_BACKLIGHT,
     .vfo_ops =        RIG_OP_FROM_VFO | RIG_OP_TO_VFO | RIG_OP_UP | RIG_OP_DOWN,
@@ -242,6 +239,7 @@ const struct rig_caps frg100_caps =
     .get_level =      frg100_get_level,
 
     .set_powerstat =  frg100_set_powerstat,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 static inline int frg100_channel_is_ok(unsigned char channel)
@@ -268,11 +266,11 @@ static int frg100_do_transaction(RIG *rig,
     memset(retbuf, 0, retbuf_len);
 
     rig_flush(&rs->rigport);
-    retval = write_block(&rs->rigport, (const char *)cmd, YAESU_CMD_LENGTH);
+    retval = write_block(&rs->rigport, cmd, YAESU_CMD_LENGTH);
 
     if (retval != RIG_OK) { return retval; }
 
-    retval = read_block(&rs->rigport, (char *)retbuf, retbuf_len);
+    retval = read_block(&rs->rigport, retbuf, retbuf_len);
 
     if (retval != retbuf_len)
     {
@@ -354,7 +352,7 @@ int frg100_open(RIG *rig)
     rig_debug(RIG_DEBUG_TRACE, "%s: called\n", __func__);
 
     /* send 0 delay pacing */
-    return write_block(&rig->state.rigport, (char *) cmd, YAESU_CMD_LENGTH);
+    return write_block(&rig->state.rigport, cmd, YAESU_CMD_LENGTH);
 
 }
 
@@ -373,7 +371,7 @@ int frg100_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
     to_bcd(cmd, freq / 10, 8);
 
     /* Frequency set */
-    return write_block(&rig->state.rigport, (char *) cmd, YAESU_CMD_LENGTH);
+    return write_block(&rig->state.rigport, cmd, YAESU_CMD_LENGTH);
 }
 
 
@@ -385,7 +383,7 @@ int frg100_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
     /* fill in p1 */
     cmd[3] = mode2rig(rig, mode, width);
 
-    return write_block(&rig->state.rigport, (char *) cmd, YAESU_CMD_LENGTH);
+    return write_block(&rig->state.rigport, cmd, YAESU_CMD_LENGTH);
 }
 
 
@@ -401,7 +399,7 @@ int frg100_set_powerstat(RIG *rig, powerstat_t status)
     cmd[3] = status == RIG_POWER_OFF ? 0x00 : 0x01;
 
     /* Frequency set */
-    return write_block(&rig->state.rigport, (char *) cmd, YAESU_CMD_LENGTH);
+    return write_block(&rig->state.rigport, cmd, YAESU_CMD_LENGTH);
 }
 
 
@@ -429,7 +427,7 @@ int frg100_set_vfo(RIG *rig, vfo_t vfo)
         return -RIG_EINVAL;     /* sorry, wrong VFO */
     }
 
-    return write_block(&rig->state.rigport, (char *) cmd, YAESU_CMD_LENGTH);
+    return write_block(&rig->state.rigport, cmd, YAESU_CMD_LENGTH);
 }
 
 
@@ -446,7 +444,7 @@ int frg100_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
     rig_flush(&rig->state.rigport);
 
     /* send READ STATUS(Meter only) cmd to rig  */
-    retval = write_block(&rig->state.rigport, (char *) cmd, YAESU_CMD_LENGTH);
+    retval = write_block(&rig->state.rigport, cmd, YAESU_CMD_LENGTH);
 
     if (retval < 0)
     {
@@ -454,7 +452,7 @@ int frg100_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
     }
 
     /* read back the 1 byte */
-    retval = read_block(&rig->state.rigport, (char *) cmd, 5);
+    retval = read_block(&rig->state.rigport, cmd, 5);
 
     if (retval < 1)
     {
