@@ -24,14 +24,11 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>     /* String function definitions */
-#include <unistd.h>     /* UNIX standard function definitions */
 #include <math.h>
 
 #include "hamlib/rig.h"
@@ -41,6 +38,143 @@
 #include "misc.h"
 #include "bandplan.h"
 
+enum ft100_native_cmd_e
+{
+
+    FT100_NATIVE_CAT_LOCK_ON = 0,
+    FT100_NATIVE_CAT_LOCK_OFF,
+    FT100_NATIVE_CAT_PTT_ON,
+    FT100_NATIVE_CAT_PTT_OFF,
+    FT100_NATIVE_CAT_SET_FREQ,
+    FT100_NATIVE_CAT_SET_MODE_LSB,
+    FT100_NATIVE_CAT_SET_MODE_USB,
+    FT100_NATIVE_CAT_SET_MODE_CW,
+    FT100_NATIVE_CAT_SET_MODE_CWR,
+    FT100_NATIVE_CAT_SET_MODE_AM,
+    FT100_NATIVE_CAT_SET_MODE_FM,
+    FT100_NATIVE_CAT_SET_MODE_DIG,
+    FT100_NATIVE_CAT_SET_MODE_WFM,
+    FT100_NATIVE_CAT_CLAR_ON,
+    FT100_NATIVE_CAT_CLAR_OFF,
+    FT100_NATIVE_CAT_SET_CLAR_FREQ,
+    FT100_NATIVE_CAT_SET_VFOAB,
+    FT100_NATIVE_CAT_SET_VFOA,
+    FT100_NATIVE_CAT_SET_VFOB,
+    FT100_NATIVE_CAT_SPLIT_ON,
+    FT100_NATIVE_CAT_SPLIT_OFF,
+    FT100_NATIVE_CAT_SET_RPT_SHIFT_MINUS,
+    FT100_NATIVE_CAT_SET_RPT_SHIFT_PLUS,
+    FT100_NATIVE_CAT_SET_RPT_SHIFT_SIMPLEX,
+    FT100_NATIVE_CAT_SET_RPT_OFFSET,
+    /* fix me */
+    FT100_NATIVE_CAT_SET_DCS_ON,
+    FT100_NATIVE_CAT_SET_CTCSS_ENC_ON,
+    FT100_NATIVE_CAT_SET_CTCSS_ENC_DEC_ON,
+    FT100_NATIVE_CAT_SET_CTCSS_DCS_OFF,
+    /* em xif */
+    FT100_NATIVE_CAT_SET_CTCSS_FREQ,
+    FT100_NATIVE_CAT_SET_DCS_CODE,
+    FT100_NATIVE_CAT_GET_RX_STATUS,
+    FT100_NATIVE_CAT_GET_TX_STATUS,
+    FT100_NATIVE_CAT_GET_FREQ_MODE_STATUS,
+    FT100_NATIVE_CAT_PWR_WAKE,
+    FT100_NATIVE_CAT_PWR_ON,
+    FT100_NATIVE_CAT_PWR_OFF,
+    FT100_NATIVE_CAT_READ_STATUS,
+    FT100_NATIVE_CAT_READ_METERS,
+    FT100_NATIVE_CAT_READ_FLAGS,
+    FT100_NATIVE_SIZE     /* end marker */
+};
+
+/*
+ *  we are able to get way more info
+ *  than we can set
+ *
+ */
+typedef struct
+{
+    unsigned char band_no;
+    unsigned char freq[4];
+    unsigned char mode;
+    unsigned char ctcss;
+    unsigned char dcs;
+    unsigned char flag1;
+    unsigned char flag2;
+    unsigned char clarifier[2];
+    unsigned char not_used;
+    unsigned char step1;
+    unsigned char step2;
+    unsigned char filter;
+
+    unsigned char stuffing[16];
+}
+FT100_STATUS_INFO;
+
+
+typedef struct
+{
+    unsigned char mic_switch_1;
+    unsigned char tx_fwd_power;
+    unsigned char tx_rev_power;
+    unsigned char s_meter;
+    unsigned char mic_level;
+    unsigned char squelch_level;
+    unsigned char mic_switch_2;
+    unsigned char final_temp;
+    unsigned char alc_level;
+}
+FT100_METER_INFO;
+
+typedef struct
+{
+    unsigned char byte[8];
+}
+FT100_FLAG_INFO;
+
+
+static int ft100_init(RIG *rig);
+static int ft100_open(RIG *rig);
+static int ft100_cleanup(RIG *rig);
+static int ft100_close(RIG *rig);
+
+static int ft100_set_freq(RIG *rig, vfo_t vfo, freq_t freq);
+static int ft100_get_freq(RIG *rig, vfo_t vfo, freq_t *freq);
+
+static int ft100_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width);
+static int ft100_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width);
+
+static int ft100_set_vfo(RIG *rig, vfo_t vfo);
+static int ft100_get_vfo(RIG *rig, vfo_t *vfo);
+
+static int ft100_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt);
+static int ft100_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt);
+static int ft100_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val);
+#if 0
+static int ft100_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val);
+
+static int ft100_set_func(RIG *rig, vfo_t vfo, setting_t func, int status);
+static int ft100_get_func(RIG *rig, vfo_t vfo, setting_t func, int *status);
+
+static int ft100_set_parm(RIG *rig, setting_t parm, value_t val);
+static int ft100_get_parm(RIG *rig, setting_t parm, value_t *val);
+#endif
+
+static int ft100_set_split_vfo(RIG *rig, vfo_t vfo, split_t split,
+                               vfo_t tx_vfo);
+static int ft100_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split,
+                               vfo_t *tx_vfo);
+
+static int ft100_set_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t shift);
+static int ft100_get_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t *shift);
+
+static int ft100_set_dcs_code(RIG *rig, vfo_t vfo, tone_t code);
+static int ft100_get_dcs_code(RIG *rig, vfo_t vfo, tone_t *code);
+
+static int ft100_set_ctcss_tone(RIG *rig, vfo_t vfo, tone_t tone);
+static int ft100_get_ctcss_tone(RIG *rig, vfo_t vfo, tone_t *tone);
+
+//static int ft100_get_info(RIG *rig, FT100_STATUS_INFO *ft100_status, FT100_METER_INFO *ft100_meter, FT100_FLAG_INFO *ft100_flags);
+
 struct ft100_priv_data
 {
     /* TODO: make use of cached data */
@@ -182,7 +316,7 @@ const struct rig_caps ft100_caps =
     RIG_MODEL(RIG_MODEL_FT100),
     .model_name =     "FT-100",
     .mfg_name =       "Yaesu",
-    .version =        "20210110.0",
+    .version =        "20210929.0",
     .copyright =      "LGPL",
     .status =         RIG_STATUS_STABLE,
     .rig_type =       RIG_TYPE_TRANSCEIVER,
@@ -202,7 +336,7 @@ const struct rig_caps ft100_caps =
     .has_get_func =   RIG_FUNC_NONE,
     .has_set_func =   FT100_FUNC_ALL,
     .has_get_level =  FT100_GET_RIG_LEVELS,
-    .has_set_level =  RIG_LEVEL_NONE,
+    .has_set_level =  RIG_LEVEL_BAND_SELECT,
     .has_get_parm =   RIG_PARM_NONE,
     .has_set_parm =   RIG_PARM_NONE,  /* FIXME: parms */
     .level_gran =     {},     /* granularity */
@@ -328,6 +462,7 @@ const struct rig_caps ft100_caps =
     .get_func =       NULL,
     .set_parm =       NULL,
     .get_parm =       NULL,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 
@@ -386,19 +521,12 @@ int ft100_close(RIG *rig)
 
 static int ft100_send_priv_cmd(RIG *rig, unsigned char cmd_index)
 {
-
-    struct rig_state *rig_s;
-    unsigned char *cmd;       /* points to sequence to send */
-
     rig_debug(RIG_DEBUG_VERBOSE, "%s called (%d)\n", __func__, cmd_index);
 
     if (!rig) { return -RIG_EINVAL; }
 
-    rig_s = &rig->state;
-
-    cmd = (unsigned char *) &ncmd[cmd_index].nseq; /* get native sequence */
-
-    return write_block(&rig_s->rigport, (char *) cmd, YAESU_CMD_LENGTH);
+    return write_block(&rig->state.rigport, (unsigned char *) &ncmd[cmd_index].nseq,
+                       YAESU_CMD_LENGTH);
 }
 
 static int ft100_read_status(RIG *rig)
@@ -419,8 +547,7 @@ static int ft100_read_status(RIG *rig)
         return ret;
     }
 
-    ret = read_block(&rig->state.rigport,
-                     (char *)&priv->status,
+    ret = read_block(&rig->state.rigport, (unsigned char *) &priv->status,
                      sizeof(FT100_STATUS_INFO));
     rig_debug(RIG_DEBUG_VERBOSE, "%s: read status=%i \n", __func__, ret);
 
@@ -448,8 +575,7 @@ static int ft100_read_flags(RIG *rig)
         return ret;
     }
 
-    ret = read_block(&rig->state.rigport,
-                     (char *)&priv->flags,
+    ret = read_block(&rig->state.rigport, (unsigned char *) &priv->flags,
                      sizeof(FT100_FLAG_INFO));
     rig_debug(RIG_DEBUG_VERBOSE, "%s: read flags=%i \n", __func__, ret);
 
@@ -463,14 +589,11 @@ static int ft100_read_flags(RIG *rig)
 
 int ft100_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 {
-    struct rig_state *rig_s;
     unsigned char p_cmd[YAESU_CMD_LENGTH];
     unsigned char cmd_index;  /* index of sequence to send */
 
     if (!rig) { return -RIG_EINVAL; }
 
-    rig_s = &rig->state;
-
     rig_debug(RIG_DEBUG_VERBOSE, "ft100: requested freq = %"PRIfreq" Hz \n", freq);
 
     cmd_index = FT100_NATIVE_CAT_SET_FREQ;
@@ -478,10 +601,10 @@ int ft100_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
     memcpy(p_cmd, &ncmd[cmd_index].nseq, YAESU_CMD_LENGTH);
 
     /* fixed 10Hz bug by OH2MMY */
-    freq = (int)freq / 10;
+    freq = (int) freq / 10;
     to_bcd(p_cmd, freq, 8); /* store bcd format in in p_cmd */
 
-    return write_block(&rig_s->rigport, (char *) p_cmd, YAESU_CMD_LENGTH);
+    return write_block(&rig->state.rigport, p_cmd, YAESU_CMD_LENGTH);
 }
 
 int ft100_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
@@ -512,11 +635,11 @@ int ft100_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 
     /* now convert it .... */
 
-    sprintf(freq_str, "%02X%02X%02X%02X",
-            priv->status.freq[0],
-            priv->status.freq[1],
-            priv->status.freq[2],
-            priv->status.freq[3]);
+    SNPRINTF(freq_str, sizeof(freq_str), "%02X%02X%02X%02X",
+             priv->status.freq[0],
+             priv->status.freq[1],
+             priv->status.freq[2],
+             priv->status.freq[3]);
 
     d1 = strtol(freq_str, NULL, 16);
     d2 = (d1 * 1.25);    /* fixed 10Hz bug by OH2MMY */
@@ -607,7 +730,7 @@ int ft100_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
         else if (width <= 2400) { p_cmd[3] = 0x00; }
         else { p_cmd[3] = 0x01; }
 
-        ret = write_block(&rig->state.rigport, (char *) p_cmd, YAESU_CMD_LENGTH);
+        ret = write_block(&rig->state.rigport, p_cmd, YAESU_CMD_LENGTH);
 
         if (ret != RIG_OK)
         {
@@ -716,7 +839,7 @@ int ft100_set_vfo(RIG *rig, vfo_t vfo)
 
     int ret;
 
-    rig_debug(RIG_DEBUG_VERBOSE, "%s called\n", __func__);
+    rig_debug(RIG_DEBUG_VERBOSE, "%s called vfo=%s\n", __func__, rig_strvfo(vfo));
 
     switch (vfo)
     {
@@ -788,6 +911,7 @@ int ft100_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
 {
 
     unsigned char cmd_index;
+    int split = rig->state.cache.split;
 
     rig_debug(RIG_DEBUG_VERBOSE, "%s called\n", __func__);
 
@@ -795,10 +919,18 @@ int ft100_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
     {
     case RIG_PTT_ON:
         cmd_index = FT100_NATIVE_CAT_PTT_ON;
+
+        if (split) { rig_set_vfo(rig, RIG_VFO_B); }
+
         break;
 
     case RIG_PTT_OFF:
         cmd_index = FT100_NATIVE_CAT_PTT_OFF;
+
+        if (split) { rig_set_vfo(rig, RIG_VFO_A); }
+
+        hl_usleep(100 *
+                  1000); // give ptt some time to do its thing -- fake it was not resetting freq after tx
         break;
 
     default:
@@ -831,15 +963,13 @@ int ft100_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
 /*
  * Rem: The FT-100(D) has no set_level ability
  */
-
-/*
- * blind implementation of get_level.
- * Please test on real hardware and send report on hamlib mailing list
- */
 int ft100_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 {
 
     int ret;
+    int split = rig->state.cache.split;
+    int ptt = rig->state.cache.ptt;
+
     FT100_METER_INFO ft100_meter;
 
     if (!rig) { return -RIG_EINVAL; }
@@ -848,14 +978,19 @@ int ft100_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 
     rig_debug(RIG_DEBUG_VERBOSE, "%s: %s\n", __func__, rig_strlevel(level));
 
+    // if in split have to switch to VFOB to read power and back to VFOA
+    if (split && ptt) { rig_set_vfo(rig, RIG_VFO_B); }
+
     ret = ft100_send_priv_cmd(rig, FT100_NATIVE_CAT_READ_METERS);
 
+    if (split && ptt) { rig_set_vfo(rig, RIG_VFO_A); }
+
     if (ret != RIG_OK)
     {
         return ret;
     }
 
-    ret = read_block(&rig->state.rigport, (char *)&ft100_meter,
+    ret = read_block(&rig->state.rigport, (unsigned char *) &ft100_meter,
                      sizeof(FT100_METER_INFO));
     rig_debug(RIG_DEBUG_VERBOSE, "%s: read meters=%d\n", __func__, ret);
 
@@ -1045,13 +1180,10 @@ int ft100_get_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t *shift)
  */
 int ft100_set_dcs_code(RIG *rig, vfo_t vfo, tone_t code)
 {
-    struct rig_state *rig_s;
     unsigned char p_cmd[YAESU_CMD_LENGTH];
     unsigned char cmd_index;  /* index of sequence to send */
     int pcode;
 
-    rig_s = &rig->state;
-
     for (pcode = 0; pcode < 104 && ft100_dcs_list[pcode] != 0; pcode++)
     {
         if (ft100_dcs_list[pcode] == code)
@@ -1075,7 +1207,7 @@ int ft100_set_dcs_code(RIG *rig, vfo_t vfo, tone_t code)
 
     p_cmd[3] = (char)pcode;
 
-    return write_block(&rig_s->rigport, (char *) p_cmd, YAESU_CMD_LENGTH);
+    return write_block(&rig->state.rigport, p_cmd, YAESU_CMD_LENGTH);
 }
 
 int ft100_get_dcs_code(RIG *rig, vfo_t vfo, tone_t *code)
@@ -1102,7 +1234,6 @@ int ft100_get_dcs_code(RIG *rig, vfo_t vfo, tone_t *code)
  */
 int ft100_set_ctcss_tone(RIG *rig, vfo_t vfo, tone_t tone)
 {
-    struct rig_state *rig_s;
     unsigned char p_cmd[YAESU_CMD_LENGTH];
     unsigned char cmd_index;  /* index of sequence to send */
     int ptone;
@@ -1121,8 +1252,6 @@ int ft100_set_ctcss_tone(RIG *rig, vfo_t vfo, tone_t tone)
         return -RIG_EINVAL;
     }
 
-    rig_s = &rig->state;
-
     rig_debug(RIG_DEBUG_VERBOSE, "%s = %.1f Hz, n=%d\n", __func__,
               (float)tone / 10, ptone);
 
@@ -1132,7 +1261,7 @@ int ft100_set_ctcss_tone(RIG *rig, vfo_t vfo, tone_t tone)
 
     p_cmd[3] = (char)ptone;
 
-    return write_block(&rig_s->rigport, (char *) p_cmd, YAESU_CMD_LENGTH);
+    return write_block(&rig->state.rigport, p_cmd, YAESU_CMD_LENGTH);
 }
 
 int ft100_get_ctcss_tone(RIG *rig, vfo_t vfo, tone_t *tone)
