@@ -35,13 +35,10 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdlib.h>
 #include <string.h>     /* String function definitions */
-#include <unistd.h>     /* UNIX standard function definitions */
 
 #include <hamlib/rig.h>
 #include "serial.h"
@@ -70,6 +67,7 @@ static int ft757_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode,
 
 static int ft757_set_vfo(RIG *rig, vfo_t vfo); /* select vfo */
 static int ft757_get_vfo(RIG *rig, vfo_t *vfo); /* get vfo */
+static int ft757gx_get_vfo(RIG *rig, vfo_t *vfo); /* get vfo */
 
 static int ft757_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt);
 static int ft757_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val);
@@ -88,7 +86,6 @@ static int rig2mode(RIG *rig, int md, rmode_t *mode, pbwidth_t *width);
 struct ft757_priv_data
 {
     unsigned char pacing;       /* pacing value */
-    unsigned int read_update_delay; /* depends on pacing value */
     unsigned char
     current_vfo;  /* active VFO from last cmd , can be either RIG_VFO_A or RIG_VFO_B only */
     unsigned char
@@ -118,7 +115,7 @@ const struct rig_caps ft757gx_caps =
     RIG_MODEL(RIG_MODEL_FT757),
     .model_name =       "FT-757GX",
     .mfg_name =     "Yaesu",
-    .version =      "20200325.0",
+    .version =      "20220429.0",
     .copyright =        "LGPL",
     .status =       RIG_STATUS_STABLE,
     .rig_type =     RIG_TYPE_MOBILE,
@@ -137,8 +134,8 @@ const struct rig_caps ft757gx_caps =
     .retry =        10,
     .has_get_func =     RIG_FUNC_NONE,
     .has_set_func =     RIG_FUNC_NONE,
-    .has_get_level =    RIG_LEVEL_NONE,
-    .has_set_level =    RIG_LEVEL_NONE,
+    .has_get_level =    RIG_LEVEL_BAND_SELECT,
+    .has_set_level =    RIG_LEVEL_BAND_SELECT,
     .has_get_parm =     RIG_PARM_NONE,
     .has_set_parm =     RIG_PARM_NONE,
     .ctcss_list =       NULL,
@@ -210,6 +207,7 @@ const struct rig_caps ft757gx_caps =
     .get_freq =     ft757gx_get_freq,   /* get freq */
     .set_mode =     NULL,           /* set mode */
     .get_mode =     NULL,           /* get mode */
+    .get_vfo =      ft757gx_get_vfo,    /* set vfo */
     .set_vfo =      ft757_set_vfo,      /* set vfo */
 
     .set_ptt =      NULL,           /* set ptt */
@@ -217,7 +215,8 @@ const struct rig_caps ft757gx_caps =
 
     .cfgparams =        ft757gx_cfg_params,
     .set_conf =     ft757gx_set_conf,
-    .get_conf =     ft757gx_get_conf
+    .get_conf =     ft757gx_get_conf,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 /*
@@ -249,7 +248,7 @@ const struct rig_caps ft757gx2_caps =
     .has_get_func =     RIG_FUNC_LOCK,
     .has_set_func =     RIG_FUNC_LOCK,
     .has_get_level =    RIG_LEVEL_RAWSTR,
-    .has_set_level =    RIG_LEVEL_NONE,
+    .has_set_level =    RIG_LEVEL_BAND_SELECT,
     .has_get_parm =     RIG_PARM_NONE,
     .has_set_parm =     RIG_PARM_NONE,
     .vfo_ops =      RIG_OP_CPY | RIG_OP_FROM_VFO | RIG_OP_TO_VFO |
@@ -340,6 +339,7 @@ const struct rig_caps ft757gx2_caps =
     .get_vfo =      ft757_get_vfo,  /* get vfo */
     .get_level =        ft757_get_level,
     .get_ptt =      ft757_get_ptt,  /* get ptt */
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 
@@ -375,8 +375,6 @@ static int ft757_init(RIG *rig)
 
     priv->pacing =
         FT757GX_PACING_DEFAULT_VALUE;   /* set pacing to minimum for now */
-    priv->read_update_delay =
-        FT757GX_DEFAULT_READ_TIMEOUT;    /* set update timeout to safe value */
     priv->current_vfo =  RIG_VFO_A;            /* default to VFO_A ? */
 
     return RIG_OK;
@@ -409,14 +407,24 @@ static int ft757_cleanup(RIG *rig)
 
 static int ft757_open(RIG *rig)
 {
+    int retval;
     struct ft757_priv_data *priv = (struct ft757_priv_data *)rig->state.priv;
 
     rig_debug(RIG_DEBUG_VERBOSE, "%s called.\n", __func__);
 
+    priv->fakefreq = 1; // turn this on by default
+
     /* FT757GX has a write-only serial port: don't try to read status data */
     if (rig->caps->rig_model == RIG_MODEL_FT757)
     {
         memset(priv->update_data, 0, FT757GX_STATUS_UPDATE_DATA_LENGTH);
+        retval = rig_set_vfo(rig, RIG_VFO_A);
+
+        if (retval != RIG_OK)
+        {
+            rig_debug(RIG_DEBUG_ERR, "%s: rig_set_vfo error: %s\n", __func__,
+                      rigerror(retval));
+        }
     }
     else
     {
@@ -451,7 +459,7 @@ static int ft757_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
     to_bcd(cmd, freq / 10, BCD_LEN);
 
     priv->curfreq = freq;
-    return write_block(&rig->state.rigport, (char *)cmd, YAESU_CMD_LENGTH);
+    return write_block(&rig->state.rigport, cmd, YAESU_CMD_LENGTH);
 }
 
 
@@ -477,7 +485,7 @@ static int ft757_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
     /* fill in p1 */
     cmd[3] = mode2rig(rig, mode, width);
 
-    return write_block(&rig->state.rigport, (char *)cmd, YAESU_CMD_LENGTH);
+    return write_block(&rig->state.rigport, cmd, YAESU_CMD_LENGTH);
 }
 
 
@@ -485,6 +493,9 @@ static int ft757gx_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 {
     struct ft757_priv_data *priv = (struct ft757_priv_data *)rig->state.priv;
 
+    rig_debug(RIG_DEBUG_VERBOSE, "%s called. fakefreq=%d\n", __func__,
+              priv->fakefreq);
+
     if (priv->fakefreq)   // only return last freq set when fakeit is turned on
     {
         *freq = priv->curfreq;
@@ -584,12 +595,12 @@ static int ft757_set_vfo(RIG *rig, vfo_t vfo)
     unsigned char cmd[YAESU_CMD_LENGTH] = { 0x00, 0x00, 0x00, 0x00, 0x05};
     struct ft757_priv_data *priv = (struct ft757_priv_data *)rig->state.priv;
 
-    rig_debug(RIG_DEBUG_VERBOSE, "%s called.\n", __func__);
+    ENTERFUNC;
 
     switch (vfo)
     {
     case RIG_VFO_CURR:
-        return RIG_OK;
+        RETURNFUNC(RIG_OK);
 
     case RIG_VFO_A:
         cmd[3] = 0x00;          /* VFO A */
@@ -600,14 +611,21 @@ static int ft757_set_vfo(RIG *rig, vfo_t vfo)
         break;
 
     default:
-        return -RIG_EINVAL;     /* sorry, wrong VFO */
+        RETURNFUNC(-RIG_EINVAL);     /* sorry, wrong VFO */
     }
 
     priv->current_vfo = vfo;
 
-    return write_block(&rig->state.rigport, (char *)cmd, YAESU_CMD_LENGTH);
+    RETURNFUNC(write_block(&rig->state.rigport, cmd, YAESU_CMD_LENGTH));
 }
 
+static int ft757gx_get_vfo(RIG *rig, vfo_t *vfo)
+{
+    struct ft757_priv_data *priv = (struct ft757_priv_data *)rig->state.priv;
+    // we'll just use the cached vfo for the 757GX since we can't read it
+    *vfo = priv->current_vfo;
+    return RIG_OK;
+}
 
 static int ft757_get_vfo(RIG *rig, vfo_t *vfo)
 {
@@ -679,7 +697,7 @@ static int ft757_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
     rig_flush(&rig->state.rigport);
 
     /* send READ STATUS(Meter only) cmd to rig  */
-    retval = write_block(&rig->state.rigport, (char *)cmd, YAESU_CMD_LENGTH);
+    retval = write_block(&rig->state.rigport, cmd, YAESU_CMD_LENGTH);
 
     if (retval < 0)
     {
@@ -687,7 +705,7 @@ static int ft757_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
     }
 
     /* read back the 1 byte */
-    retval = read_block(&rig->state.rigport, (char *)cmd, 1);
+    retval = read_block(&rig->state.rigport, cmd, 1);
 
     if (retval != 1)
     {
@@ -729,7 +747,7 @@ static int ft757_get_update_data(RIG *rig)
         rig_flush(&rig->state.rigport);
 
         /* send READ STATUS cmd to rig  */
-        retval = write_block(&rig->state.rigport, (char *)cmd, YAESU_CMD_LENGTH);
+        retval = write_block(&rig->state.rigport, cmd, YAESU_CMD_LENGTH);
 
         if (retval < 0)
         {
@@ -738,7 +756,7 @@ static int ft757_get_update_data(RIG *rig)
 
         /* read back the 75 status bytes */
         retval = read_block(&rig->state.rigport,
-                            (char *)priv->update_data,
+                            priv->update_data,
                             FT757GX_STATUS_UPDATE_DATA_LENGTH);
 
         if (retval == FT757GX_STATUS_UPDATE_DATA_LENGTH)
@@ -874,7 +892,7 @@ static int rig2mode(RIG *rig, int md, rmode_t *mode, pbwidth_t *width)
 /*
  * Assumes rig!=NULL, rig->state.priv!=NULL
  */
-static int ft757gx_get_conf(RIG *rig, token_t token, char *val)
+static int ft757gx_get_conf2(RIG *rig, token_t token, char *val, int val_len)
 {
     struct ft757_priv_data *priv;
     struct rig_state *rs;
@@ -889,7 +907,7 @@ static int ft757gx_get_conf(RIG *rig, token_t token, char *val)
     switch (token)
     {
     case TOK_FAKEFREQ:
-        sprintf(val, "%d", priv->fakefreq);
+        SNPRINTF(val, val_len, "%d", priv->fakefreq);
         break;
 
     default:
@@ -899,6 +917,12 @@ static int ft757gx_get_conf(RIG *rig, token_t token, char *val)
 
     return RIG_OK;
 }
+
+static int ft757gx_get_conf(RIG *rig, token_t token, char *val)
+{
+    return ft757gx_get_conf2(rig, token, val, 128);
+}
+
 /*
  * Assumes rig!=NULL, rig->state.priv!=NULL
  */
