@@ -31,9 +31,6 @@
 /*
  * Unimplemented features supported by the FT-817:
  *
- *   - RIT ON/OFF without touching the RIT offset. This would
- *     need frontend support (eg. a new RIG_FUNC_xxx)
- *
  *   - RX status command returns info that is not used:
  *      - discriminator centered (yes/no flag)
  *      - received ctcss/dcs matched (yes/no flag)                     TBC
@@ -47,13 +44,10 @@
  * - the many "fixme" stuff around
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdlib.h>
 #include <string.h>     /* String function definitions */
-#include <unistd.h>     /* UNIX standard function definitions */
 #include <stdbool.h>
 
 #ifdef HAVE_SYS_TIME_H
@@ -132,7 +126,7 @@ struct ft817_priv_data
     /* freq & mode status */
     struct timeval fm_status_tv;
     unsigned char fm_status[5]; /* 5 bytes, NOT related to YAESU_CMD_LENGTH */
-    /* Digi mode is not part of regular fm_status reponse.
+    /* Digi mode is not part of regular fm_status response.
      * So keep track of it in a separate variable. */
     unsigned char dig_mode;
 };
@@ -300,7 +294,7 @@ const struct rig_caps ft817_caps =
     RIG_MODEL(RIG_MODEL_FT817),
     .model_name =          "FT-817",
     .mfg_name =            "Yaesu",
-    .version =             "20210916.0",
+    .version =             "20220419.0",
     .copyright =           "LGPL",
     .status =              RIG_STATUS_STABLE,
     .rig_type =            RIG_TYPE_TRANSCEIVER,
@@ -318,11 +312,11 @@ const struct rig_caps ft817_caps =
     .timeout =             FT817_TIMEOUT,
     .retry =               5,
     .has_get_func =        RIG_FUNC_NONE,
-    .has_set_func =        RIG_FUNC_LOCK | RIG_FUNC_TONE | RIG_FUNC_TSQL,
+    .has_set_func =        RIG_FUNC_LOCK | RIG_FUNC_TONE | RIG_FUNC_TSQL | RIG_FUNC_CSQL | RIG_FUNC_RIT,
     .has_get_level =
     RIG_LEVEL_STRENGTH | RIG_LEVEL_RAWSTR | RIG_LEVEL_RFPOWER |
     RIG_LEVEL_ALC | RIG_LEVEL_SWR,
-    .has_set_level =       RIG_LEVEL_NONE,
+    .has_set_level =       RIG_LEVEL_BAND_SELECT,
     .has_get_parm =        RIG_PARM_NONE,
     .has_set_parm =        RIG_PARM_NONE,
     .level_gran =          {},                     /* granularity */
@@ -439,6 +433,7 @@ const struct rig_caps ft817_caps =
     .get_level =        ft817_get_level,
     .set_func =         ft817_set_func,
     .vfo_op =           ft817_vfo_op,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 const struct rig_caps ft818_caps =
@@ -446,7 +441,7 @@ const struct rig_caps ft818_caps =
     RIG_MODEL(RIG_MODEL_FT818),
     .model_name =          "FT-818",
     .mfg_name =            "Yaesu",
-    .version =             "20200710.0",
+    .version =             "20220419.0",
     .copyright =           "LGPL",
     .status =              RIG_STATUS_STABLE,
     .rig_type =            RIG_TYPE_TRANSCEIVER,
@@ -464,11 +459,11 @@ const struct rig_caps ft818_caps =
     .timeout =             FT817_TIMEOUT,
     .retry =               5,
     .has_get_func =        RIG_FUNC_NONE,
-    .has_set_func =        RIG_FUNC_LOCK | RIG_FUNC_TONE | RIG_FUNC_TSQL,
+    .has_set_func =        RIG_FUNC_LOCK | RIG_FUNC_TONE | RIG_FUNC_TSQL | RIG_FUNC_RIT,
     .has_get_level =
     RIG_LEVEL_STRENGTH | RIG_LEVEL_RAWSTR | RIG_LEVEL_RFPOWER |
     RIG_LEVEL_ALC | RIG_LEVEL_SWR,
-    .has_set_level =       RIG_LEVEL_NONE,
+    .has_set_level =       RIG_LEVEL_BAND_SELECT,
     .has_get_parm =        RIG_PARM_NONE,
     .has_set_parm =        RIG_PARM_NONE,
     .level_gran =          {},                     /* granularity */
@@ -592,6 +587,7 @@ const struct rig_caps ft818_caps =
     .get_level =        ft817_get_level,
     .set_func =         ft817_set_func,
     .vfo_op =           ft817_vfo_op,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 /* ---------------------------------------------------------------------- */
@@ -683,9 +679,9 @@ static int ft817_read_eeprom(RIG *rig, unsigned short addr, unsigned char *out)
     data[0] = addr >> 8;
     data[1] = addr & 0xfe;
 
-    write_block(&rig->state.rigport, (char *) data, YAESU_CMD_LENGTH);
+    write_block(&rig->state.rigport, data, YAESU_CMD_LENGTH);
 
-    if ((n = read_block(&rig->state.rigport, (char *) data, 2)) < 0)
+    if ((n = read_block(&rig->state.rigport, data, 2)) < 0)
     {
         return n;
     }
@@ -748,9 +744,9 @@ static int ft817_get_status(RIG *rig, int status)
     do
     {
         rig_flush(&rig->state.rigport);
-        write_block(&rig->state.rigport, (char *) ncmd[status].nseq,
+        write_block(&rig->state.rigport, ncmd[status].nseq,
                     YAESU_CMD_LENGTH);
-        n = read_block(&rig->state.rigport, (char *) data, len);
+        n = read_block(&rig->state.rigport, data, len);
     }
     while (retries-- && n < 0);
 
@@ -824,7 +820,15 @@ static int ft817_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
     int retries = rig->state.rigport.retry +
                   1; // +1 because, because 2 steps are needed even in best scenario
 
-    rig_debug(RIG_DEBUG_VERBOSE, "%s: called\n", __func__);
+    rig_debug(RIG_DEBUG_VERBOSE, "%s: called, vfo=%s, ptt=%d, split=%d\n", __func__,
+              rig_strvfo(vfo), rig->state.cache.ptt, rig->state.cache.split);
+
+    // we can't query VFOB while in transmit and split mode
+    if (rig->state.cache.ptt && vfo == RIG_VFO_B && rig->state.cache.split)
+    {
+        *freq = rig->state.cache.freqMainB;
+        return RIG_OK;
+    }
 
     while ((f1 == 0 || f1 != f2) && retries-- > 0)
     {
@@ -1313,7 +1317,7 @@ static int ft818_get_ant(RIG *rig, vfo_t vfo, ant_t ant, value_t *option,
 
 int ft817_read_ack(RIG *rig)
 {
-    char dummy;
+    unsigned char dummy;
     rig_debug(RIG_DEBUG_VERBOSE, "%s: called\n", __func__);
 
     if (rig->state.rigport.post_write_delay == 0)
@@ -1354,7 +1358,7 @@ static int ft817_send_cmd(RIG *rig, int index)
     }
 
     rig_flush(&rig->state.rigport);
-    write_block(&rig->state.rigport, (char *) ncmd[index].nseq, YAESU_CMD_LENGTH);
+    write_block(&rig->state.rigport, ncmd[index].nseq, YAESU_CMD_LENGTH);
     return ft817_read_ack(rig);
 }
 
@@ -1376,7 +1380,7 @@ static int ft817_send_icmd(RIG *rig, int index, unsigned char *data)
     cmd[YAESU_CMD_LENGTH - 1] = ncmd[index].nseq[YAESU_CMD_LENGTH - 1];
     memcpy(cmd, data, YAESU_CMD_LENGTH - 1);
 
-    write_block(&rig->state.rigport, (char *) cmd, YAESU_CMD_LENGTH);
+    write_block(&rig->state.rigport, cmd, YAESU_CMD_LENGTH);
     return ft817_read_ack(rig);
 }
 
@@ -1615,6 +1619,26 @@ static int ft817_set_func(RIG *rig, vfo_t vfo, setting_t func, int status)
             return ft817_send_cmd(rig, FT817_NATIVE_CAT_SET_CTCSS_DCS_OFF);
         }
 
+    case RIG_FUNC_CSQL:
+        if (status)
+        {
+            return ft817_send_cmd(rig, FT817_NATIVE_CAT_SET_DCS_ON);
+        }
+        else
+        {
+            return ft817_send_cmd(rig, FT817_NATIVE_CAT_SET_CTCSS_DCS_OFF);
+        }
+
+    case RIG_FUNC_RIT:
+        if (status)
+        {
+            return ft817_send_cmd(rig, FT817_NATIVE_CAT_CLAR_ON);
+        }
+        else
+        {
+            return ft817_send_cmd(rig, FT817_NATIVE_CAT_CLAR_OFF);
+        }
+
     default:
         return -RIG_EINVAL;
     }
@@ -1775,6 +1799,8 @@ static int ft817_set_rit(RIG *rig, vfo_t vfo, shortfreq_t rit)
     }
 
     /* the rig rejects if these are repeated - don't confuse user with retcode */
+
+    /* not used anymore, RIG_FUNC_RIT implemented
     if (rit == 0)
     {
         ft817_send_cmd(rig, FT817_NATIVE_CAT_CLAR_OFF);
@@ -1782,7 +1808,7 @@ static int ft817_set_rit(RIG *rig, vfo_t vfo, shortfreq_t rit)
     else
     {
         ft817_send_cmd(rig, FT817_NATIVE_CAT_CLAR_ON);
-    }
+    } */
 
     return RIG_OK;
 }
@@ -1800,9 +1826,9 @@ int ft817_set_powerstat(RIG *rig, powerstat_t status)
     case RIG_POWER_ON:
         // send 5 bytes first, snooze a bit, then PWR_ON
         write_block(&rig->state.rigport,
-                    (char *) ncmd[FT817_NATIVE_CAT_PWR_WAKE].nseq, YAESU_CMD_LENGTH);
+                    ncmd[FT817_NATIVE_CAT_PWR_WAKE].nseq, YAESU_CMD_LENGTH);
         hl_usleep(200 * 1000);
-        write_block(&rig->state.rigport, (char *) ncmd[FT817_NATIVE_CAT_PWR_ON].nseq,
+        write_block(&rig->state.rigport, ncmd[FT817_NATIVE_CAT_PWR_ON].nseq,
                     YAESU_CMD_LENGTH);
         return RIG_OK;
 
@@ -1864,6 +1890,8 @@ static int ft817_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t tx_vfo)
         return n;
     }
 
+    rig->state.cache.split = split;
+
     return RIG_OK;
 
 }
