@@ -25,13 +25,10 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdlib.h>
 #include <string.h>  /* String function definitions */
-#include <unistd.h>  /* UNIX standard function definitions */
 
 #include "hamlib/rig.h"
 #include "bandplan.h"
@@ -40,6 +37,39 @@
 #include "yaesu.h"
 #include "ft840.h"
 
+/*
+ * Native FT840 functions. More to come :-)
+ *
+ */
+
+enum ft840_native_cmd_e
+{
+    FT840_NATIVE_SPLIT_OFF = 0,
+    FT840_NATIVE_SPLIT_ON,
+    FT840_NATIVE_RECALL_MEM,
+    FT840_NATIVE_VFO_TO_MEM,
+    FT840_NATIVE_VFO_A,
+    FT840_NATIVE_VFO_B,
+    FT840_NATIVE_MEM_TO_VFO,
+    FT840_NATIVE_CLARIFIER_OPS,
+    FT840_NATIVE_FREQ_SET,
+    FT840_NATIVE_MODE_SET,
+    FT840_NATIVE_PACING,
+    FT840_NATIVE_PTT_OFF,
+    FT840_NATIVE_PTT_ON,
+    FT840_NATIVE_MEM_CHNL,
+    FT840_NATIVE_OP_DATA,
+    FT840_NATIVE_VFO_DATA,
+    FT840_NATIVE_MEM_CHNL_DATA,
+    FT840_NATIVE_TUNER_OFF,
+    FT840_NATIVE_TUNER_ON,
+    FT840_NATIVE_TUNER_START,
+    FT840_NATIVE_READ_METER,
+    FT840_NATIVE_READ_FLAGS,
+    FT840_NATIVE_SIZE             /* end marker, value indicates number of */
+    /* native cmd entries */
+};
+
 /*
  *
  * The FT-840 backend is cloned after the FT-890 backend.
@@ -197,11 +227,9 @@ static const yaesu_cmd_set_t ncmd[] =
 struct ft840_priv_data
 {
     unsigned char pacing;                     /* pacing value */
-    unsigned int read_update_delay;           /* depends on pacing value */
     vfo_t current_vfo;                        /* active VFO from last cmd */
     unsigned char
     p_cmd[YAESU_CMD_LENGTH];    /* private copy of 1 constructed CAT cmd */
-    yaesu_cmd_set_t pcs[FT840_NATIVE_SIZE];   /* private cmd set */
     unsigned char
     update_data[FT840_ALL_DATA_LENGTH]; /* returned data--max value, some are less */
     unsigned char current_mem;                   /* private memory channel number */
@@ -238,7 +266,7 @@ const struct rig_caps ft840_caps =
     .has_get_func =       RIG_FUNC_TUNER,
     .has_set_func =       RIG_FUNC_TUNER,
     .has_get_level =      RIG_LEVEL_STRENGTH,
-    .has_set_level =      RIG_LEVEL_NONE,
+    .has_set_level =      RIG_LEVEL_BAND_SELECT,
     .has_get_parm =       RIG_PARM_NONE,
     .has_set_parm =       RIG_PARM_NONE,
     .ctcss_list =         NULL,
@@ -329,6 +357,7 @@ const struct rig_caps ft840_caps =
     .set_func =           ft840_set_func,
     .get_level =          ft840_get_level,
     .vfo_op =             ft840_vfo_op,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 
@@ -366,15 +395,8 @@ static int ft840_init(RIG *rig)
 
     priv = rig->state.priv;
 
-    /*
-     * Copy native cmd set to private cmd storage area
-     */
-    memcpy(priv->pcs, ncmd, sizeof(ncmd));
-
     /* TODO: read pacing from preferences */
     priv->pacing = FT840_PACING_DEFAULT_VALUE; /* set pacing to minimum for now */
-    priv->read_update_delay =
-        FT840_DEFAULT_READ_TIMEOUT; /* set update timeout to safe value */
     priv->current_vfo =  RIG_VFO_MAIN;  /* default to whatever */
 
     return RIG_OK;
@@ -1703,7 +1725,6 @@ static int ft840_vfo_op(RIG *rig, vfo_t vfo, vfo_op_t op)
 
 static int ft840_get_update_data(RIG *rig, unsigned char ci, unsigned char rl)
 {
-    struct rig_state *rig_s;
     struct ft840_priv_data *priv;
     int n, err;                       /* for read_  */
 
@@ -1715,7 +1736,6 @@ static int ft840_get_update_data(RIG *rig, unsigned char ci, unsigned char rl)
     }
 
     priv = (struct ft840_priv_data *)rig->state.priv;
-    rig_s = &rig->state;
 
     err = ft840_send_static_cmd(rig, ci);
 
@@ -1724,7 +1744,7 @@ static int ft840_get_update_data(RIG *rig, unsigned char ci, unsigned char rl)
         return err;
     }
 
-    n = read_block(&rig_s->rigport, (char *) priv->update_data, rl);
+    n = read_block(&rig->state.rigport, priv->update_data, rl);
 
     if (n < 0)
     {
@@ -1743,7 +1763,7 @@ static int ft840_get_update_data(RIG *rig, unsigned char ci, unsigned char rl)
  * TODO: place variant of this in yaesu.c
  *
  * Arguments:   *rig    Valid RIG instance
- *              ci      Command index of the pcs struct
+ *              ci      Command index of the ncmd table
  *
  * Returns:     RIG_OK if all called functions are successful,
  *              otherwise returns error from called functiion
@@ -1751,8 +1771,6 @@ static int ft840_get_update_data(RIG *rig, unsigned char ci, unsigned char rl)
 
 static int ft840_send_static_cmd(RIG *rig, unsigned char ci)
 {
-    struct rig_state *rig_s;
-    struct ft840_priv_data *priv;
     int err;
 
     rig_debug(RIG_DEBUG_VERBOSE, "%s called\n", __func__);
@@ -1762,17 +1780,14 @@ static int ft840_send_static_cmd(RIG *rig, unsigned char ci)
         return -RIG_EINVAL;
     }
 
-    priv = (struct ft840_priv_data *)rig->state.priv;
-    rig_s = &rig->state;
-
-    if (!priv->pcs[ci].ncomp)
+    if (!ncmd[ci].ncomp)
     {
         rig_debug(RIG_DEBUG_TRACE,
                   "%s: Attempt to send incomplete sequence\n", __func__);
         return -RIG_EINVAL;
     }
 
-    err = write_block(&rig_s->rigport, (char *) priv->pcs[ci].nseq,
+    err = write_block(&rig->state.rigport, ncmd[ci].nseq,
                       YAESU_CMD_LENGTH);
 
     if (err != RIG_OK)
@@ -1791,7 +1806,7 @@ static int ft840_send_static_cmd(RIG *rig, unsigned char ci)
  * TODO: place variant of this in yaesu.c
  *
  * Arguments:   *rig    Valid RIG instance
- *              ci      Command index of the pcs struct
+ *              ci      Command index of the ncmd table
  *              p1-p4   Command parameters
  *
  * Returns:     RIG_OK if all called functions are successful,
@@ -1802,7 +1817,6 @@ static int ft840_send_dynamic_cmd(RIG *rig, unsigned char ci,
                                   unsigned char p1, unsigned char p2,
                                   unsigned char p3, unsigned char p4)
 {
-    struct rig_state *rig_s;
     struct ft840_priv_data *priv;
     int err;
 
@@ -1820,14 +1834,13 @@ static int ft840_send_dynamic_cmd(RIG *rig, unsigned char ci,
 
     priv = (struct ft840_priv_data *)rig->state.priv;
 
-    if (priv->pcs[ci].ncomp)
+    if (ncmd[ci].ncomp)
     {
         rig_debug(RIG_DEBUG_TRACE,
                   "%s: Attempt to modify complete sequence\n", __func__);
         return -RIG_EINVAL;
     }
 
-    rig_s = &rig->state;
     memcpy(&priv->p_cmd, &ncmd[ci].nseq, YAESU_CMD_LENGTH);
 
     priv->p_cmd[P1] = p1;         /* ick */
@@ -1835,7 +1848,7 @@ static int ft840_send_dynamic_cmd(RIG *rig, unsigned char ci,
     priv->p_cmd[P3] = p3;
     priv->p_cmd[P4] = p4;
 
-    err = write_block(&rig_s->rigport, (char *) &priv->p_cmd,
+    err = write_block(&rig->state.rigport, (unsigned char *) &priv->p_cmd,
                       YAESU_CMD_LENGTH);
 
     if (err != RIG_OK)
@@ -1854,7 +1867,7 @@ static int ft840_send_dynamic_cmd(RIG *rig, unsigned char ci,
  * TODO: place variant of this in yaesu.c
  *
  * Arguments:   *rig    Valid RIG instance
- *              ci      Command index of the pcs struct
+ *              ci      Command index of the ncmd table
  *              freq    freq_t frequency value
  *
  * Returns:     RIG_OK if all called functions are successful,
@@ -1863,7 +1876,6 @@ static int ft840_send_dynamic_cmd(RIG *rig, unsigned char ci,
 
 static int ft840_send_dial_freq(RIG *rig, unsigned char ci, freq_t freq)
 {
-    struct rig_state *rig_s;
     struct ft840_priv_data *priv;
     int err;
     // cppcheck-suppress *
@@ -1881,15 +1893,13 @@ static int ft840_send_dial_freq(RIG *rig, unsigned char ci, freq_t freq)
 
     priv = (struct ft840_priv_data *)rig->state.priv;
 
-    if (priv->pcs[ci].ncomp)
+    if (ncmd[ci].ncomp)
     {
         rig_debug(RIG_DEBUG_TRACE,
                   "%s: Attempt to modify complete sequence\n", __func__);
         return -RIG_EINVAL;
     }
 
-    rig_s = &rig->state;
-
     /* Copy native cmd freq_set to private cmd storage area */
     memcpy(&priv->p_cmd, &ncmd[ci].nseq, YAESU_CMD_LENGTH);
 
@@ -1899,7 +1909,7 @@ static int ft840_send_dial_freq(RIG *rig, unsigned char ci, freq_t freq)
     rig_debug(RIG_DEBUG_TRACE, fmt, __func__, (int64_t)from_bcd(priv->p_cmd,
               FT840_BCD_DIAL) * 10);
 
-    err = write_block(&rig_s->rigport, (char *) &priv->p_cmd,
+    err = write_block(&rig->state.rigport, (unsigned char *) &priv->p_cmd,
                       YAESU_CMD_LENGTH);
 
     if (err != RIG_OK)
@@ -1918,7 +1928,7 @@ static int ft840_send_dial_freq(RIG *rig, unsigned char ci, freq_t freq)
  * TODO: place variant of this in yaesu.c
  *
  * Arguments:   *rig    Valid RIG instance
- *              ci      Command index of the pcs struct
+ *              ci      Command index of the ncmd table
  *              rit     shortfreq_t frequency value
  *              p1      P1 value -- CLAR_SET_FREQ
  *              p2      P2 value -- CLAR_OFFSET_PLUS || CLAR_OFFSET_MINUS
@@ -1932,7 +1942,6 @@ static int ft840_send_dial_freq(RIG *rig, unsigned char ci, freq_t freq)
 #ifdef USE_FT840_SET_RIT
 static int ft840_send_rit_freq(RIG *rig, unsigned char ci, shortfreq_t rit)
 {
-    struct rig_state *rig_s;
     struct ft840_priv_data *priv;
     unsigned char p1;
     unsigned char p2;
@@ -1950,15 +1959,13 @@ static int ft840_send_rit_freq(RIG *rig, unsigned char ci, shortfreq_t rit)
 
     priv = (struct ft840_priv_data *)rig->state.priv;
 
-    if (priv->pcs[ci].ncomp)
+    if (ncmd[ci].ncomp)
     {
         rig_debug(RIG_DEBUG_TRACE,
                   "%s: Attempt to modify complete sequence\n", __func__);
         return -RIG_EINVAL;
     }
 
-    rig_s = &rig->state;
-
     p1 = CLAR_SET_FREQ;
 
     if (rit < 0)
@@ -1984,7 +1991,7 @@ static int ft840_send_rit_freq(RIG *rig, unsigned char ci, shortfreq_t rit)
     priv->p_cmd[P1] = p1;         /* ick */
     priv->p_cmd[P2] = p2;
 
-    err = write_block(&rig_s->rigport, (char *) &priv->p_cmd,
+    err = write_block(&rig->state.rigport, (char *) &priv->p_cmd,
                       YAESU_CMD_LENGTH);
 
     if (err != RIG_OK)
