@@ -44,13 +44,10 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdlib.h>
 #include <string.h>  /* String function definitions */
-#include <unistd.h>  /* UNIX standard function definitions */
 
 #include "hamlib/rig.h"
 #include "serial.h"
@@ -60,6 +57,153 @@
 #include "bandplan.h"
 #include "tones.h"
 
+/*
+ * Native FT847 functions. This is what I have to work with :-)
+ *
+ */
+
+enum ft847_native_cmd_e
+{
+
+    /* Set commands to the rig */
+
+    FT_847_NATIVE_CAT_ON = 0,
+    FT_847_NATIVE_CAT_OFF,
+
+    FT_847_NATIVE_CAT_PTT_ON,
+    FT_847_NATIVE_CAT_PTT_OFF,
+
+    FT_847_NATIVE_CAT_SAT_MODE_ON,
+    FT_847_NATIVE_CAT_SAT_MODE_OFF,
+
+    FT_847_NATIVE_CAT_SET_FREQ_MAIN,
+    FT_847_NATIVE_CAT_SET_FREQ_SAT_RX_VFO,
+    FT_847_NATIVE_CAT_SET_FREQ_SAT_TX_VFO,
+
+    FT_847_NATIVE_CAT_SET_MODE_MAIN_LSB, /* MAIN VFO */
+    FT_847_NATIVE_CAT_SET_MODE_MAIN_USB,
+    FT_847_NATIVE_CAT_SET_MODE_MAIN_CW,
+    FT_847_NATIVE_CAT_SET_MODE_MAIN_CWR,
+    FT_847_NATIVE_CAT_SET_MODE_MAIN_AM,
+    FT_847_NATIVE_CAT_SET_MODE_MAIN_FM,
+    FT_847_NATIVE_CAT_SET_MODE_MAIN_CWN,
+    FT_847_NATIVE_CAT_SET_MODE_MAIN_CWRN,
+    FT_847_NATIVE_CAT_SET_MODE_MAIN_AMN,
+    FT_847_NATIVE_CAT_SET_MODE_MAIN_FMN,
+
+    FT_847_NATIVE_CAT_SET_MODE_SAT_RX_LSB, /* SAT RX VFO */
+    FT_847_NATIVE_CAT_SET_MODE_SAT_RX_USB,
+    FT_847_NATIVE_CAT_SET_MODE_SAT_RX_CW,
+    FT_847_NATIVE_CAT_SET_MODE_SAT_RX_CWR,
+    FT_847_NATIVE_CAT_SET_MODE_SAT_RX_AM,
+    FT_847_NATIVE_CAT_SET_MODE_SAT_RX_FM,
+    FT_847_NATIVE_CAT_SET_MODE_SAT_RX_CWN,
+    FT_847_NATIVE_CAT_SET_MODE_SAT_RX_CWRN,
+    FT_847_NATIVE_CAT_SET_MODE_SAT_RX_AMN,
+    FT_847_NATIVE_CAT_SET_MODE_SAT_RX_FMN,
+
+    FT_847_NATIVE_CAT_SET_MODE_SAT_TX_LSB, /* SAT TX VFO */
+    FT_847_NATIVE_CAT_SET_MODE_SAT_TX_USB,
+    FT_847_NATIVE_CAT_SET_MODE_SAT_TX_CW,
+    FT_847_NATIVE_CAT_SET_MODE_SAT_TX_CWR,
+    FT_847_NATIVE_CAT_SET_MODE_SAT_TX_AM,
+    FT_847_NATIVE_CAT_SET_MODE_SAT_TX_FM,
+    FT_847_NATIVE_CAT_SET_MODE_SAT_TX_CWN,
+    FT_847_NATIVE_CAT_SET_MODE_SAT_TX_CWRN,
+    FT_847_NATIVE_CAT_SET_MODE_SAT_TX_AMN,
+    FT_847_NATIVE_CAT_SET_MODE_SAT_TX_FMN,
+
+    FT_847_NATIVE_CAT_SET_DCS_ON_MAIN, /* MAIN CTCSS/DCS */
+    FT_847_NATIVE_CAT_SET_CTCSS_ENC_DEC_ON_MAIN,
+    FT_847_NATIVE_CAT_SET_CTCSS_ENC_ON_MAIN,
+    FT_847_NATIVE_CAT_SET_CTCSS_DCS_OFF_MAIN,
+
+    FT_847_NATIVE_CAT_SET_DCS_ON_SAT_RX, /* SAT RX CTCSS/DCS */
+    FT_847_NATIVE_CAT_SET_CTCSS_ENC_DEC_ON_SAT_RX,
+    FT_847_NATIVE_CAT_SET_CTCSS_ENC_ON_SAT_RX,
+    FT_847_NATIVE_CAT_SET_CTCSS_DCS_OFF_SAT_RX,
+
+    FT_847_NATIVE_CAT_SET_DCS_ON_SAT_TX, /* SAT TX CTCSS/DCS */
+    FT_847_NATIVE_CAT_SET_CTCSS_ENC_DEC_ON_SAT_TX,
+    FT_847_NATIVE_CAT_SET_CTCSS_ENC_ON_SAT_TX,
+    FT_847_NATIVE_CAT_SET_CTCSS_DCS_OFF_SAT_TX,
+
+    FT_847_NATIVE_CAT_SET_CTCSS_FREQ_MAIN, /* CTCSS Freq */
+    FT_847_NATIVE_CAT_SET_CTCSS_FREQ_SAT_RX,
+    FT_847_NATIVE_CAT_SET_CTCSS_FREQ_SAT_TX,
+
+    FT_847_NATIVE_CAT_SET_DCS_CODE_MAIN, /* DCS code */
+    FT_847_NATIVE_CAT_SET_DCS_CODE_SAT_RX,
+    FT_847_NATIVE_CAT_SET_DCS_CODE_SAT_TX,
+
+    FT_847_NATIVE_CAT_SET_RPT_SHIFT_MINUS, /* RPT SHIFT */
+    FT_847_NATIVE_CAT_SET_RPT_SHIFT_PLUS,
+    FT_847_NATIVE_CAT_SET_RPT_SHIFT_SIMPLEX,
+
+    FT_847_NATIVE_CAT_SET_RPT_OFFSET, /* RPT Offset frequency */
+
+    /* Get info from the rig */
+
+    FT_847_NATIVE_CAT_GET_RX_STATUS,
+    FT_847_NATIVE_CAT_GET_TX_STATUS,
+
+    FT_847_NATIVE_CAT_GET_FREQ_MODE_STATUS_MAIN,
+    FT_847_NATIVE_CAT_GET_FREQ_MODE_STATUS_SAT_RX,
+    FT_847_NATIVE_CAT_GET_FREQ_MODE_STATUS_SAT_TX,
+
+    FT_847_NATIVE_SIZE        /* end marker, value indicates number of */
+    /* native cmd entries */
+
+};
+
+typedef enum ft847_native_cmd_e ft847_native_cmd_t;
+
+
+
+/*
+ * API local implementation
+ */
+
+static int ft847_init(RIG *rig);
+static int ft847_open(RIG *rig);
+
+static int ft847_cleanup(RIG *rig);
+static int ft847_close(RIG *rig);
+
+static int ft847_set_freq(RIG *rig, vfo_t vfo, freq_t freq);
+static int ft847_get_freq(RIG *rig, vfo_t vfo, freq_t *freq);
+
+static int ft847_set_mode(RIG *rig, vfo_t vfo, rmode_t mode,
+                          pbwidth_t width); /* select mode */
+static int ft847_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode,
+                          pbwidth_t *width); /* get mode */
+
+static int ft847_set_split_vfo(RIG *rig, vfo_t vfo, split_t split,
+                               vfo_t tx_vfo);
+static int ft847_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split,
+                               vfo_t *tx_vfo);
+
+static int ft847_set_split_freq(RIG *rig, vfo_t vfo, freq_t freq);
+static int ft847_get_split_freq(RIG *rig, vfo_t vfo, freq_t *freq);
+
+static int ft847_set_split_mode(RIG *rig, vfo_t vfo, rmode_t mode,
+                                pbwidth_t width);
+static int ft847_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *mode,
+                                pbwidth_t *width);
+
+static int ft847_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt);
+static int ft847_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt);
+static int ft847_get_dcd(RIG *rig, vfo_t vfo, dcd_t *dcd);
+
+static int ft847_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val);
+
+static int ft847_set_func(RIG *rig, vfo_t vfo, setting_t func, int status);
+static int ft847_set_ctcss_tone(RIG *rig, vfo_t vfo, tone_t tone);
+static int ft847_set_ctcss_sql(RIG *rig, vfo_t vfo, tone_t tone);
+static int ft847_set_dcs_sql(RIG *rig, vfo_t vfo, tone_t code);
+static int ft847_set_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t rptr_shift);
+static int ft847_set_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t rptr_offs);
+
 /*
  * ft847 instance - private data
  *
@@ -179,7 +323,7 @@ static const yaesu_cmd_set_t ncmd[] =
  * Receiver caps
  */
 
-
+#define UNIDIRECTIONAL (rig->caps->rig_model == RIG_MODEL_FT847UNI || rig->caps->rig_model == RIG_MODEL_FT650)
 #define FT847_ALL_RX_MODES (RIG_MODE_AM|RIG_MODE_CW|RIG_MODE_CWR|RIG_MODE_SSB|RIG_MODE_FM)
 #define FT847_SSB_CW_RX_MODES (RIG_MODE_CW|RIG_MODE_CWR|RIG_MODE_SSB)
 #define FT847_AM_FM_RX_MODES (RIG_MODE_AM|RIG_MODE_FM)
@@ -220,12 +364,21 @@ static tone_t ft847_ctcss_list[] =
     1862, 1928, 2035, 2107, 2181, 2257, 2336, 2418, 2503,
     0
 };
+#if 0
+static tone_t ft650_ctcss_list[] =
+{
+    670,  719,  744,  770,  797,  825,  854,  885,  915,
+    948,  974, 1000, 1035, 1072, 1109, 1148, 1188, 1230, 1273,
+    1318, 1365, 1413, 1462, 1514, 1567, 1622, 1679, 1738, 1799,
+    1862, 1928, 2035, 2107, 2181, 2257, 2336, 2418, 2503,
+    0
+};
+#endif
 
 
 /*
- * ft847 rigs capabilities.
+ * ft847 rig capabilities.
  * Notice that some rigs share the same functions.
- * Also this struct is READONLY!
  */
 
 const struct rig_caps ft847_caps =
@@ -233,7 +386,7 @@ const struct rig_caps ft847_caps =
     RIG_MODEL(RIG_MODEL_FT847),
     .model_name = "FT-847",
     .mfg_name =  "Yaesu",
-    .version =  "20210221.0",
+    .version =  "20220525.0",
     .copyright =  "LGPL",
     .status =  RIG_STATUS_STABLE,
     .rig_type =  RIG_TYPE_TRANSCEIVER,
@@ -254,7 +407,7 @@ const struct rig_caps ft847_caps =
     .has_get_func =  RIG_FUNC_NONE,
     .has_set_func =  FT847_FUNC_ALL,
     .has_get_level =  FT847_LEVEL_ALL,
-    .has_set_level =  RIG_LEVEL_NONE,
+    .has_set_level =  RIG_LEVEL_BAND_SELECT,
     .has_get_parm =  RIG_PARM_NONE,
     .has_set_parm =  RIG_PARM_NONE,
     .level_gran =  {},      /* granularity */
@@ -375,16 +528,314 @@ const struct rig_caps ft847_caps =
     .set_dcs_sql    = ft847_set_dcs_sql,
     .set_rptr_shift = ft847_set_rptr_shift,
     .set_rptr_offs  = ft847_set_rptr_offs,
-
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
+/*
+ * ft600 rigs capabilities.
+ * Notice that some rigs share the same functions.
+ */
+
+const struct rig_caps ft650_caps =
+{
+    RIG_MODEL(RIG_MODEL_FT650),
+    .model_name = "FT-650",
+    .mfg_name =  "Yaesu",
+    .version =  "20220525.0",
+    .copyright =  "LGPL",
+    .status =  RIG_STATUS_STABLE,
+    .rig_type =  RIG_TYPE_TRANSCEIVER,
+    .ptt_type =  RIG_PTT_RIG,
+    .dcd_type =  RIG_DCD_RIG,
+    .port_type =  RIG_PORT_SERIAL,
+    .serial_rate_min =  4800,
+    .serial_rate_max =  4800,
+    .serial_data_bits =  8,
+    .serial_stop_bits =  2,
+    .serial_parity =  RIG_PARITY_NONE,
+    .serial_handshake =  RIG_HANDSHAKE_NONE,
+    .write_delay =  FT847_WRITE_DELAY,
+    .post_write_delay =  FT847_POST_WRITE_DELAY,
+    .timeout =  1000,
+    .retry =  0,
+
+    .has_get_func =  RIG_FUNC_NONE,
+    .has_set_func =  FT847_FUNC_ALL,
+    //.has_get_level =  FT847_LEVEL_ALL,
+    //.has_set_level =  RIG_LEVEL_BAND_SELECT,
+    .has_get_parm =  RIG_PARM_NONE,
+    .has_set_parm =  RIG_PARM_NONE,
+    .level_gran =  {},      /* granularity */
+    .parm_gran =  {},
+    //.ctcss_list =  ft847_ctcss_list,
+    .preamp =   { RIG_DBLST_END, }, /* no preamp/att in CAT */
+    .attenuator =   { RIG_DBLST_END, },
+    .max_rit =  Hz(0),
+    .max_xit =  Hz(0),
+    .max_ifshift =  Hz(0),
+    //.targetable_vfo =  RIG_TARGETABLE_FREQ | RIG_TARGETABLE_MODE | RIG_TARGETABLE_TONE | RIG_TARGETABLE_FUNC,
+    .transceive =  RIG_TRN_OFF,
+    .bank_qty =   0,
+    .chan_desc_sz =  0,
+
+    .chan_list =  { RIG_CHAN_END, }, /* FIXME: memory chan list: 78, but only in clonable mode? */
+
+    .rx_range_list1 =  {
+        {MHz(24.5), MHz(56), FT847_ALL_RX_MODES, -1, -1, FT847_VFOS, FT847_ANTS, "EUR"},
+        RIG_FRNG_END,
+    }, /* rx range end */
+
+    .tx_range_list1 =  {
+        FRQ_RNG_HF(1, FT847_OTHER_TX_MODES, W(5), W(100), FT847_VFOS, FT847_ANTS),
+        FRQ_RNG_HF(1, FT847_AM_TX_MODES, W(1), W(25), FT847_VFOS, FT847_ANTS),
+
+        FRQ_RNG_6m(1, FT847_OTHER_TX_MODES, W(5), W(100), FT847_VFOS, FT847_ANTS),
+        FRQ_RNG_6m(1, FT847_AM_TX_MODES, W(1), W(25), FT847_VFOS, FT847_ANTS),
+
+        FRQ_RNG_4m(1, FT847_OTHER_TX_MODES, W(1), W(50), FT847_VFOS, FT847_ANTS),
+        FRQ_RNG_4m(1, FT847_AM_TX_MODES, W(1), W(12.5), FT847_VFOS, FT847_ANTS),
+
+        FRQ_RNG_2m(1, FT847_OTHER_TX_MODES, W(1), W(50), FT847_VFOS, FT847_ANTS),
+        FRQ_RNG_2m(1, FT847_AM_TX_MODES, W(1), W(12.5), FT847_VFOS, FT847_ANTS),
+
+        FRQ_RNG_70cm(1, FT847_OTHER_TX_MODES, W(1), W(50), FT847_VFOS, FT847_ANTS),
+        FRQ_RNG_70cm(1, FT847_AM_TX_MODES, W(1), W(12.5), FT847_VFOS, FT847_ANTS),
+
+        RIG_FRNG_END,
+    }, /* tx range end */
+
+    .rx_range_list2 =  {
+        {MHz(24.5), MHz(56), FT847_ALL_RX_MODES, -1, -1, FT847_VFOS, FT847_ANTS, "USA"},
+
+        RIG_FRNG_END,
+    }, /* rx range end */
+
+    .tx_range_list2 =  {
+        {kHz(24500), kHz(25000), FT847_OTHER_TX_MODES, W(10), W(100), FT847_VFOS, FT847_ANTS},
+        {kHz(24500), kHz(25000), FT847_AM_TX_MODES, W(10), W(100), FT847_VFOS, FT847_ANTS},
+        {kHz(28000), kHz(29700), FT847_OTHER_TX_MODES, W(10), W(100), FT847_VFOS, FT847_ANTS},
+        {kHz(28000), kHz(29700), FT847_AM_TX_MODES, W(10), W(100), FT847_VFOS, FT847_ANTS},
+        {kHz(50000), kHz(54000), FT847_OTHER_TX_MODES, W(10), W(100), FT847_VFOS, FT847_ANTS},
+        {kHz(50000), kHz(54000), FT847_AM_TX_MODES, W(10), W(100), FT847_VFOS, FT847_ANTS},
+        RIG_FRNG_END,
+    }, /* tx range end */
+
+    /* mode/filter list, .remember =  order matters! */
+    .filters =  {
+        {RIG_MODE_SSB | RIG_MODE_CW, kHz(2.4)},
+        {RIG_MODE_SSB, kHz(2.2)},
+        {RIG_MODE_SSB, kHz(2.0)},
+        {RIG_MODE_SSB, kHz(1.8)},
+        {RIG_MODE_CW, Hz(1200)},
+        {RIG_MODE_CW, Hz(600)},
+        {RIG_MODE_CW, Hz(300)},
+        {RIG_MODE_AM, kHz(9)},
+        {RIG_MODE_AM, kHz(2.2)},
+        {RIG_MODE_FM, kHz(15)},
+        {RIG_MODE_FM, kHz(9)},
+        RIG_FLT_END,
+    },
+
+    .priv =   NULL,
+    .rig_init =   ft847_init,
+    .rig_cleanup =  ft847_cleanup,
+    .rig_open =   ft847_open,
+    .rig_close =  ft847_close,
+
+    .set_freq =   ft847_set_freq,       /* set freq */
+    .get_freq =  ft847_get_freq,        /* get freq */
+    .set_mode =  ft847_set_mode,        /* set mode */
+    .get_mode =  ft847_get_mode,        /* get mode */
+//    .set_split_vfo  = ft847_set_split_vfo,
+//    .get_split_vfo  = ft847_get_split_vfo,
+//    .set_split_freq = ft847_set_split_freq,
+//    .get_split_freq = ft847_get_split_freq,
+//    .set_split_mode = ft847_set_split_mode,
+//    .get_split_mode = ft847_get_split_mode,
+    .set_ptt =  ft847_set_ptt,      /* set ptt */
+    .get_ptt =  ft847_get_ptt,      /* get ptt */
+//    .get_dcd =  ft847_get_dcd,      /* get dcd */
+//    .get_level = ft847_get_level,           /* get level */
+
+    .set_func       = ft847_set_func,
+    .set_ctcss_tone = ft847_set_ctcss_tone,
+//    .set_ctcss_sql  = ft847_set_ctcss_sql,
+//    .set_dcs_sql    = ft847_set_dcs_sql,
+//    .set_rptr_shift = ft847_set_rptr_shift,
+//    .set_rptr_offs  = ft847_set_rptr_offs,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
+};
+
+const struct rig_caps mchfqrp_caps =
+{
+    RIG_MODEL(RIG_MODEL_MCHFQRP),
+    .model_name = "mcHF QRP",
+    .mfg_name =  "M0NKA",
+    .version =  "20220525.0",
+    .copyright =  "LGPL",
+    .status =  RIG_STATUS_STABLE,
+    .rig_type =  RIG_TYPE_TRANSCEIVER,
+    .ptt_type =  RIG_PTT_RIG,
+    .dcd_type =  RIG_DCD_RIG,
+    .port_type =  RIG_PORT_SERIAL,
+    .serial_rate_min =  4800,
+    .serial_rate_max =  115200,
+    .serial_data_bits =  8,
+    .serial_stop_bits =  2,
+    .serial_parity =  RIG_PARITY_NONE,
+    .serial_handshake =  RIG_HANDSHAKE_NONE,
+    .write_delay =  FT847_WRITE_DELAY,
+    .post_write_delay =  FT847_POST_WRITE_DELAY,
+    .timeout =  1000,
+    .retry =  0,
+
+    .has_get_func =  RIG_FUNC_NONE,
+    .has_set_func =  FT847_FUNC_ALL,
+    .has_get_level =  FT847_LEVEL_ALL,
+    .has_set_level =  RIG_LEVEL_BAND_SELECT,
+    .has_get_parm =  RIG_PARM_NONE,
+    .has_set_parm =  RIG_PARM_NONE,
+    .level_gran =  {},      /* granularity */
+    .parm_gran =  {},
+    .ctcss_list =  ft847_ctcss_list,
+    .dcs_list =  common_dcs_list,
+    .preamp =   { RIG_DBLST_END, }, /* no preamp/att in CAT */
+    .attenuator =   { RIG_DBLST_END, },
+    .max_rit =  Hz(0),
+    .max_xit =  Hz(0),
+    .max_ifshift =  Hz(0),
+    .targetable_vfo =  RIG_TARGETABLE_FREQ | RIG_TARGETABLE_MODE | RIG_TARGETABLE_TONE | RIG_TARGETABLE_FUNC,
+    .transceive =  RIG_TRN_OFF,
+    .bank_qty =   0,
+    .chan_desc_sz =  0,
+    .str_cal = FT847_STR_CAL,
+
+    .chan_list =  { RIG_CHAN_END, }, /* FIXME: memory chan list: 78, but only in clonable mode? */
+
+    .rx_range_list1 =  {
+        {kHz(100), MHz(30), FT847_ALL_RX_MODES, -1, -1, FT847_VFOS, FT847_ANTS, "EUR"}, /* rx range begin */
+        // {MHz(36), MHz(76), FT847_ALL_RX_MODES, -1, -1, FT847_VFOS, FT847_ANTS, "EUR"},
+        // {MHz(108), MHz(174), FT847_ALL_RX_MODES, -1, -1, FT847_VFOS, FT847_ANTS, "EUR"},
+        // {MHz(420), MHz(512), FT847_ALL_RX_MODES, -1, -1, FT847_VFOS, FT847_ANTS, "EUR"},
+
+        RIG_FRNG_END,
+    }, /* rx range end */
+
+    .tx_range_list1 =  {
+        FRQ_RNG_HF(1, FT847_OTHER_TX_MODES, W(5), W(100), FT847_VFOS, FT847_ANTS),
+        FRQ_RNG_HF(1, FT847_AM_TX_MODES, W(1), W(25), FT847_VFOS, FT847_ANTS),
+
+        // FRQ_RNG_6m(1, FT847_OTHER_TX_MODES, W(5), W(100), FT847_VFOS, FT847_ANTS),
+        // FRQ_RNG_6m(1, FT847_AM_TX_MODES, W(1), W(25), FT847_VFOS, FT847_ANTS),
+
+        // FRQ_RNG_4m(1, FT847_OTHER_TX_MODES, W(1), W(50), FT847_VFOS, FT847_ANTS),
+        // FRQ_RNG_4m(1, FT847_AM_TX_MODES, W(1), W(12.5), FT847_VFOS, FT847_ANTS),
+
+        // FRQ_RNG_2m(1, FT847_OTHER_TX_MODES, W(1), W(50), FT847_VFOS, FT847_ANTS),
+        // FRQ_RNG_2m(1, FT847_AM_TX_MODES, W(1), W(12.5), FT847_VFOS, FT847_ANTS),
+
+        // FRQ_RNG_70cm(1, FT847_OTHER_TX_MODES, W(1), W(50), FT847_VFOS, FT847_ANTS),
+        // FRQ_RNG_70cm(1, FT847_AM_TX_MODES, W(1), W(12.5), FT847_VFOS, FT847_ANTS),
+
+        RIG_FRNG_END,
+    }, /* tx range end */
+
+    .rx_range_list2 =  {
+        {kHz(100), MHz(30), FT847_ALL_RX_MODES, -1, -1, FT847_VFOS, FT847_ANTS, "USA"}, /* rx range begin */
+        // {MHz(36), MHz(76), FT847_ALL_RX_MODES, -1, -1, FT847_VFOS, FT847_ANTS, "USA"},
+        // {MHz(108), MHz(174), FT847_ALL_RX_MODES, -1, -1, FT847_VFOS, FT847_ANTS, "USA"},
+        // {MHz(420), MHz(512), FT847_ALL_RX_MODES, -1, -1, FT847_VFOS, FT847_ANTS, "USA"},
+
+        RIG_FRNG_END,
+    }, /* rx range end */
+
+    .tx_range_list2 =  {
+        FRQ_RNG_HF(2, FT847_OTHER_TX_MODES, W(5), W(100), FT847_VFOS, FT847_ANTS),
+        FRQ_RNG_HF(2, FT847_AM_TX_MODES, W(1), W(25), FT847_VFOS, FT847_ANTS),
+
+        // FRQ_RNG_6m(2, FT847_OTHER_TX_MODES, W(5), W(100), FT847_VFOS, FT847_ANTS),
+        // FRQ_RNG_6m(2, FT847_AM_TX_MODES, W(1), W(25), FT847_VFOS, FT847_ANTS),
+
+        // FRQ_RNG_2m(2, FT847_OTHER_TX_MODES, W(1), W(50), FT847_VFOS, FT847_ANTS),
+        // FRQ_RNG_2m(2, FT847_AM_TX_MODES, W(1), W(12.5), FT847_VFOS, FT847_ANTS),
+
+        // FRQ_RNG_70cm(2, FT847_OTHER_TX_MODES, W(1), W(50), FT847_VFOS, FT847_ANTS),
+        // FRQ_RNG_70cm(2, FT847_AM_TX_MODES, W(1), W(12.5), FT847_VFOS, FT847_ANTS),
+
+        RIG_FRNG_END,
+    }, /* tx range end */
+
+    .tuning_steps =  { {FT847_SSB_CW_RX_MODES, 1}, /* normal */
+        {FT847_SSB_CW_RX_MODES, 10}, /* fast */
+        {FT847_SSB_CW_RX_MODES, 100}, /* faster */
+
+
+        {FT847_AM_FM_RX_MODES, 10}, /* normal */
+        {FT847_AM_FM_RX_MODES, 100}, /* fast  */
+
+        RIG_TS_END,
+    },
+    /* mode/filter list, .remember =  order matters! */
+    .filters =  {
+        {RIG_MODE_SSB | RIG_MODE_CW | RIG_MODE_CWR, kHz(2.2)},
+        {RIG_MODE_CW | RIG_MODE_CWR, Hz(500)},
+        {RIG_MODE_AM, kHz(9)},
+        {RIG_MODE_AM, kHz(2.2)},
+        {RIG_MODE_FM, kHz(15)},
+        {RIG_MODE_FM, kHz(9)},
+        RIG_FLT_END,
+    },
+
+    .priv =   NULL,
+    .rig_init =   ft847_init,
+    .rig_cleanup =  ft847_cleanup,
+    .rig_open =   ft847_open,
+    .rig_close =  ft847_close,
+
+    .set_freq =   ft847_set_freq,       /* set freq */
+    .get_freq =  ft847_get_freq,        /* get freq */
+    .set_mode =  ft847_set_mode,        /* set mode */
+    .get_mode =  ft847_get_mode,        /* get mode */
+    .set_split_vfo  = ft847_set_split_vfo,
+    .get_split_vfo  = ft847_get_split_vfo,
+    .set_split_freq = ft847_set_split_freq,
+    .get_split_freq = ft847_get_split_freq,
+    .set_split_mode = ft847_set_split_mode,
+    .get_split_mode = ft847_get_split_mode,
+    .set_ptt =  ft847_set_ptt,      /* set ptt */
+    .get_ptt =  ft847_get_ptt,      /* get ptt */
+    .get_dcd =  ft847_get_dcd,      /* get dcd */
+    .get_level = ft847_get_level,           /* get level */
+
+    .set_func       = ft847_set_func,
+    .set_ctcss_tone = ft847_set_ctcss_tone,
+    .set_ctcss_sql  = ft847_set_ctcss_sql,
+    .set_dcs_sql    = ft847_set_dcs_sql,
+    .set_rptr_shift = ft847_set_rptr_shift,
+    .set_rptr_offs  = ft847_set_rptr_offs,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
+};
 
 /*
  * ft847uni rigs capabilities.
  * Notice that some rigs share the same functions.
  * Also this struct is READONLY!
- */
+ Yaesu appears to use the following format for serial numbers on their
+amateur products:
 
+Year of manufacture-Month of manufacture-Production Run-Individual Unit
+number, where the month of manufacture is offset by 2, so "C" means January,
+"D" means February, "E" means March, and so forth.
+
+Example: 8G051234 = 1998, May (fifth month, or "G"), Production Run 05, unit
+#1234 in this run.
+
+One key serial number range is 8G05. This seems to be the point at which
+Yaesu had corrected the bi-directional CAT issue and made some other
+improvements. This version was made in May 1998. Later serial numbers (e.g.,
+8L09nnnn) all seem to have incorporated the earlier improvements plus new
+ones...."
+ */
 const struct rig_caps ft847uni_caps =
 {
     RIG_MODEL(RIG_MODEL_FT847UNI),
@@ -411,7 +862,7 @@ const struct rig_caps ft847uni_caps =
     .has_get_func =  RIG_FUNC_NONE,
     .has_set_func =  FT847_FUNC_ALL,
     .has_get_level =  FT847_LEVEL_ALL,
-    .has_set_level =  RIG_LEVEL_NONE,
+    .has_set_level =  RIG_LEVEL_BAND_SELECT,
     .has_get_parm =  RIG_PARM_NONE,
     .has_set_parm =  RIG_PARM_NONE,
     .level_gran =  {},      /* granularity */
@@ -530,7 +981,7 @@ const struct rig_caps ft847uni_caps =
     .set_dcs_sql    = ft847_set_dcs_sql,
     .set_rptr_shift = ft847_set_rptr_shift,
     .set_rptr_offs  = ft847_set_rptr_offs,
-
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 /*
@@ -542,7 +993,7 @@ const struct rig_caps ft847uni_caps =
  * serial port is already open (rig->state->fd)
  */
 
-int ft847_init(RIG *rig)
+static int ft847_init(RIG *rig)
 {
     struct ft847_priv_data *priv;
 
@@ -584,7 +1035,7 @@ int ft847_init(RIG *rig)
  * the serial port is closed by the frontend
  */
 
-int ft847_cleanup(RIG *rig)
+static int ft847_cleanup(RIG *rig)
 {
     if (!rig)
     {
@@ -609,14 +1060,15 @@ int ft847_cleanup(RIG *rig)
  *
  */
 
-int ft847_open(RIG *rig)
+static int ft847_open(RIG *rig)
 {
-
+    int retval;
     /* Good time to set CAT ON */
 
     rig_debug(RIG_DEBUG_VERBOSE, "%s: called\n", __func__);
 
-    return ft847_send_priv_cmd(rig, FT_847_NATIVE_CAT_ON);
+    retval = ft847_send_priv_cmd(rig, FT_847_NATIVE_CAT_ON);
+    RETURNFUNC2(retval);
 }
 
 /*
@@ -624,7 +1076,7 @@ int ft847_open(RIG *rig)
  *
  */
 
-int ft847_close(RIG *rig)
+static int ft847_close(RIG *rig)
 {
 
     /* Good time to set CAT OFF */
@@ -645,15 +1097,6 @@ int ft847_close(RIG *rig)
 
 static int ft847_send_priv_cmd(RIG *rig, int cmd_index)
 {
-
-    struct rig_state *rig_s;
-    unsigned char *cmd;       /* points to sequence to send */
-
-    if (!rig)
-    {
-        return -RIG_EINVAL;
-    }
-
     if (! ncmd[cmd_index].ncomp)
     {
         rig_debug(RIG_DEBUG_VERBOSE, "%s: attempt to send incomplete sequence\n",
@@ -661,11 +1104,8 @@ static int ft847_send_priv_cmd(RIG *rig, int cmd_index)
         return -RIG_EINVAL;
     }
 
-    rig_s = &rig->state;
-
-    cmd = (unsigned char *) ncmd[cmd_index].nseq; /* get native sequence */
-
-    return write_block(&rig_s->rigport, (char *) cmd, YAESU_CMD_LENGTH);
+    return write_block(&rig->state.rigport, ncmd[cmd_index].nseq,
+                       YAESU_CMD_LENGTH);
 }
 
 
@@ -715,9 +1155,8 @@ static int opcode_vfo(RIG *rig, unsigned char *cmd, int cmd_index, vfo_t vfo)
  */
 
 
-int ft847_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
+static int ft847_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 {
-    struct rig_state *rig_s;
     unsigned char p_cmd[YAESU_CMD_LENGTH]; /* sequence to send */
     int ret;
     // cppcheck-suppress *
@@ -728,8 +1167,6 @@ int ft847_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
         return -RIG_EINVAL;
     }
 
-    rig_s = &rig->state;
-
     rig_debug(RIG_DEBUG_VERBOSE, "ft847: requested freq = %"PRIfreq" Hz, vfo=%s\n",
               freq, rig_strvfo(vfo));
 
@@ -745,7 +1182,7 @@ int ft847_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
     rig_debug(RIG_DEBUG_VERBOSE, fmt, __func__,
               (int64_t)from_bcd_be(p_cmd, 8) * 10);
 
-    if (rig->caps->rig_model == RIG_MODEL_FT847UNI)
+    if (UNIDIRECTIONAL)
     {
         struct ft847_priv_data *priv = (struct ft847_priv_data *)rig->state.priv;
 
@@ -761,7 +1198,7 @@ int ft847_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
         }
     }
 
-    return write_block(&rig_s->rigport, (char *)p_cmd, YAESU_CMD_LENGTH);
+    return write_block(&rig->state.rigport, p_cmd, YAESU_CMD_LENGTH);
 }
 
 #define MD_LSB  0x00
@@ -788,7 +1225,7 @@ static int get_freq_and_mode(RIG *rig, vfo_t vfo, freq_t *freq, rmode_t *mode,
     rig_debug(RIG_DEBUG_VERBOSE, "%s: vfo =%s \n",
               __func__, rig_strvfo(vfo));
 
-    if (rig->caps->rig_model == RIG_MODEL_FT847UNI)
+    if (UNIDIRECTIONAL)
     {
         if (vfo == RIG_VFO_MAIN)
         {
@@ -818,14 +1255,14 @@ static int get_freq_and_mode(RIG *rig, vfo_t vfo, freq_t *freq, rmode_t *mode,
         return n;
     }
 
-    n = write_block(&rs->rigport, (char *) p_cmd, YAESU_CMD_LENGTH);
+    n = write_block(&rs->rigport, p_cmd, YAESU_CMD_LENGTH);
 
     if (n < 0)
     {
         return n;
     }
 
-    n = read_block(&rs->rigport, (char *) data, YAESU_CMD_LENGTH);
+    n = read_block(&rs->rigport, data, YAESU_CMD_LENGTH);
 
     if (n != YAESU_CMD_LENGTH)
     {
@@ -892,7 +1329,7 @@ static int get_freq_and_mode(RIG *rig, vfo_t vfo, freq_t *freq, rmode_t *mode,
  * runs. The Operating Manual does not show the codes for polling the radio.
  * Note that you cannot query the sub-VFO, nor can you swap VFOs via software.
  */
-int ft847_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
+static int ft847_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 {
     rmode_t mode;
     pbwidth_t width;
@@ -903,7 +1340,7 @@ int ft847_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 
 
 
-int ft847_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
+static int ft847_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 {
     unsigned char cmd_index;  /* index of sequence to send */
     struct rig_state *rs = &rig->state;
@@ -917,7 +1354,7 @@ int ft847_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
     rig_debug(RIG_DEBUG_VERBOSE, "%s: generic mode = %s\n", __func__,
               rig_strrmode(mode));
 
-    if (rig->caps->rig_model == RIG_MODEL_FT847UNI)
+    if (UNIDIRECTIONAL)
     {
         struct ft847_priv_data *priv = (struct ft847_priv_data *)rig->state.priv;
         priv->mode = mode;
@@ -1015,10 +1452,10 @@ int ft847_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
         return ret;
     }
 
-    return write_block(&rs->rigport, (char *)p_cmd, YAESU_CMD_LENGTH);
+    return write_block(&rs->rigport, p_cmd, YAESU_CMD_LENGTH);
 }
 
-int ft847_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
+static int ft847_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
 {
     freq_t freq;
 
@@ -1029,7 +1466,7 @@ int ft847_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
 /*
  * Not exactly Split mode, this will set *SAT* Mode
  */
-int ft847_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t tx_vfo)
+static int ft847_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t tx_vfo)
 {
     struct ft847_priv_data *priv = (struct ft847_priv_data *)rig->state.priv;
     unsigned char cmd_index;  /* index of sequence to send */
@@ -1061,7 +1498,8 @@ int ft847_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t tx_vfo)
     return ret;
 }
 
-int ft847_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_vfo)
+static int ft847_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split,
+                               vfo_t *tx_vfo)
 {
     struct ft847_priv_data *priv = (struct ft847_priv_data *)rig->state.priv;
 
@@ -1073,26 +1511,28 @@ int ft847_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_vfo)
     return RIG_OK;
 }
 
-int ft847_set_split_freq(RIG *rig, vfo_t vfo, freq_t freq)
+static int ft847_set_split_freq(RIG *rig, vfo_t vfo, freq_t freq)
 {
     int retval = rig_set_split_vfo(rig, RIG_VFO_A, RIG_SPLIT_ON, RIG_VFO_B);
 
-    if (retval != RIG_OK) { RETURNFUNC(retval); }
+    if (retval != RIG_OK) { RETURNFUNC2(retval); }
 
     return ft847_set_freq(rig, RIG_VFO_TX, freq);
 }
 
-int ft847_get_split_freq(RIG *rig, vfo_t vfo, freq_t *freq)
+static int ft847_get_split_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 {
     return ft847_get_freq(rig, RIG_VFO_TX, freq);
 }
 
-int ft847_set_split_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
+static int ft847_set_split_mode(RIG *rig, vfo_t vfo, rmode_t mode,
+                                pbwidth_t width)
 {
     return ft847_set_mode(rig, RIG_VFO_TX, mode, width);
 }
 
-int ft847_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
+static int ft847_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *mode,
+                                pbwidth_t *width)
 {
     return ft847_get_mode(rig, RIG_VFO_TX, mode, width);
 }
@@ -1103,13 +1543,13 @@ int ft847_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
  */
 
 
-int ft847_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
+static int ft847_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
 {
     unsigned char cmd_index;  /* index of sequence to send */
 
     rig_debug(RIG_DEBUG_VERBOSE, "%s: called \n", __func__);
 
-    if (rig->caps->rig_model == RIG_MODEL_FT847UNI)
+    if (UNIDIRECTIONAL)
     {
         struct ft847_priv_data *priv = (struct ft847_priv_data *)rig->state.priv;
         priv->ptt = ptt;
@@ -1144,7 +1584,7 @@ static int ft847_get_status(RIG *rig, int status_ci)
     int len;
     int n;
 
-    if (rig->caps->rig_model == RIG_MODEL_FT847UNI)
+    if (UNIDIRECTIONAL)
     {
         return -RIG_ENIMPL;
     }
@@ -1168,7 +1608,7 @@ static int ft847_get_status(RIG *rig, int status_ci)
 
     rig_flush(&rig->state.rigport);
 
-    n = write_block(&rig->state.rigport, (char *) ncmd[status_ci].nseq,
+    n = write_block(&rig->state.rigport, ncmd[status_ci].nseq,
                     YAESU_CMD_LENGTH);
 
     if (n < 0)
@@ -1176,7 +1616,7 @@ static int ft847_get_status(RIG *rig, int status_ci)
         return n;
     }
 
-    n = read_block(&rig->state.rigport, (char *) data, len);
+    n = read_block(&rig->state.rigport, data, len);
 
     if (n < 0)
     {
@@ -1191,13 +1631,13 @@ static int ft847_get_status(RIG *rig, int status_ci)
     return RIG_OK;
 }
 
-int ft847_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
+static int ft847_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
 {
 
     struct ft847_priv_data *p = (struct ft847_priv_data *) rig->state.priv;
     int n;
 
-    if (rig->caps->rig_model == RIG_MODEL_FT847UNI)
+    if (UNIDIRECTIONAL)
     {
         struct ft847_priv_data *priv = (struct ft847_priv_data *)rig->state.priv;
         *ptt = priv->ptt;
@@ -1225,13 +1665,13 @@ int ft847_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
 }
 
 
-int ft847_get_dcd(RIG *rig, vfo_t vfo, dcd_t *dcd)
+static int ft847_get_dcd(RIG *rig, vfo_t vfo, dcd_t *dcd)
 {
 
     struct ft847_priv_data *p = (struct ft847_priv_data *) rig->state.priv;
     int n;
 
-    if (rig->caps->rig_model == RIG_MODEL_FT847UNI)
+    if (UNIDIRECTIONAL)
     {
         return -RIG_ENIMPL;
     }
@@ -1260,7 +1700,7 @@ static int ft847_get_rawstr_level(RIG *rig, value_t *val)
     struct ft847_priv_data *p = (struct ft847_priv_data *) rig->state.priv;
     int n;
 
-    if (rig->caps->rig_model == RIG_MODEL_FT847UNI)
+    if (UNIDIRECTIONAL)
     {
         return -RIG_ENIMPL;
     }
@@ -1287,7 +1727,7 @@ static int ft847_get_smeter_level(RIG *rig, value_t *val)
 {
     int n;
 
-    if (rig->caps->rig_model == RIG_MODEL_FT847UNI)
+    if (UNIDIRECTIONAL)
     {
         return -RIG_ENIMPL;
     }
@@ -1332,7 +1772,7 @@ static int ft847_get_alc_level(RIG *rig, value_t *val)
     struct ft847_priv_data *p = (struct ft847_priv_data *) rig->state.priv;
     int n;
 
-    if (rig->caps->rig_model == RIG_MODEL_FT847UNI)
+    if (UNIDIRECTIONAL)
     {
         return -RIG_ENIMPL;
     }
@@ -1358,9 +1798,9 @@ static int ft847_get_alc_level(RIG *rig, value_t *val)
  * and PO/ALC in transmit mode.  There is no way
  * to determine whether it's PO or ALC, unfortunately.
  */
-int ft847_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
+static int ft847_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 {
-    if (rig->caps->rig_model == RIG_MODEL_FT847UNI)
+    if (UNIDIRECTIONAL)
     {
         return -RIG_ENIMPL;
     }
@@ -1384,7 +1824,7 @@ int ft847_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 
 }
 
-int ft847_set_func(RIG *rig, vfo_t vfo, setting_t func, int status)
+static int ft847_set_func(RIG *rig, vfo_t vfo, setting_t func, int status)
 {
     unsigned char p_cmd[YAESU_CMD_LENGTH]; /* sequence to send */
     int ret;
@@ -1418,11 +1858,11 @@ int ft847_set_func(RIG *rig, vfo_t vfo, setting_t func, int status)
         return ret;
     }
 
-    return write_block(&rig->state.rigport, (char *)p_cmd, YAESU_CMD_LENGTH);
+    return write_block(&rig->state.rigport, p_cmd, YAESU_CMD_LENGTH);
 }
 
 
-int ft847_set_ctcss_tone(RIG *rig, vfo_t vfo, tone_t tone)
+static int ft847_set_ctcss_tone(RIG *rig, vfo_t vfo, tone_t tone)
 {
     unsigned char p_cmd[YAESU_CMD_LENGTH]; /* sequence to send */
     int i, ret;
@@ -1463,16 +1903,16 @@ int ft847_set_ctcss_tone(RIG *rig, vfo_t vfo, tone_t tone)
     /* get associated CAT code */
     p_cmd[0] = ft847_ctcss_cat[i];
 
-    return write_block(&rig->state.rigport, (char *)p_cmd, YAESU_CMD_LENGTH);
+    return write_block(&rig->state.rigport, p_cmd, YAESU_CMD_LENGTH);
 }
 
-int ft847_set_ctcss_sql(RIG *rig, vfo_t vfo, tone_t tone)
+static int ft847_set_ctcss_sql(RIG *rig, vfo_t vfo, tone_t tone)
 {
     /* same opcode as tone */
     return ft847_set_ctcss_tone(rig, vfo, tone);
 }
 
-int ft847_set_dcs_sql(RIG *rig, vfo_t vfo, tone_t code)
+static int ft847_set_dcs_sql(RIG *rig, vfo_t vfo, tone_t code)
 {
     unsigned char p_cmd[YAESU_CMD_LENGTH]; /* sequence to send */
     int ret;
@@ -1489,11 +1929,11 @@ int ft847_set_dcs_sql(RIG *rig, vfo_t vfo, tone_t code)
     /* DCS Code # (i.e. 07, 54=DCS Code 754) */
     to_bcd_be(p_cmd, code, 4); /* store bcd format in in p_cmd */
 
-    return write_block(&rig->state.rigport, (char *)p_cmd, YAESU_CMD_LENGTH);
+    return write_block(&rig->state.rigport, p_cmd, YAESU_CMD_LENGTH);
 
 }
 
-int ft847_set_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t rptr_shift)
+static int ft847_set_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t rptr_shift)
 {
     unsigned char cmd_index;  /* index of sequence to send */
 
@@ -1518,11 +1958,11 @@ int ft847_set_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t rptr_shift)
     return ft847_send_priv_cmd(rig, cmd_index);
 }
 
-int ft847_set_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t rptr_offs)
+static int ft847_set_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t rptr_offs)
 {
     unsigned char p_cmd[YAESU_CMD_LENGTH]; /* sequence to send */
 
-    if (rig->caps->rig_model == RIG_MODEL_FT847UNI)
+    if (UNIDIRECTIONAL)
     {
         return -RIG_ENIMPL;
     }
@@ -1532,6 +1972,6 @@ int ft847_set_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t rptr_offs)
 
     to_bcd_be(p_cmd, rptr_offs / 10, 8); /* store bcd format in in p_cmd */
 
-    return write_block(&rig->state.rigport, (char *)p_cmd, YAESU_CMD_LENGTH);
+    return write_block(&rig->state.rigport, p_cmd, YAESU_CMD_LENGTH);
 }
 
