@@ -50,7 +50,7 @@
 typedef char ncboolean;
 
 /* shared function version */
-#define NEWCAT_VER "20211202"
+#define NEWCAT_VER "20221025"
 
 /* Hopefully large enough for future use, 128 chars plus '\0' */
 #define NEWCAT_DATA_LEN                 129
@@ -69,6 +69,18 @@ typedef char ncboolean;
     .ctcss_sql = 1,\
 }
 
+#define YAESU_DEFAULT_VD_METER_200W_CAL \
+{ \
+    3, \
+    { \
+        {0, 0.0f}, \
+        {196, 50.0f}, \
+        {255, 65.0f}, \
+    } \
+}
+
+
+
 extern const struct confparams newcat_cfg_params[];
 
 /*
@@ -102,9 +114,6 @@ struct newcat_priv_caps
  */
 struct newcat_priv_data
 {
-    unsigned int
-    read_update_delay;              /* depends on pacing value */
-//    vfo_t               current_vfo;                    /* active VFO from last cmd */
     char                cmd_str[NEWCAT_DATA_LEN];       /* command string buffer */
     char
     ret_data[NEWCAT_DATA_LEN];      /* returned data--max value, most are less */
@@ -163,6 +172,7 @@ int newcat_close(RIG *rig);
 
 int newcat_set_conf(RIG *rig, token_t token, const char *val);
 int newcat_get_conf(RIG *rig, token_t token, char *val);
+int newcat_get_conf2(RIG *rig, token_t token, char *val, int val_len);
 
 int newcat_set_freq(RIG *rig, vfo_t vfo, freq_t freq);
 int newcat_get_freq(RIG *rig, vfo_t vfo, freq_t *freq);
@@ -226,7 +236,7 @@ int newcat_set_clock(RIG *rig, int year, int month, int day, int hour, int min,
                      int sec, double msec, int utc_offset);
 int newcat_get_clock(RIG *rig, int *year, int *month, int *day, int *hour,
                      int *min, int *sec, double *msec, int *utc_offset);
-
+int newcat_scan(RIG *rig, vfo_t vfo, scan_t scan, int ch);
 
 #define TOKEN_BACKEND(t) (t)
 
