@@ -28,14 +28,11 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <unistd.h>
 
 #include "idx_builtin.h"
 #include "hamlib/rig.h"
@@ -221,7 +218,7 @@ const struct rig_caps vx1700_caps =
     .mfg_name =  "Vertex Standard",
     .version =  "20210221.0",
     .copyright =  "LGPL",
-    .status =   RIG_STATUS_ALPHA,
+    .status =   RIG_STATUS_STABLE,
     .rig_type =  RIG_TYPE_TRANSCEIVER,
     .ptt_type =  RIG_PTT_RIG_MICDATA,
     .dcd_type =  RIG_DCD_NONE,  /* we have DCD pin in DATA Jack, but get_dcd() is unavailable (yet?) */
@@ -239,7 +236,7 @@ const struct rig_caps vx1700_caps =
 
     .has_get_func = (RIG_FUNC_LOCK | RIG_FUNC_TUNER),
     .has_set_func =  RIG_FUNC_NONE,
-    .has_get_level =  RIG_LEVEL_NONE,
+    .has_get_level =  RIG_LEVEL_BAND_SELECT,
     .has_set_level =  RIG_LEVEL_RFPOWER,
     .has_get_parm =  RIG_PARM_NONE,
     .has_set_parm =  RIG_PARM_NONE,
@@ -319,6 +316,7 @@ const struct rig_caps vx1700_caps =
     .set_mem        = vx1700_set_mem,
     .get_mem        = vx1700_get_mem,
     .vfo_op         = vx1700_vfo_op,
+    .hamlib_check_rig_caps = HAMLIB_CHECK_RIG_CAPS
 };
 
 
@@ -357,11 +355,11 @@ static int vx1700_do_transaction(RIG *rig,
     memset(retbuf, 0, retbuf_len);
 
     rig_flush(&rs->rigport);
-    retval = write_block(&rs->rigport, (const char *)cmd, YAESU_CMD_LENGTH);
+    retval = write_block(&rs->rigport, cmd, YAESU_CMD_LENGTH);
 
     if (retval != RIG_OK) { return retval; }
 
-    retval = read_block(&rs->rigport, (char *)retbuf, retbuf_len);
+    retval = read_block(&rs->rigport, retbuf, retbuf_len);
 
     if (retval != retbuf_len)
     {
