@@ -1,2 +1,2 @@
 /* This date time is from the last non-merge commit to Hamlib. */
-#define HAMLIBDATETIME "Mon Sep 13 23:12:03 2021 +0000 SHA=c2ed3d"
+#define HAMLIBDATETIME "Sat Oct 29 23:39:41 2022 +0000 SHA=58f9fd"
