@@ -32,13 +32,10 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -256,7 +253,7 @@ static int generic_retr_extl(RIG *rig,
 
     if (chan->ext_levels == NULL)
     {
-        p = chan->ext_levels = malloc(2 * sizeof(struct ext_list));
+        p = chan->ext_levels = calloc(1, 2 * sizeof(struct ext_list));
     }
     else
     {
@@ -819,7 +816,7 @@ int HAMLIB_API rig_set_channel(RIG *rig, vfo_t vfo, const channel_t *chan)
  *  and memory number selected. Depending on backend and rig capabilities,
  *  the chan struct may not be filled in completely.
  *
- *  Note: chan->ext_levels is a pointer to a newly mallocated memory.
+ *  Note: chan->ext_levels is a pointer to a newly allocated memory.
  *  This is the responsibility of the caller to manage and eventually
  *  free it.
  *
@@ -1181,6 +1178,7 @@ int HAMLIB_API rig_set_chan_all(RIG *rig, vfo_t vfo, const channel_t chans[])
     }
 
     rc = rig->caps;
+    memset(&map_arg, 0, sizeof(map_arg));
     map_arg.chans = (channel_t *) chans;
 
     if (rc->set_chan_all_cb)
@@ -1223,6 +1221,7 @@ int HAMLIB_API rig_get_chan_all(RIG *rig, vfo_t vfo, channel_t chans[])
     }
 
     rc = rig->caps;
+    memset(&map_arg, 0, sizeof(map_arg));
     map_arg.chans = chans;
 
     if (rc->get_chan_all_cb)
