@@ -32,9 +32,7 @@
  * \file serial.c
  */
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdlib.h>
 #include <stdio.h>   /* Standard input/output definitions */
@@ -42,9 +40,7 @@
 #include <unistd.h>  /* UNIX standard function definitions */
 #include <fcntl.h>   /* File control definitions */
 #include <errno.h>   /* Error number definitions */
-#include <sys/time.h>
 #include <sys/types.h>
-#include <unistd.h>
 #include <ctype.h>
 
 #ifdef HAVE_SYS_IOCTL_H
@@ -141,11 +137,9 @@ int HAMLIB_API serial_open(hamlib_port_t *rp)
     int fd;               /* File descriptor for the port */
     int err;
 
-    ENTERFUNC;
-
     if (!rp)
     {
-        RETURNFUNC(-RIG_EINVAL);
+        return (-RIG_EINVAL);
     }
 
 
@@ -161,13 +155,13 @@ int HAMLIB_API serial_open(hamlib_port_t *rp)
          */
         if (rp->parm.serial.parity != RIG_PARITY_NONE)
         {
-            RETURNFUNC(-RIG_EIO);
+            return (-RIG_EIO);
         }
 
         if ((rp->parm.serial.handshake != RIG_HANDSHAKE_HARDWARE) &&
                 (rp->parm.serial.handshake != RIG_HANDSHAKE_NONE))
         {
-            RETURNFUNC(-RIG_EIO);
+            return (-RIG_EIO);
         }
 
         /*
@@ -182,7 +176,7 @@ int HAMLIB_API serial_open(hamlib_port_t *rp)
 
         if (fd == -1)
         {
-            RETURNFUNC(-RIG_EIO);
+            return (-RIG_EIO);
         }
 
         rp->fd = fd;
@@ -218,7 +212,7 @@ int HAMLIB_API serial_open(hamlib_port_t *rp)
          *    from the transceiver will be returned to both applications.
          */
         uh_radio_fd = fd;
-        RETURNFUNC(RIG_OK);
+        return (RIG_OK);
     }
 
     /*
@@ -247,7 +241,7 @@ int HAMLIB_API serial_open(hamlib_port_t *rp)
                   __func__,
                   rp->pathname,
                   strerror(errno));
-        RETURNFUNC(-RIG_EIO);
+        return (-RIG_EIO);
     }
 
     rp->fd = fd;
@@ -257,13 +251,13 @@ int HAMLIB_API serial_open(hamlib_port_t *rp)
     if (err != RIG_OK)
     {
         CLOSE(fd);
-        RETURNFUNC(err);
+        return (err);
     }
 
     serial_flush(rp); // ensure nothing is there when we open
     hl_usleep(50 * 1000); // give a little time for MicroKeyer to finish
 
-    RETURNFUNC(RIG_OK);
+    return (RIG_OK);
 }
 
 
@@ -288,11 +282,9 @@ int HAMLIB_API serial_setup(hamlib_port_t *rp)
 #endif
     term_options_backup_t *term_backup = NULL;
 
-    ENTERFUNC;
-
     if (!rp)
     {
-        RETURNFUNC(-RIG_EINVAL);
+        return (-RIG_EINVAL);
     }
 
     fd = rp->fd;
@@ -376,6 +368,13 @@ int HAMLIB_API serial_setup(hamlib_port_t *rp)
         break;
 #endif
 
+#ifdef B500000
+
+    case 500000:
+        speed = B500000;    /* extra super awesome! */
+        break;
+#endif
+
     default:
         rig_debug(RIG_DEBUG_ERR,
                   "%s: unsupported rate specified: %d\n",
@@ -383,7 +382,7 @@ int HAMLIB_API serial_setup(hamlib_port_t *rp)
                   rp->parm.serial.rate);
         CLOSE(fd);
 
-        RETURNFUNC(-RIG_ECONF);
+        return (-RIG_ECONF);
     }
 
     /* TODO */
@@ -430,7 +429,7 @@ int HAMLIB_API serial_setup(hamlib_port_t *rp)
                   rp->parm.serial.data_bits);
         CLOSE(fd);
 
-        RETURNFUNC(-RIG_ECONF);
+        return (-RIG_ECONF);
         break;
     }
 
@@ -455,7 +454,7 @@ int HAMLIB_API serial_setup(hamlib_port_t *rp)
                   rp->parm.serial.stop_bits);
         CLOSE(fd);
 
-        RETURNFUNC(-RIG_ECONF);
+        return (-RIG_ECONF);
         break;
     }
 
@@ -501,7 +500,7 @@ int HAMLIB_API serial_setup(hamlib_port_t *rp)
                   rp->parm.serial.parity);
         CLOSE(fd);
 
-        RETURNFUNC(-RIG_ECONF);
+        return (-RIG_ECONF);
         break;
     }
 
@@ -513,16 +512,19 @@ int HAMLIB_API serial_setup(hamlib_port_t *rp)
     switch (rp->parm.serial.handshake)
     {
     case RIG_HANDSHAKE_NONE:
+        rig_debug(RIG_DEBUG_TRACE, "%s: Handshake=None\n", __func__);
         options.c_cflag &= ~CRTSCTS;
         options.c_iflag &= ~IXON;
         break;
 
     case RIG_HANDSHAKE_XONXOFF:
+        rig_debug(RIG_DEBUG_TRACE, "%s: Handshake=Xon/Xoff\n", __func__);
         options.c_cflag &= ~CRTSCTS;
         options.c_iflag |= IXON;        /* Enable Xon/Xoff software handshaking */
         break;
 
     case RIG_HANDSHAKE_HARDWARE:
+        rig_debug(RIG_DEBUG_TRACE, "%s: Handshake=Hardware\n", __func__);
         options.c_cflag |= CRTSCTS;     /* Enable Hardware handshaking */
         options.c_iflag &= ~IXON;
         break;
@@ -534,7 +536,7 @@ int HAMLIB_API serial_setup(hamlib_port_t *rp)
                   rp->parm.serial.handshake);
         CLOSE(fd);
 
-        RETURNFUNC(-RIG_ECONF);
+        return (-RIG_ECONF);
         break;
     }
 
@@ -579,7 +581,7 @@ int HAMLIB_API serial_setup(hamlib_port_t *rp)
                   strerror(errno));
         CLOSE(fd);
 
-        RETURNFUNC(-RIG_ECONF);       /* arg, so close! */
+        return (-RIG_ECONF);      /* arg, so close! */
     }
 
 #elif defined(HAVE_TERMIO_H)
@@ -594,7 +596,7 @@ int HAMLIB_API serial_setup(hamlib_port_t *rp)
                   strerror(errno));
         CLOSE(fd);
 
-        RETURNFUNC(-RIG_ECONF);       /* arg, so close! */
+        return (-RIG_ECONF);      /* arg, so close! */
     }
 
 #else
@@ -610,13 +612,13 @@ int HAMLIB_API serial_setup(hamlib_port_t *rp)
                   strerror(errno));
         CLOSE(fd);
 
-        RETURNFUNC(-RIG_ECONF);       /* arg, so close! */
+        return (-RIG_ECONF);      /* arg, so close! */
     }
 
 #endif
 
     // Store a copy of the original options for this FD, to be restored on close.
-    term_backup = malloc(sizeof(term_options_backup_t));
+    term_backup = calloc(1, sizeof(term_options_backup_t));
     term_backup-> fd = fd;
 #if defined(HAVE_TERMIOS_H) || defined(HAVE_TERMIO_H)
     memcpy(&term_backup->options, &orig_options, sizeof(orig_options));
@@ -628,7 +630,7 @@ int HAMLIB_API serial_setup(hamlib_port_t *rp)
     term_backup->next = term_options_backup_head;
     term_options_backup_head = term_backup;
 
-    RETURNFUNC(RIG_OK);
+    return (RIG_OK);
 }
 
 
@@ -642,7 +644,6 @@ int HAMLIB_API serial_flush(hamlib_port_t *p)
     int len;
     int timeout_save;
     unsigned char buf[4096];
-    ENTERFUNC;
 
     if (p->fd == uh_ptt_fd || p->fd == uh_radio_fd || p->flushx)
     {
@@ -668,7 +669,7 @@ int HAMLIB_API serial_flush(hamlib_port_t *p)
         rig_debug(RIG_DEBUG_TRACE, "read flushed %d bytes\n", nbytes);
 
 
-        RETURNFUNC(RIG_OK);
+        return (RIG_OK);
     }
 
     timeout_save = p->timeout;
@@ -679,7 +680,7 @@ int HAMLIB_API serial_flush(hamlib_port_t *p)
         // we pass an empty stopset so read_string can determine
         // the appropriate stopset for async data
         char stopset[1];
-        len = read_string(p, (char *)buf, sizeof(buf) - 1, stopset, 0, 1, 1);
+        len = read_string(p, buf, sizeof(buf) - 1, stopset, 0, 1, 1);
 
         if (len > 0)
         {
@@ -692,9 +693,10 @@ int HAMLIB_API serial_flush(hamlib_port_t *p)
 
             if (binary)
             {
-                char *hbuf = calloc(len * 3 + 1, 1);
+                int bytes = len * 3 + 1;
+                char *hbuf = calloc(bytes, 1);
 
-                for (i = 0; i < len; ++i) { sprintf(&hbuf[i * 3], "%02X ", buf[i]); }
+                for (i = 0; i < len; ++i) { SNPRINTF(&hbuf[i * 3], bytes - (i * 3),  "%02X ", buf[i]); }
 
                 rig_debug(RIG_DEBUG_WARN, "%s: flush hex:%s\n", __func__, hbuf);
                 free(hbuf);
@@ -710,7 +712,7 @@ int HAMLIB_API serial_flush(hamlib_port_t *p)
     p->timeout = timeout_save;
     //rig_debug(RIG_DEBUG_VERBOSE, "tcflush%s\n", "");
     //tcflush(p->fd, TCIFLUSH);
-    RETURNFUNC(RIG_OK);
+    return (RIG_OK);
 }
 
 
@@ -771,7 +773,7 @@ int ser_open(hamlib_port_t *p)
     }
 
     p->fd = ret;
-    RETURNFUNC(ret);
+    return (ret);
 }
 
 
@@ -785,7 +787,7 @@ int ser_close(hamlib_port_t *p)
     int rc;
     term_options_backup_t *term_backup, *term_backup_prev;
 
-    rig_debug(RIG_DEBUG_VERBOSE, "%s called\n", __func__);
+    //rig_debug(RIG_DEBUG_VERBOSE, "%s called\n", __func__);
 
     /*
      * For microHam devices, do not close the
@@ -798,7 +800,7 @@ int ser_close(hamlib_port_t *p)
         uh_close_ptt();
         uh_ptt_fd = -1;
         p->fd = -1;
-        RETURNFUNC(0);
+        return (0);
     }
 
     if (p->fd == uh_radio_fd)
@@ -806,7 +808,7 @@ int ser_close(hamlib_port_t *p)
         uh_close_radio();
         uh_radio_fd = -1;
         p->fd = -1;
-        RETURNFUNC(0);
+        return (0);
     }
 
     // Find backup termios options to restore before closing
@@ -880,7 +882,7 @@ int ser_close(hamlib_port_t *p)
 
     rc = CLOSE(p->fd);
     p->fd = -1;
-    RETURNFUNC(rc);
+    return (rc);
 }
 
 
@@ -895,14 +897,12 @@ int HAMLIB_API ser_set_rts(hamlib_port_t *p, int state)
     unsigned int y = TIOCM_RTS;
     int rc;
 
-    ENTERFUNC;
-
     rig_debug(RIG_DEBUG_VERBOSE, "%s: RTS=%d\n", __func__, state);
 
     // ignore this for microHam ports
     if (p->fd == uh_ptt_fd || p->fd == uh_radio_fd)
     {
-        RETURNFUNC(RIG_OK);
+        return (RIG_OK);
     }
 
 #if defined(TIOCMBIS) && defined(TIOCMBIC)
@@ -932,10 +932,10 @@ int HAMLIB_API ser_set_rts(hamlib_port_t *p, int state)
                   "%s: Cannot change RTS - %s\n",
                   __func__,
                   strerror(errno));
-        RETURNFUNC(-RIG_EIO);
+        return (-RIG_EIO);
     }
 
-    RETURNFUNC(RIG_OK);
+    return (RIG_OK);
 }
 
 
@@ -973,21 +973,19 @@ int HAMLIB_API ser_set_dtr(hamlib_port_t *p, int state)
     unsigned int y = TIOCM_DTR;
     int rc;
 
-    ENTERFUNC;
-
     rig_debug(RIG_DEBUG_VERBOSE, "%s: DTR=%d\n", __func__, state);
 
     // silently ignore on microHam RADIO channel,
     // but (un)set ptt on microHam PTT channel.
     if (p->fd == uh_radio_fd)
     {
-        RETURNFUNC(RIG_OK);
+        return (RIG_OK);
     }
 
     if (p->fd == uh_ptt_fd)
     {
         uh_set_ptt(state);
-        RETURNFUNC(RIG_OK);
+        return (RIG_OK);
     }
 
 #if defined(TIOCMBIS) && defined(TIOCMBIC)
@@ -1017,10 +1015,10 @@ int HAMLIB_API ser_set_dtr(hamlib_port_t *p, int state)
                   "%s: Cannot change DTR - %s\n",
                   __func__,
                   strerror(errno));
-        RETURNFUNC(-RIG_EIO);
+        return (-RIG_EIO);
     }
 
-    RETURNFUNC(RIG_OK);
+    return (RIG_OK);
 }
 
 
