@@ -20,9 +20,7 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdio.h>   /* Standard input/output definitions */
 #include <string.h>  /* String function definitions */
@@ -41,9 +39,10 @@
 // eventually should make this -RIG_EINTERNAL
 int check_buffer_overflow(char *str, int len, int nlen)
 {
-    if (len + 1 >= nlen)
+    if (len + 32 >= nlen) // make sure at least 32 bytes are available
     {
-        rig_debug(RIG_DEBUG_ERR, "%s: buffer overflow, len=%u, nlen=%d, str='%s'\n",
+        rig_debug(RIG_DEBUG_ERR,
+                  "%s: buffer overflow, len=%u, nlen=%d, str='%s', len+32 must be >= nlen\n",
                   __func__, len, nlen, str);
     }
 
@@ -71,10 +70,10 @@ int rig_sprintf_vfo(char *str, int nlen, vfo_t vfo)
         if (sv && sv[0] && (strstr(sv, "None") == 0))
         {
             len += sprintf(str + len, "%s ", sv);
+            check_buffer_overflow(str, len, nlen);
         }
     }
 
-    check_buffer_overflow(str, len, nlen);
     return len;
 }
 
@@ -102,14 +101,14 @@ int rig_sprintf_mode(char *str, int nlen, rmode_t mode)
         strcat(str, ms);
         strcat(str, " ");
         len += strlen(ms) + 1;
+        check_buffer_overflow(str, len, nlen);
     }
 
-    check_buffer_overflow(str, len, nlen);
     return len;
 }
 
 
-int rig_sprintf_ant(char *str, int nlen, ant_t ant)
+int rig_sprintf_ant(char *str, int str_len, ant_t ant)
 {
     int i, len = 0;
     char *ant_name;
@@ -118,7 +117,7 @@ int rig_sprintf_ant(char *str, int nlen, ant_t ant)
 
     if (ant == RIG_ANT_NONE)
     {
-        sprintf(str, "ANT_NONE");
+        SNPRINTF(str, str_len, "ANT_NONE");
         return 0;
     }
 
@@ -149,10 +148,10 @@ int rig_sprintf_ant(char *str, int nlen, ant_t ant)
             }
 
             len += sprintf(str + len, "%s ", ant_name);
+            check_buffer_overflow(str, len, str_len);
         }
     }
 
-    check_buffer_overflow(str, len, nlen);
     return len;
 }
 
@@ -180,9 +179,9 @@ int rig_sprintf_func(char *str, int nlen, setting_t func)
         strcat(str, ms);
         strcat(str, " ");
         len += strlen(ms) + 1;
+        check_buffer_overflow(str, len, nlen);
     }
 
-    check_buffer_overflow(str, len, nlen);
     return len;
 }
 
@@ -210,9 +209,9 @@ int rot_sprintf_func(char *str, int nlen, setting_t func)
         strcat(str, ms);
         strcat(str, " ");
         len += strlen(ms) + 1;
+        check_buffer_overflow(str, len, nlen);
     }
 
-    check_buffer_overflow(str, len, nlen);
     return len;
 }
 
@@ -240,9 +239,9 @@ int rig_sprintf_level(char *str, int nlen, setting_t level)
         strcat(str, ms);
         strcat(str, " ");
         len += strlen(ms) + 1;
+        check_buffer_overflow(str, len, nlen);
     }
 
-    check_buffer_overflow(str, len, nlen);
     return len;
 }
 
@@ -270,9 +269,9 @@ int rot_sprintf_level(char *str, int nlen, setting_t level)
         strcat(str, ms);
         strcat(str, " ");
         len += strlen(ms) + 1;
+        check_buffer_overflow(str, len, nlen);
     }
 
-    check_buffer_overflow(str, len, nlen);
     return len;
 }
 
@@ -300,9 +299,9 @@ int amp_sprintf_level(char *str, int nlen, setting_t level)
         strcat(str, ms);
         strcat(str, " ");
         len += strlen(ms) + 1;
+        check_buffer_overflow(str, len, nlen);
     }
 
-    check_buffer_overflow(str, len, nlen);
     return len;
 }
 
@@ -341,9 +340,10 @@ int sprintf_level_ext(char *str, int nlen, const struct confparams *extlevels)
             /* ignore case RIG_CONF_BUTTON */
             break;
         }
+
+        check_buffer_overflow(str, len, nlen);
     }
 
-    check_buffer_overflow(str, len, nlen);
     return len;
 }
 
@@ -399,9 +399,10 @@ int rig_sprintf_level_gran(char *str, int nlen, setting_t level,
                            gran[i].max.i,
                            gran[i].step.i);
         }
+
+        check_buffer_overflow(str, len, nlen);
     }
 
-    check_buffer_overflow(str, len, nlen);
     return len;
 }
 
@@ -457,9 +458,10 @@ int rot_sprintf_level_gran(char *str, int nlen, setting_t level,
                            gran[i].max.i,
                            gran[i].step.i);
         }
+
+        check_buffer_overflow(str, len, nlen);
     }
 
-    check_buffer_overflow(str, len, nlen);
     return len;
 }
 
@@ -487,9 +489,9 @@ int rig_sprintf_parm(char *str, int nlen, setting_t parm)
         strcat(str, ms);
         strcat(str, " ");
         len += strlen(ms) + 1;
+        check_buffer_overflow(str, len, nlen);
     }
 
-    check_buffer_overflow(str, len, nlen);
     return len;
 }
 
@@ -517,9 +519,9 @@ int rot_sprintf_parm(char *str, int nlen, setting_t parm)
         strcat(str, ms);
         strcat(str, " ");
         len += strlen(ms) + 1;
+        check_buffer_overflow(str, len, nlen);
     }
 
-    check_buffer_overflow(str, len, nlen);
     return len;
 }
 
@@ -575,9 +577,10 @@ int rig_sprintf_parm_gran(char *str, int nlen, setting_t parm,
                            gran[i].max.i,
                            gran[i].step.i);
         }
+
+        check_buffer_overflow(str, len, nlen);
     }
 
-    check_buffer_overflow(str, len, nlen);
     return len;
 }
 
@@ -633,9 +636,10 @@ int rot_sprintf_parm_gran(char *str, int nlen, setting_t parm,
                            gran[i].max.i,
                            gran[i].step.i);
         }
+
+        check_buffer_overflow(str, len, nlen);
     }
 
-    check_buffer_overflow(str, len, nlen);
     return len;
 }
 
@@ -663,9 +667,9 @@ int rig_sprintf_vfop(char *str, int nlen, vfo_op_t op)
         strcat(str, ms);
         strcat(str, " ");
         len += strlen(ms) + 1;
+        check_buffer_overflow(str, len, nlen);
     }
 
-    check_buffer_overflow(str, len, nlen);
     return len;
 }
 
@@ -693,16 +697,17 @@ int rig_sprintf_scan(char *str, int nlen, scan_t rscan)
         strcat(str, ms);
         strcat(str, " ");
         len += strlen(ms) + 1;
+        check_buffer_overflow(str, len, nlen);
     }
 
-    check_buffer_overflow(str, len, nlen);
     return len;
 }
 
 
 int rot_sprintf_status(char *str, int nlen, rot_status_t status)
 {
-    int i, len = 0;
+    int len = 0;
+    unsigned long i;
 
     rig_debug(RIG_DEBUG_TRACE, "%s: status=%08x\n", __func__, status);
     *str = '\0';
@@ -721,22 +726,24 @@ int rot_sprintf_status(char *str, int nlen, rot_status_t status)
         {
             len += sprintf(str + len, "%s ", sv);
         }
+
+        check_buffer_overflow(str, len, nlen);
     }
 
-    check_buffer_overflow(str, len, nlen);
     return len;
 }
 
 int rig_sprintf_spectrum_modes(char *str, int nlen,
                                const enum rig_spectrum_mode_e *modes)
 {
-    int i, len = 0, lentmp;
+    int i, len = 0;
 
     *str = '\0';
 
     for (i = 0; i < HAMLIB_MAX_SPECTRUM_MODES; i++)
     {
         const char *sm;
+        int lentmp;
 
         if (modes[i] == RIG_SPECTRUM_MODE_NONE)
         {
@@ -760,6 +767,8 @@ int rig_sprintf_spectrum_modes(char *str, int nlen,
 
         len += lentmp;
 
+        check_buffer_overflow(str, len, nlen);
+
     }
 
     return len;
@@ -767,12 +776,14 @@ int rig_sprintf_spectrum_modes(char *str, int nlen,
 
 int rig_sprintf_spectrum_spans(char *str, int nlen, const freq_t *spans)
 {
-    int i, len = 0, lentmp;
+    int i, len = 0;
 
     *str = '\0';
 
     for (i = 0; i < HAMLIB_MAX_SPECTRUM_SPANS; i++)
     {
+        int lentmp;
+
         if (spans[i] == 0)
         {
             break;
@@ -787,6 +798,7 @@ int rig_sprintf_spectrum_spans(char *str, int nlen, const freq_t *spans)
         }
 
         len += lentmp;
+        check_buffer_overflow(str, len, nlen);
     }
 
     return len;
@@ -795,12 +807,14 @@ int rig_sprintf_spectrum_spans(char *str, int nlen, const freq_t *spans)
 int rig_sprintf_spectrum_avg_modes(char *str, int nlen,
                                    const struct rig_spectrum_avg_mode *avg_modes)
 {
-    int i, len = 0, lentmp;
+    int i, len = 0;
 
     *str = '\0';
 
     for (i = 0; i < HAMLIB_MAX_SPECTRUM_MODES; i++)
     {
+        int lentmp;
+
         if (avg_modes[i].name == NULL || avg_modes[i].id < 0)
         {
             break;
@@ -816,6 +830,7 @@ int rig_sprintf_spectrum_avg_modes(char *str, int nlen,
         }
 
         len += lentmp;
+        check_buffer_overflow(str, len, nlen);
     }
 
     return len;
