@@ -19,19 +19,17 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdio.h>
 #include <string.h>
-#include <stdlib.h>
 
 #include <hamlib/rig.h>
 #include "misc.h"
 
 #include "sprintflst.h"
 #include "rigctl_parse.h"
+#include "../rigs/icom/icom.h"
 
 void range_print(FILE *fout, const struct freq_range_list range_list[], int rx);
 int range_sanity_check(const struct freq_range_list range_list[], int rx);
@@ -247,8 +245,8 @@ int dumpcaps(RIG *rig, FILE *fout)
             caps->targetable_vfo ? "Y" : "N");
 
     fprintf(fout,
-            "Has transceive: %s\n",
-            caps->transceive ? "Y" : "N");
+            "Has async data support: %s\n",
+            caps->async_data_supported ? "Y" : "N");
 
     fprintf(fout, "Announce: 0x%x\n", caps->announces);
     fprintf(fout,
@@ -295,10 +293,26 @@ int dumpcaps(RIG *rig, FILE *fout)
 
     fprintf(fout, "AGC levels:");
 
-    for (i = 0; i < HAMLIB_MAX_AGC_LEVELS && i < caps->agc_level_count; i++)
+    const struct icom_priv_caps *priv_caps =
+        (const struct icom_priv_caps *) rig->caps->priv;
+
+    if (priv_caps && RIG_BACKEND_NUM(rig->caps->rig_model) == RIG_ICOM
+            && priv_caps->agc_levels_present)
     {
-        fprintf(fout, " %d=%s", caps->agc_levels[i],
-                rig_stragclevel(caps->agc_levels[i]));
+        for (i = 0; i <= RIG_AGC_LAST && priv_caps->agc_levels[i].level != RIG_AGC_LAST
+                && priv_caps->agc_levels[i].icom_level >= 0; i++)
+        {
+            fprintf(fout, " %d=%s", priv_caps->agc_levels[i].level,
+                    rig_stragclevel(priv_caps->agc_levels[i].level));
+        }
+    }
+    else
+    {
+        for (i = 0; i < HAMLIB_MAX_AGC_LEVELS && i < caps->agc_level_count; i++)
+        {
+            fprintf(fout, " %d=%s", caps->agc_levels[i],
+                    rig_stragclevel(caps->agc_levels[i]));
+        }
     }
 
     if (i == 0)
@@ -1199,6 +1213,7 @@ static void dump_chan_caps(const channel_cap_t *chan, FILE *fout)
 
 int dumpconf(RIG *rig, FILE *fout)
 {
+    fprintf(fout, "model: %s\n", rig->caps->model_name);
     rig_token_foreach(rig, print_conf_list, (rig_ptr_t)rig);
 
     return 0;
