@@ -21,14 +21,11 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 #include <ctype.h>
 #include <errno.h>
 
@@ -304,7 +301,6 @@ static char *mystrtok(char *s, char delim)
         }
     }
 
-    // cppcheck-suppress *
     return str + ent_pos;
 }
 
