@@ -27,9 +27,7 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #ifdef WIN32
 #define WIN32_LEAN_AND_MEAN
@@ -44,7 +42,6 @@
 // cppcheck-suppress *
 #include <string.h>
 // cppcheck-suppress *
-#include <unistd.h>
 // cppcheck-suppress *
 #include <ctype.h>
 // cppcheck-suppress *
@@ -92,9 +89,10 @@
  * NB: do NOT use -W since it's reserved by POSIX.
  * TODO: add an option to read from a file
  */
-#define SHORT_OPTIONS "m:r:R:p:d:P:D:s:S:c:C:lLuvhVZ"
+#define SHORT_OPTIONS "B:m:r:R:p:d:P:D:s:S:c:C:lLuvhVZ"
 static struct option long_options[] =
 {
+    {"mapa2b",          0, 0, 'B'},
     {"model",           1, 0, 'm'},
     {"rig-file",        1, 0, 'r'},
     {"rig-file2",       1, 0, 'R'},
@@ -122,6 +120,11 @@ static int handle_ts2000(void *arg);
 static RIG *my_rig;             /* handle to rig */
 static hamlib_port_t my_com;    /* handle to virtual COM port */
 static int verbose;
+/* CW Skimmer can only set VFOA */
+/* IC7300 for example can run VFOA on FM and VFOB on CW */
+/* So -A/--mapa2b changes set_freq on VFOA to VFOB */
+/* This allows working CW Skimmer in split mode and transmit on VFOB */
+static int mapa2b;              /* maps set_freq on VFOA to VFOB instead */
 
 #ifdef HAVE_SIG_ATOMIC_T
 static sig_atomic_t volatile ctrl_c;
@@ -221,7 +224,7 @@ int main(int argc, char *argv[])
     char conf_parms[MAXCONFLEN] = "";
     int status;
 
-    printf("rigctlcom Version 1.2\n");
+    printf("rigctlcom Version 1.3\n");
 
     while (1)
     {
@@ -250,6 +253,10 @@ int main(int argc, char *argv[])
             version();
             exit(0);
 
+        case 'B':
+            mapa2b = 1;
+            break;
+
         case 'm':
             if (!optarg)
             {
@@ -622,10 +629,12 @@ int main(int argc, char *argv[])
         memset(ts2000, 0, sizeof(ts2000));
 
         status = read_string(&my_com,
-                             ts2000,
+                             (unsigned char *) ts2000,
                              sizeof(ts2000),
                              stop_set,
-                             strlen(stop_set));
+                             strlen(stop_set),
+                             0,
+                             1);
 
         rig_debug(RIG_DEBUG_TRACE, "%s: status=%d\n", __func__, status);
 
@@ -693,7 +702,7 @@ static int write_block2(void *func,
                         const char *txbuffer,
                         size_t count)
 {
-    int retval = write_block(p, txbuffer, count);
+    int retval = write_block(p, (unsigned char *) txbuffer, count);
     hl_usleep(5000);
 
     if (retval != RIG_OK)
@@ -797,7 +806,7 @@ static int handle_ts2000(void *arg)
             rig_debug(RIG_DEBUG_ERR, "%s: unexpected vfo=%d\n", __func__, vfo);
         }
 
-        snprintf(response,
+        SNPRINTF(response,
                  sizeof(response),
                  fmt,
                  (uint64_t)freq,
@@ -822,14 +831,14 @@ static int handle_ts2000(void *arg)
         rmode_t mode = ts2000_get_mode();
         char response[32];
 
-        snprintf(response, sizeof(response), "MD%1d;", (int)mode);
+        SNPRINTF(response, sizeof(response), "MD%1d;", (int)mode);
         return write_block2((void *)__func__, &my_com, response, strlen(response));
     }
     else if (strcmp(arg, "AG0;") == 0)
     {
         char response[32];
 
-        snprintf(response, sizeof(response), "AG0000;");
+        SNPRINTF(response, sizeof(response), "AG0000;");
         return write_block2((void *)__func__, &my_com, response, strlen(response));
     }
     else if (strcmp(arg, "FA;") == 0)
@@ -847,7 +856,7 @@ static int handle_ts2000(void *arg)
             return retval;
         }
 
-        snprintf(response, sizeof(response), "FA%011"PRIll";", (uint64_t)freq);
+        SNPRINTF(response, sizeof(response), "FA%011"PRIll";", (uint64_t)freq);
         return write_block2((void *)__func__, &my_com, response, strlen(response));
     }
     else if (strcmp(arg, "FB;") == 0)
@@ -864,14 +873,14 @@ static int handle_ts2000(void *arg)
             return retval;
         }
 
-        snprintf(response, sizeof(response), "FB%011"PRIll";", (uint64_t)freq);
+        SNPRINTF(response, sizeof(response), "FB%011"PRIll";", (uint64_t)freq);
         return write_block2((void *)__func__, &my_com, response, strlen(response));
     }
     else if (strcmp(arg, "SA;") == 0)
     {
         char response[32];
 
-        snprintf(response, sizeof(response), "SA0;");
+        SNPRINTF(response, sizeof(response), "SA0;");
         return write_block2((void *)__func__, &my_com, response, strlen(response));
     }
     else if (strcmp(arg, "RX;") == 0)
@@ -879,7 +888,7 @@ static int handle_ts2000(void *arg)
         char response[32];
 
         rig_set_ptt(my_rig, vfo_fixup(my_rig, RIG_VFO_A, my_rig->state.cache.split), 0);
-        snprintf(response, sizeof(response), "RX0;");
+        SNPRINTF(response, sizeof(response), "RX0;");
         return write_block2((void *)__func__, &my_com, response, strlen(response));
     }
     // Now some commands to set things
@@ -888,7 +897,7 @@ static int handle_ts2000(void *arg)
         if (strcmp(arg, "SA;") == 0)
         {
             // should we silently fail with RIG_OK instead? TBD
-            RETURNFUNC(-RIG_ENIMPL);
+            return (-RIG_ENIMPL);
         }
 
         if (strlen(arg) > 3 && ((char *)arg)[2] == '1')
@@ -901,7 +910,7 @@ static int handle_ts2000(void *arg)
             }
             else
             {
-                RETURNFUNC(-RIG_ENAVAIL);
+                return (-RIG_ENAVAIL);
             }
         }
 
@@ -955,7 +964,7 @@ static int handle_ts2000(void *arg)
             return retval;
         }
 
-        snprintf(response, sizeof(response), "FR%c;", nvfo + '0');
+        SNPRINTF(response, sizeof(response), "FR%c;", nvfo + '0');
         return write_block2((void *)__func__, &my_com, response, strlen(response));
 
         return retval;
@@ -984,7 +993,7 @@ static int handle_ts2000(void *arg)
             return retval;
         }
 
-        snprintf(response, sizeof(response), "FT%c;", nvfo + '0');
+        SNPRINTF(response, sizeof(response), "FT%c;", nvfo + '0');
         return write_block2((void *)__func__, &my_com, response, strlen(response));
 
         return retval;
@@ -1002,7 +1011,7 @@ static int handle_ts2000(void *arg)
             return retval;
         }
 
-        snprintf(response, sizeof(response), "TN%02d;", val);
+        SNPRINTF(response, sizeof(response), "TN%02d;", val);
         return write_block2((void *)__func__, &my_com, response, strlen(response));
     }
     else if (strncmp(arg, "TN", 2) == 0)
@@ -1057,7 +1066,7 @@ static int handle_ts2000(void *arg)
             return retval;
         }
 
-        snprintf(response, sizeof(response), "PA%c%c;", valA + '0', valB + '0');
+        SNPRINTF(response, sizeof(response), "PA%c%c;", valA + '0', valB + '0');
         return write_block2((void *)__func__, &my_com, response, strlen(response));
     }
     else if (strncmp(arg, "PA", 2) == 0)
@@ -1116,7 +1125,7 @@ static int handle_ts2000(void *arg)
             return retval;
         }
 
-        snprintf(response, sizeof(response), "XT%c;", val + '0');
+        SNPRINTF(response, sizeof(response), "XT%c;", val + '0');
         return write_block2((void *)__func__, &my_com, response, strlen(response));
     }
     else if (strncmp(arg, "XT", 2) == 0)
@@ -1160,7 +1169,7 @@ static int handle_ts2000(void *arg)
             return retval;
         }
 
-        snprintf(response, sizeof(response), "NR%c;", val + '0');
+        SNPRINTF(response, sizeof(response), "NR%c;", val + '0');
         return write_block2((void *)__func__, &my_com, response, strlen(response));
     }
     else if (strncmp(arg, "NR", 2) == 0)
@@ -1205,7 +1214,7 @@ static int handle_ts2000(void *arg)
             return retval;
         }
 
-        snprintf(response, sizeof(response), "NB%c;", val + '0');
+        SNPRINTF(response, sizeof(response), "NB%c;", val + '0');
         return write_block2((void *)__func__, &my_com, response, strlen(response));
     }
     else if (strncmp(arg, "NB", 2) == 0)
@@ -1252,7 +1261,7 @@ static int handle_ts2000(void *arg)
         }
 
         level = val.f * 255;
-        snprintf(response, sizeof(response), "AG0%03d;", level);
+        SNPRINTF(response, sizeof(response), "AG0%03d;", level);
         return write_block2((void *)__func__, &my_com, response, strlen(response));
     }
     else if (strncmp(arg, "AG", 2) == 0)
@@ -1303,7 +1312,7 @@ static int handle_ts2000(void *arg)
         }
 
         speechLevel = val.f * 255;
-        snprintf(response, sizeof(response), "PR%03d;", speechLevel);
+        SNPRINTF(response, sizeof(response), "PR%03d;", speechLevel);
         return write_block2((void *)__func__, &my_com, response, strlen(response));
     }
     else if (strncmp(arg, "PR", 2) == 0)
@@ -1361,7 +1370,7 @@ static int handle_ts2000(void *arg)
         }
 
         agcLevel = val.f * 255;
-        snprintf(response, sizeof(response), "GT%03d;", agcLevel);
+        SNPRINTF(response, sizeof(response), "GT%03d;", agcLevel);
         return write_block2((void *)__func__, &my_com, response, strlen(response));
     }
     else if (strncmp(arg, "GT", 2) == 0)
@@ -1412,7 +1421,7 @@ static int handle_ts2000(void *arg)
         }
 
         sqlev = val.f * 255;
-        snprintf(response, sizeof(response), "SQ%03d;", sqlev);
+        SNPRINTF(response, sizeof(response), "SQ%03d;", sqlev);
         return write_block2((void *)__func__, &my_com, response, strlen(response));
     }
     else if (strncmp(arg, "SQ", 2) == 0)
@@ -1461,7 +1470,7 @@ static int handle_ts2000(void *arg)
             return retval;
         }
 
-        snprintf(response, sizeof(response), "DC%c;", split + '0');
+        SNPRINTF(response, sizeof(response), "DC%c;", split + '0');
         return write_block2((void *)__func__, &my_com, response, strlen(response));
 
         return retval;
@@ -1492,7 +1501,7 @@ static int handle_ts2000(void *arg)
             return retval;
         }
 
-        snprintf(response, sizeof(response), "DC%c;", split + '0');
+        SNPRINTF(response, sizeof(response), "DC%c;", split + '0');
         return write_block2((void *)__func__, &my_com, response, strlen(response));
 
         return retval;
@@ -1512,9 +1521,12 @@ static int handle_ts2000(void *arg)
     else if (strncmp(arg, "FA0", 3) == 0)
     {
         freq_t freq;
+        vfo_t vfo = RIG_VFO_A;
+
+        if (mapa2b) { vfo = RIG_VFO_B; }
 
         sscanf((char *)arg + 2, "%"SCNfreq, &freq);
-        return rig_set_freq(my_rig, vfo_fixup(my_rig, RIG_VFO_A,
+        return rig_set_freq(my_rig, vfo_fixup(my_rig, vfo,
                                               my_rig->state.cache.split), freq);
     }
     else if (strncmp(arg, "FB0", 3) == 0)
@@ -1556,7 +1568,7 @@ static int handle_ts2000(void *arg)
         case 9: mode = RIG_MODE_RTTYR; break;
         }
 
-        snprintf(response, sizeof(response), "MD%c;", mode + '0');
+        SNPRINTF(response, sizeof(response), "MD%c;", mode + '0');
         return write_block2((void *)__func__, &my_com, response, strlen(response));
     }
     else if (strcmp(arg, "PS1;") == 0)
@@ -1571,7 +1583,7 @@ static int handle_ts2000(void *arg)
     {
         char response[32];
 
-        snprintf(response, sizeof(response), "PS1;");
+        SNPRINTF(response, sizeof(response), "PS1;");
         return write_block2((void *)__func__, &my_com, response, strlen(response));
     }
     else if (strncmp(arg, "SB", 2) == 0
@@ -1604,7 +1616,7 @@ static int handle_ts2000(void *arg)
     {
         char response[32];
 
-        snprintf(response, sizeof(response), "?;");
+        SNPRINTF(response, sizeof(response), "?;");
         return write_block2((void *)__func__, &my_com, response, strlen(response));
     }
     else
@@ -1625,7 +1637,7 @@ void usage()
 {
     char *name = "rigctlcom";
     printf("Usage: %s -m rignumber -r comport -s baud -R comport [OPTIONS]...\n\n"
-           "A TS-2000 emulator for rig sharing with programs that don't support Hamlib to be able\n"
+           "A TS-2000 emulator for rig sharing with programs that don't support Hamlib or FLRig to be able\n"
            "to use a connected radio transceiver or receiver with FLRig or rigctld via Hamlib.\n\n",
            name);
 
@@ -1646,6 +1658,7 @@ void usage()
         "  -S, --serial-speed2=BAUD      set serial speed of the virtual com port [default=115200]\n"
         "  -c, --civaddr=ID              set CI-V address, decimal (for Icom rigs only)\n"
         "  -C, --set-conf=PARM=VAL       set config parameters\n"
+        "  -B, --mapa2b                  maps set_freq on VFOA to VFOB -- useful for CW Skimmer\n"
         "  -L, --show-conf               list all config parameters\n"
         "  -l, --list                    list all model numbers and exit\n"
         "  -u, --dump-caps               dump capabilities and exit\n"
