@@ -23,9 +23,7 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #ifdef WIN32
 #define WIN32_LEAN_AND_MEAN
@@ -71,7 +69,6 @@
 #endif
 
 #include <hamlib/rig.h>
-#include <hamlibdatetime.h>
 #include "misc.h"
 #include "iofunc.h"
 #include "serial.h"
@@ -86,7 +83,7 @@
  *      keep up to date SHORT_OPTIONS, usage()'s output and man page. thanks.
  * TODO: add an option to read from a file
  */
-#define SHORT_OPTIONS "m:r:p:d:P:D:s:c:T:t:C:W:x:z:lLuovhVZM"
+#define SHORT_OPTIONS "m:r:R:p:d:P:D:s:S:c:T:t:C:W:w:x:z:lLuovhVZMA:n:"
 static struct option long_options[] =
 {
     {"model",           1, 0, 'm'},
@@ -96,6 +93,7 @@ static struct option long_options[] =
     {"ptt-type",        1, 0, 'P'},
     {"dcd-type",        1, 0, 'D'},
     {"serial-speed",    1, 0, 's'},
+    {"separator",       1, 0, 'S'},
     {"civaddr",         1, 0, 'c'},
     {"listen-addr",     1, 0, 'T'},
     {"port",            1, 0, 't'},
@@ -108,9 +106,13 @@ static struct option long_options[] =
     {"help",            0, 0, 'h'},
     {"version",         0, 0, 'V'},
     {"twiddle_timeout", 1, 0, 'W'},
+    {"twiddle_rit",     1, 0, 'w'},
     {"uplink",          1, 0, 'x'},
     {"debug-time-stamps", 0, 0, 'Z'},
     {"multicast-addr",  1, 0, 'M'},
+    {"multicast-port",  1, 0, 'n'},
+    {"password",        1, 0, 'A'},
+    {"rigctld-idle",    0, 0, 'R'},
     {0, 0, 0, 0}
 };
 
@@ -122,6 +124,7 @@ struct handle_data
     struct sockaddr_storage cli_addr;
     socklen_t clilen;
     int vfo_mode;
+    int use_password;
 };
 
 
@@ -134,6 +137,7 @@ static unsigned client_count;
 #endif
 
 static RIG *my_rig;             /* handle to rig (instance) */
+static volatile int rig_opened = 0;
 static int verbose;
 
 #ifdef HAVE_SIG_ATOMIC_T
@@ -145,6 +149,11 @@ static int volatile ctrl_c;
 const char *portno = "4532";
 const char *src_addr = NULL; /* INADDR_ANY */
 const char *multicast_addr = "0.0.0.0";
+int multicast_port = 4532;
+extern char rigctld_password[65];
+char resp_sep = '\n';
+extern int lock_mode;
+extern powerstat_t rig_powerstat;
 
 #define MAXCONFLEN 1024
 
@@ -232,7 +241,6 @@ static void handle_error(enum rig_debug_level_e lvl, const char *msg)
 #endif
 }
 
-
 int main(int argc, char *argv[])
 {
     rig_model_t my_model = RIG_MODEL_DUMMY;
@@ -252,10 +260,14 @@ int main(int argc, char *argv[])
     int sock_listen;
     int reuseaddr = 1;
     int twiddle_timeout = 0;
+    int twiddle_rit = 0;
     int uplink = 0;
+    int rigctld_idle =
+        0; // if true then rig will close when no clients are connected
     char host[NI_MAXHOST];
     char serv[NI_MAXSERV];
     char rigstartup[1024];
+    char vbuf[1024];
 #if HAVE_SIGACTION
     struct sigaction act;
 #endif
@@ -267,6 +279,13 @@ int main(int argc, char *argv[])
     struct handle_data *arg;
     int vfo_mode = 0; /* vfo_mode=0 means target VFO is current VFO */
     int i;
+    extern int is_rigctld;
+
+    is_rigctld = 1;
+
+    int err = setvbuf(stderr, vbuf, _IOFBF, sizeof(vbuf));
+    if (err) rig_debug(RIG_DEBUG_ERR, "%s: setvbuf err=%s\n", __func__, strerror(err));
+
 
     while (1)
     {
@@ -295,6 +314,19 @@ int main(int argc, char *argv[])
             printf("rigctl %s\n", hamlib_version2);
             exit(0);
 
+        case 'R':
+            rigctld_idle = 1;
+            break;
+
+        case 'A':
+            strncpy(rigctld_password, optarg, sizeof(rigctld_password) - 1);
+            //char *md5 = rig_make_m d5(rigctld_password);
+            char md5[HAMLIB_SECRET_LENGTH + 1];
+            rig_password_generate_secret(rigctld_password, md5);
+            printf("Secret key: %s\n", md5);
+            rig_settings_save("sharedkey", md5, e_CHAR);
+            break;
+
         case 'm':
             if (!optarg)
             {
@@ -443,6 +475,17 @@ int main(int argc, char *argv[])
             civaddr = optarg;
             break;
 
+        case 'S':
+            if (!optarg)
+            {
+                usage();    /* wrong arg count */
+                exit(1);
+            }
+
+            resp_sep = *optarg;
+            rig_debug(RIG_DEBUG_VERBOSE, "%s: resp_sep=%c\n", __func__, resp_sep);
+            break;
+
         case 's':
             if (!optarg)
             {
@@ -533,6 +576,19 @@ int main(int argc, char *argv[])
                     "twiddle_timeout is deprecated...use e.g. --set-conf=twiddle_timeout=5\n");
             break;
 
+        case 'w':
+            if (!optarg)
+            {
+                usage();    /* wrong arg count */
+                exit(1);
+            }
+
+            twiddle_rit = atoi(optarg);
+            fprintf(stderr,
+                    "twiddle_timeout is deprecated...use e.g. --set-conf=twiddle_timeout=5\n");
+            break;
+
+
         case 'x':
             if (!optarg)
             {
@@ -558,6 +614,23 @@ int main(int argc, char *argv[])
             multicast_addr = optarg;
             break;
 
+        case 'n':
+            if (!optarg)
+            {
+                usage();    /* wrong arg count */
+                exit(1);
+            }
+
+            multicast_port = atoi(optarg);
+
+            if (multicast_port == 0)
+            {
+                fprintf(stderr, "Invalid multicast port: %s\n", optarg);
+                exit(1);
+            }
+
+            break;
+
         default:
             usage();    /* unknown option? */
             exit(1);
@@ -576,7 +649,7 @@ int main(int argc, char *argv[])
 
     rig_set_debug(verbose);
 
-    snprintf(rigstartup, sizeof(rigstartup), "%s(%d) Startup:", __FILE__, __LINE__);
+    SNPRINTF(rigstartup, sizeof(rigstartup), "%s(%d) Startup:", __FILE__, __LINE__);
 
     for (i = 0; i < argc; ++i) { strcat(rigstartup, " "); strcat(rigstartup, argv[i]); }
 
@@ -585,6 +658,8 @@ int main(int argc, char *argv[])
     rig_debug(RIG_DEBUG_VERBOSE, "rigctld %s\n", hamlib_version2);
     rig_debug(RIG_DEBUG_VERBOSE, "%s",
               "Report bugs to <hamlib-developer@lists.sourceforge.net>\n\n");
+    rig_debug(RIG_DEBUG_VERBOSE, "Max# of rigctld client services=%d\n",
+              NI_MAXSERV);
 
     my_rig = rig_init(my_model);
 
@@ -612,6 +687,7 @@ int main(int argc, char *argv[])
     }
 
     my_rig->state.twiddle_timeout = twiddle_timeout;
+    my_rig->state.twiddle_rit = twiddle_rit;
     my_rig->state.uplink = uplink;
     rig_debug(RIG_DEBUG_TRACE, "%s: twiddle=%d, uplink=%d, twiddle_rit=%d\n",
               __func__,
@@ -623,27 +699,34 @@ int main(int argc, char *argv[])
     if (ptt_type != RIG_PTT_NONE)
     {
         my_rig->state.pttport.type.ptt = ptt_type;
+        my_rig->state.pttport_deprecated.type.ptt = ptt_type;
     }
 
     if (dcd_type != RIG_DCD_NONE)
     {
         my_rig->state.dcdport.type.dcd = dcd_type;
+        my_rig->state.dcdport_deprecated.type.dcd = dcd_type;
     }
 
     if (ptt_file)
     {
         strncpy(my_rig->state.pttport.pathname, ptt_file, HAMLIB_FILPATHLEN - 1);
+        strncpy(my_rig->state.pttport_deprecated.pathname, ptt_file,
+                HAMLIB_FILPATHLEN - 1);
     }
 
     if (dcd_file)
     {
         strncpy(my_rig->state.dcdport.pathname, dcd_file, HAMLIB_FILPATHLEN - 1);
+        strncpy(my_rig->state.dcdport_deprecated.pathname, dcd_file,
+                HAMLIB_FILPATHLEN - 1);
     }
 
     /* FIXME: bound checking and port type == serial */
     if (serial_rate != 0)
     {
         my_rig->state.rigport.parm.serial.rate = serial_rate;
+        my_rig->state.rigport_deprecated.parm.serial.rate = serial_rate;
     }
 
     if (civaddr)
@@ -670,14 +753,15 @@ int main(int argc, char *argv[])
         exit(0);
     }
 
-    /* open and close rig connection to check early for issues */
+    /* attempt to open rig to check early for issues */
     retcode = rig_open(my_rig);
+    rig_opened = retcode == RIG_OK ? 1 : 0;
 
     if (retcode != RIG_OK)
     {
         fprintf(stderr, "rig_open: error = %s %s %s \n", rigerror(retcode), rig_file,
                 strerror(errno));
-        exit(2);
+        // continue even if opening the rig fails, because it may be powered off
     }
 
     if (verbose > RIG_DEBUG_ERR)
@@ -690,17 +774,20 @@ int main(int argc, char *argv[])
     rig_debug(RIG_DEBUG_VERBOSE, "Backend version: %s, Status: %s\n",
               my_rig->caps->version, rig_strstatus(my_rig->caps->status));
 
-#if 0
-    rig_close(my_rig);          /* we will reopen for clients */
-
-    if (verbose > RIG_DEBUG_ERR)
+    // Normally we keep the rig open to speed up the 1st client connect
+    // But some rigs like the FT-736 have to lock the rig for CAT control
+    // So they need to release the rig when no clients are connected
+    if (rigctld_idle)
     {
-        printf("Closed rig model %d, '%s - will reopen for clients'\n",
-               my_rig->caps->rig_model,
-               my_rig->caps->model_name);
-    }
+        rig_close(my_rig);          /* we will reopen for clients */
 
-#endif
+        if (verbose > RIG_DEBUG_ERR)
+        {
+            printf("Closed rig model %d, '%s - will reopen for clients'\n",
+                   my_rig->caps->rig_model,
+                   my_rig->caps->model_name);
+        }
+    }
 
 #ifdef __MINGW32__
 #  ifndef SO_OPENTYPE
@@ -758,7 +845,8 @@ int main(int argc, char *argv[])
 
     enum multicast_item_e items = RIG_MULTICAST_POLL | RIG_MULTICAST_TRANSCEIVE |
                                   RIG_MULTICAST_SPECTRUM;
-    retcode = network_multicast_server(my_rig, multicast_addr, 4532, items);
+    retcode = network_multicast_publisher_start(my_rig, multicast_addr,
+              multicast_port, items);
 
     if (retcode != RIG_OK)
     {
@@ -901,19 +989,24 @@ int main(int argc, char *argv[])
     /*
      * main loop accepting connections
      */
+    rig_debug(RIG_DEBUG_TRACE, "%s: rigctld listening on port %s\n", __func__,
+              portno);
+
     do
     {
         fd_set set;
         struct timeval timeout;
 
-        arg = malloc(sizeof(struct handle_data));
+        arg = calloc(1, sizeof(struct handle_data));
 
         if (!arg)
         {
-            rig_debug(RIG_DEBUG_ERR, "malloc: %s\n", strerror(errno));
+            rig_debug(RIG_DEBUG_ERR, "calloc: %s\n", strerror(errno));
             exit(1);
         }
 
+        if (rigctld_password[0] != 0) { arg->use_password = 1; }
+
         /* use select to allow for periodic checks for CTRL+C */
         FD_ZERO(&set);
         FD_SET(sock_listen, &set);
@@ -927,7 +1020,12 @@ int main(int argc, char *argv[])
             rig_debug(RIG_DEBUG_ERR, "%s: select() failed: %s\n", __func__,
                       strerror(errno_stored));
 
-            // TODO: FIXME: Why does this select() return EINTR after any command when set_trn RIG is enabled?
+            if (ctrl_c)
+            {
+                rig_debug(RIG_DEBUG_VERBOSE, "%s: ctrl_c when retcode==-1\n", __func__);
+                break;
+            }
+
             if (errno == EINTR)
             {
                 rig_debug(RIG_DEBUG_VERBOSE, "%s: ignoring interrupted system call\n",
@@ -939,6 +1037,7 @@ int main(int argc, char *argv[])
         {
             if (ctrl_c)
             {
+                rig_debug(RIG_DEBUG_VERBOSE, "%s: ctrl_c when retcode==0\n", __func__);
                 break;
             }
         }
@@ -995,9 +1094,12 @@ int main(int argc, char *argv[])
     }
     while (retcode == 0 && !ctrl_c);
 
+    rig_debug(RIG_DEBUG_VERBOSE, "%s: while loop done\n", __func__);
+
 #ifdef HAVE_PTHREAD
     /* allow threads to finish current action */
     mutex_rigctld(1);
+    HAMLIB_TRACE;
 
     if (client_count)
     {
@@ -1005,10 +1107,17 @@ int main(int argc, char *argv[])
     }
 
     rig_close(my_rig);
+    HAMLIB_TRACE;
     mutex_rigctld(0);
+    HAMLIB_TRACE;
 #else
     rig_close(my_rig); /* close port */
 #endif
+
+    HAMLIB_TRACE;
+    network_multicast_publisher_stop(my_rig);
+
+    HAMLIB_TRACE;
     rig_cleanup(my_rig); /* if you care about memory */
 
 #ifdef __MINGW32__
@@ -1058,9 +1167,7 @@ void *handle_socket(void *arg)
     char serv[NI_MAXSERV];
     char send_cmd_term = '\r';  /* send_cmd termination char */
     int ext_resp = 0;
-    char resp_sep = '\n';
-
-    ENTERFUNC;
+    rig_powerstat = RIG_POWER_ON; // defaults to power on
 
     fsockin = get_fsockin(handle_data_arg);
 
@@ -1078,6 +1185,7 @@ void *handle_socket(void *arg)
     {
         rig_debug(RIG_DEBUG_ERR, "%s: fdopen out: %s\n", __func__, strerror(errno));
         fclose(fsockin);
+        fsockin = NULL;
 
         goto handle_exit;
     }
@@ -1104,7 +1212,9 @@ void *handle_socket(void *arg)
 
     mutex_rigctld(0);
 #else
+    mutext_rigctld(1);
     retcode = rig_open(my_rig);
+    mutext_rigctld(1);
 
     if (RIG_OK == retcode && verbose > RIG_DEBUG_ERR)
     {
@@ -1115,10 +1225,18 @@ void *handle_socket(void *arg)
 
 #endif
 
-    int rig_opened = 1;  // our rig is already open
+    if (my_rig->caps->get_powerstat)
+    {
+        mutex_rigctld(1);
+        rig_get_powerstat(my_rig, &rig_powerstat);
+        mutex_rigctld(0);
+        my_rig->state.powerstat = rig_powerstat;
+    }
 
     do
     {
+        mutex_rigctld(1);
+
         if (!rig_opened)
         {
             retcode = rig_open(my_rig);
@@ -1127,15 +1245,30 @@ void *handle_socket(void *arg)
                       retcode);
         }
 
+        mutex_rigctld(0);
+
         if (rig_opened) // only do this if rig is open
         {
-            rig_debug(RIG_DEBUG_TRACE, "%s: doing rigctl_parse vfo_mode=%d\n", __func__,
-                      handle_data_arg->vfo_mode);
+            powerstat_t powerstat;
+            rig_debug(RIG_DEBUG_TRACE, "%s: doing rigctl_parse vfo_mode=%d, secure=%d\n",
+                      __func__,
+                      handle_data_arg->vfo_mode, handle_data_arg->use_password);
             retcode = rigctl_parse(handle_data_arg->rig, fsockin, fsockout, NULL, 0,
                                    mutex_rigctld,
-                                   1, 0, &handle_data_arg->vfo_mode, send_cmd_term, &ext_resp, &resp_sep);
+                                   1, 0, &handle_data_arg->vfo_mode, send_cmd_term, &ext_resp, &resp_sep,
+                                   handle_data_arg->use_password);
 
             if (retcode != 0) { rig_debug(RIG_DEBUG_VERBOSE, "%s: rigctl_parse retcode=%d\n", __func__, retcode); }
+
+            // update our power stat in case power gets turned off
+            if (retcode == -RIG_ETIMEOUT
+                    && my_rig->caps->get_powerstat) // if we get a timeout we might be powered off
+            {
+                rig_get_powerstat(my_rig, &powerstat);
+                rig_powerstat = powerstat;
+
+                if (powerstat == RIG_POWER_OFF) { retcode = -RIG_EPOWER; }
+            }
         }
         else
         {
@@ -1147,22 +1280,34 @@ void *handle_socket(void *arg)
         if (retcode < 0 && !RIG_IS_SOFT_ERRCODE(-retcode))
         {
             int retry = 3;
-            rig_debug(RIG_DEBUG_ERR, "%s: i/o error\n", __func__)
+            rig_debug(RIG_DEBUG_ERR, "%s: i/o error\n", __func__);
 
             do
             {
+                mutex_rigctld(1);
                 retcode = rig_close(my_rig);
-                hl_usleep(1000 * 1000);
+                rig_opened = 0;
+                mutex_rigctld(0);
                 rig_debug(RIG_DEBUG_ERR, "%s: rig_close retcode=%d\n", __func__, retcode);
-                retcode = rig_open(my_rig);
-                rig_opened = retcode == RIG_OK ? 1 : 0;
-                rig_debug(RIG_DEBUG_ERR, "%s: rig_open retcode=%d, opened=%d\n", __func__,
-                          retcode, rig_opened);
+
+                hl_usleep(1000 * 1000);
+
+                mutex_rigctld(1);
+
+                if (!rig_opened)
+                {
+                    retcode = rig_open(my_rig);
+                    rig_opened = retcode == RIG_OK ? 1 : 0;
+                    rig_debug(RIG_DEBUG_ERR, "%s: rig_open retcode=%d, opened=%d\n", __func__,
+                              retcode, rig_opened);
+                }
+
+                mutex_rigctld(0);
             }
-            while (retry-- > 0 && retcode != RIG_OK);
+            while (!ctrl_c && !rig_opened && retry-- > 0 && retcode != RIG_OK);
         }
     }
-    while (retcode == RIG_OK || RIG_IS_SOFT_ERRCODE(-retcode));
+    while (!ctrl_c && (retcode == RIG_OK || RIG_IS_SOFT_ERRCODE(-retcode)));
 
 #ifdef HAVE_PTHREAD
 #if 0
@@ -1260,6 +1405,7 @@ void usage(void)
         "  -s, --serial-speed=BAUD       set serial speed of the serial port\n"
         "  -c, --civaddr=ID              set CI-V address, decimal (for Icom rigs only)\n"
         "  -t, --port=NUM                set TCP listening port, default %s\n"
+        "  -S, --separator=CHAR          set char as rigctld response separator, default is \\n\n"
         "  -T, --listen-addr=IPADDR      set listening IP address, default ANY\n"
         "  -C, --set-conf=PARM=VAL       set config parameters\n"
         "  -L, --show-conf               list all config parameters\n"
@@ -1268,16 +1414,27 @@ void usage(void)
         "  -o, --vfo                     do not default to VFO_CURR, require extra vfo arg\n"
         "  -v, --verbose                 set verbose mode, cumulative (-v to -vvvvv)\n"
         "  -W, --twiddle_timeout         timeout after detecting vfo manual change\n"
-        "  -W, --twiddle_rit             suppress VFOB getfreq so RIT can be twiddled\n"
+        "  -w, --twiddle_rit             suppress VFOB getfreq so RIT can be twiddled\n"
         "  -x, --uplink                  set uplink get_freq ignore, 1=Sub, 2=Main\n"
         "  -Z, --debug-time-stamps       enable time stamps for debug messages\n"
-        "  -M, --multicast-addr=addr     set multicast addr, default 0.0.0.0 (off), recommend 224.0.1.1\n"
+        "  -M, --multicast-addr=addr     set multicast UDP address, default 0.0.0.0 (off), recommend 224.0.1.1\n"
+        "  -n, --multicast-port=port     set multicast UDP port, default 4532\n"
+        "  -A, --password                set password for rigctld access\n"
+        "  -R, --rigctld-idle            make rigctld close the rig when no clients are connected\n"
         "  -h, --help                    display this help and exit\n"
         "  -V, --version                 output version information and exit\n\n",
         portno);
 
     usage_rig(stdout);
 
+    printf("\nError codes and messages\n");
+
+    for (enum rig_errcode_e e = 0; e < RIG_EEND; ++e)
+    {
+        printf("-%d - %s", e, rigerror2(e));
+    }
+
+
     printf("\nReport bugs to <hamlib-developer@lists.sourceforge.net>.\n");
 
 }
