@@ -26,9 +26,7 @@
 
 // TODO: Add "symmetric" set_conf + get_conf to rigctl+rotctl
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -275,7 +273,7 @@ struct mod_lst
     char model_name[32];    /* caps->model_name */
     char version[32];       /* caps->version */
     char status[32];        /* caps->status */
-    char macro_name[32];    /* caps->macro_name */
+    char macro_name[64];    /* caps->macro_name */
     UT_hash_handle hh;      /* makes this structure hashable */
 };
 
@@ -293,14 +291,14 @@ void hash_add_model(int id,
 {
     struct mod_lst *s;
 
-    s = (struct mod_lst *)malloc(sizeof(struct mod_lst));
+    s = (struct mod_lst *)calloc(1, sizeof(struct mod_lst));
 
     s->id = id;
-    snprintf(s->mfg_name, sizeof(s->mfg_name), "%s", mfg_name);
-    snprintf(s->model_name, sizeof(s->model_name), "%s", model_name);
-    snprintf(s->version, sizeof(s->version), "%s", version);
-    snprintf(s->status, sizeof(s->status), "%s", status);
-    snprintf(s->macro_name, sizeof(s->macro_name), "%s", macro_name);
+    SNPRINTF(s->mfg_name, sizeof(s->mfg_name), "%s", mfg_name);
+    SNPRINTF(s->model_name, sizeof(s->model_name), "%s", model_name);
+    SNPRINTF(s->version, sizeof(s->version), "%s", version);
+    SNPRINTF(s->status, sizeof(s->status), "%s", status);
+    SNPRINTF(s->macro_name, sizeof(s->macro_name), "%s", macro_name);
 
     HASH_ADD_INT(models, id, s);    /* id: name of key field */
 }
@@ -1031,7 +1029,7 @@ int rotctl_parse(ROT *my_rot, FILE *fin, FILE *fout, char *argv[], int argc,
             /* The starting position of the source string is the first
              * character past the initial '\'.
              */
-            snprintf(cmd_name, sizeof(cmd_name), "%s", parsed_input[0] + 1);
+            SNPRINTF(cmd_name, sizeof(cmd_name), "%s", parsed_input[0] + 1);
 
             /* Sanity check as valid multiple character commands consist of
              * alphanumeric characters and the underscore ('_') character.
@@ -1566,7 +1564,7 @@ int print_conf_list(const struct confparams *cfp, rig_ptr_t data)
     int i;
     char buf[128] = "";
 
-    rot_get_conf(rot, cfp->token, buf);
+    rot_get_conf2(rot, cfp->token, buf, sizeof(buf));
     printf("%s: \"%s\"\n" "\tDefault: %s, Value: %s\n",
            cfp->name,
            cfp->tooltip,
@@ -2342,7 +2340,7 @@ declare_proto_rot(inter_set_conf)
         return -RIG_EINVAL;
     }
 
-    sprintf(buf, "%s=%s", arg1, arg2);
+    SNPRINTF(buf, sizeof(buf), "%s=%s", arg1, arg2);
     return set_conf(rot, buf);
 }
 
@@ -2432,8 +2430,8 @@ declare_proto_rot(send_cmd)
     struct rot_state *rs;
     int backend_num, cmd_len;
 #define BUFSZ 128
-    char bufcmd[BUFSZ];
-    char buf[BUFSZ];
+    unsigned char bufcmd[BUFSZ];
+    unsigned char buf[BUFSZ];
     char eom_buf[4] = { 0xa, 0xd, 0, 0 };
 
     /*
@@ -2463,10 +2461,10 @@ declare_proto_rot(send_cmd)
     else
     {
         /* text protocol */
-        strncpy(bufcmd, arg1, BUFSZ);
+        strncpy((char *) bufcmd, arg1, BUFSZ);
         bufcmd[BUFSZ - 2] = '\0';
 
-        cmd_len = strlen(bufcmd);
+        cmd_len = strlen((char *) bufcmd);
 
         /* Automatic termination char */
         if (send_cmd_term != 0)
@@ -2499,7 +2497,7 @@ declare_proto_rot(send_cmd)
          * assumes CR or LF is end of line char
          * for all ascii protocols
          */
-        retval = read_string(&rs->rotport, buf, BUFSZ, eom_buf, strlen(eom_buf));
+        retval = read_string(&rs->rotport, buf, BUFSZ, eom_buf, strlen(eom_buf), 0, 1);
 
         if (retval < 0)
         {
